;----------------------------------------------------------------------------------------------------------------
; File:				paradigm.asm
; Purpose:			Demo and tests of some ASM paradigmns
; Author:			M Shaw
; Language:			Ticol TicASM under Ticol v1.26+
; Runs under:		Ticol Tcl ticol_asm.dll Plugin and asm.tcl
;
; Versions:
; 1.00	07/Nov/2018 First version
; ----------------------------------------------------------------------------------------------------------------
; Notes:
; FIX Consider redesign with MOVB opcode to move byte values
; ----------------------------------------------------------------------------------------------------------------
/*
Paradigms:

It is important to understand that this is a 'pseudo-assembler' and the instructions may not work
in precisely the same manner as a real CPU based assembler. The ASM paradigm leads to a fairly 
simple and concise set of instructions

Move integer literal to register:

	mov ax, 12345

Move integer variable to register:

	; Integer is in 'intval'
	mov ax, intval

Move register to integer

	; Integer is in 'intval'
	mov intval, ax

Move double to register
	; Double must be stored in either string or a register
	; Double, here, is in: 'doubleval'
	mov ax, 2.3		; Move literal

Move integer to memory location:

	; Integer is in 'intval'
	mov ax, intval
	mov [var], ax

Move double storage variable  to register
	mov ax, [doubleval]

Move literal double to memory location
	; Location is doubleval
	mov [doubleval], 2.0

Dereference single-byte
	; This is done via register dereference
	mov ax, str
	mov bx, [ax]		; BX now contains 1st char of str

Copy string to memory location

	; String is in 'str' and indexed by address 'str'
	mov [var], str

Increment integer

	; Integer is in 'intval'
	mov ax, intval
	inc ax
	mov intval, ax
*/
; ----------------------------------------------------------------------------------------------------------------

	intval:						; Declare variables
	str_intval:
	str_dblval:
	str_buf:
	msg_fail:
	msg_pass:
	tmp:
	str_foo:
	str_moo:

	mov str_intval, edp			; Define variables
	db 25, ""
	mov str_dblval, edp
	db 25, ""
	mov msg_fail, edp
	db ? "* Test failed at line: "
	mov msg_pass, edp
	db ? "* All tests pass"
	mov str_foo, edp
	db ?, "Foo"
	mov str_moo, edp
	db ?, "Moo"
	mov str_buf, edp
	db 256, ""
	
	mov intval, 12345			; Set literal integer to variable

	; -- RUN --
	; Move integer literal to register
	mov dx, __LINE__			; Save source line number
	mov ax, intval				; Set integer variable
	cmp ax, 12345
	jnz fail

	; Move integer variable to register
	mov dx, __LINE__			; Save source line number
	mov cx, 456					; Set register to literal integer
	mov intval, cx				; Set integer to register integer
	mov ax, intval				; Test using round trip to register
	cmp ax, 456
	jnz fail

	; Move integer to variable memory location
	; Move integer variable from memory location to register
	mov dx, __LINE__			; Save source line number
	mov [str_intval], 12345		; Copy literal to memory
	mov ax, [str_intval]		; Copy back from memory to register
	cmp ax, 12345
	jnz fail

	; Increment an integer
	; Integer is in 'intval'
	mov dx, __LINE__			; Save source line number
	mov intval, 98765
	mov cx, intval
	inc cx
	mov intval, cx				; Round trip via registers
	mov bx, intval
	cmp bx, 98766
	jnz fail

	; Integer math
	mov dx, __LINE__			; Save source line number
	mov intval, 98765
	mov cx, intval
	add cx, 10					; ADD
	sub cx, 20					; SUB
	add cx, 10					; ADD
	mul cx, 2
	div cx, 2
	mov intval, cx				; Round trip via registers
	mov bx, intval
	cmp bx, 98765				; We should end up with original value
	jnz fail

	; Move double literal to register
	mov dx, __LINE__			; Save source line number
	mov ax, 123.456				; Store literal to reg AX
	cmp ax, 123.456
	jnz fail

	; Move double literal to variable memory
	; Move double variable in memory to register
	; Doubles can be held only in memory location or a register
	; Simple (non-memory) variables can hold only an integer value or integer address
	mov dx, __LINE__			; Save source line number
	mov [str_dblval], 123.456	; Store literal to memory 
	mov ax, [str_dblval]		; Retrieve double from memory
	cmp ax, 123.456
	jnz fail

	; Copy double from one memory variable to another using a temporary
	; Using value from previous test
	mov dx, __LINE__			; Save source line number
	mov [str_buf], str_dblval	; Copy contents by address index
	mov ax, [str_buf]			; Copy memory contents to register
	cmp ax, 123.456
	jnz fail

	; Copy double from one memory variable to another using a temporary 'tmp'
	; Using value from previous test
	mov dx, __LINE__			; Save source line number
	mov tmp, str_dblval			; Index tmp to variable as an integer address
	mov [str_buf], tmp			; Copy contents
	mov ax, [str_buf]			; Copy memory contents to register
	cmp ax, 123.456
	jnz fail

	; MOV [VAR], VAR
	; Copy string or BCD number from source location to address location
	; MOV VAR, VAR will simply copy addresses as pointers
	mov dx, __LINE__			; Save source line number
	;mov str_moo, -1			; Simulate error condition
	mov [str_buf], str_foo 		; Copy string 'moo' to buf
	push str_buf				; Push compare string
	push str_foo
	call strcmp					; "C" strcmp
	cmp ax, 0					; Check if equal
	jnz fail

	; MOV VAR, [VAR]
	; Copy string or BCD number from source location to address location
	; MOV VAR, VAR will simply copy addresses as pointers
	mov dx, __LINE__			; Save source line number
	;mov str_moo, -1			; Simulate error condition
	mov str_buf, [str_moo]		; Copy string 'moo' to buf
	push str_moo				; Push compare string
	push str_buf
	call strcmp					; "C" strcmp
	cmp ax, 0					; Check if equal
	jnz fail

	; Index and print variable in memory
	mov ax, str_dblval			; Index AX to the string
	call 9						; Call print string
	mov ax, 10					; Line Feed Char
	call 2						; Call print char

	; Copy single byte to string in memory via REG
	; Dereferencing via REG will handle single-bytes
	mov dx, __LINE__			; Save source line number
	mov ax, str_foo				; Index AX into string 'foo'
	mov [ax], 77				; Replace letter M (ASCII 77)
	push str_moo				; Push compare string
	push ax						; Push main string
	call strcmp					; "C" strcmp
	cmp ax, 0					; Check if equal
	jnz fail

	; Retrieve single byte from string in memory via REG
	; Dereferencing a REG will handle single-bytes
	; Note: cmp [ax], N is not implemented. We use intermediate BX
	mov dx, __LINE__			; Save source line number
	mov ax, str_foo				; Index AX into string 'foo'
	inc ax						; Move to 2nd character
	mov bx, [ax]				; Retrieve a single character
	cmp bx, 111					; Lower case 'o' (ASCII 111)
	jnz fail

	; -- SUCCESS --
	mov ax, 10					; Green
	call 3						; Call set text colour
	mov ax, msg_pass			; Index into success message
	call 9						; Call print string
	mov ax, 10					; Line feed char
	call 2						; Call print char
	mov ax, msg_pass			; Return to Tcl
	jmp end						; Done

fail:
	dump						; Display memory and registers
	mov ax, msg_fail			; Index into failure message string
	call 9						; Print
	mov ax, dx					; Index into DX value holding line number
	call 8						; Call print literal number
	mov ax, 10					; LF
	call 2						; Call print character
	xor ax, ax					; Clear AX
	jmp end						; Done

end:
	end							; Exit script

; EOF ;