########################################################################################
# File:				test_2d_carray.tcl
# Purpose:			2 dimensional indexing of a 1 dimensional carray space
# See also:			test_3d_carray.tcl
# Language:			Ticol Tcl v1.26+
# Author:			M Shaw
#
# Versions:
# 1.00 17/Dec/2021 First version
# 1.01 07/Jun/2022	Revised for changes to [loop] bounds
# 1.02 28/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
########################################################################################
# Notes:
# Demonstrates using a proc to perform indexing arithmetic to handle a 1-dimensional
# carray as a multi-dimensional (N-dimensional) one
# Multiple dimensions means further multiplication in addition to the Y axis Y Z etc.
# A future release of ticol_carray may include the basic procs below
#
# FIX Check for memory leaks on loop reassign test
# FIX carray set can't 'add' only change existing elemetns
########################################################################################

option expression off

textcolor cyan
lib ticol_carray -unload -nocomplain
lib ticol_carray -verbose -nocomplain
textcolor
newline

# row-major order (x==row)
# x 10, y 5 == 10 x 5 matrix
set dim_x 5		# Major row
set dim_y 3
/* 5 x 3 Layout (15)
+-----+
|XXXXX|
|XXXXX|
|XXXXX|
+-----+
3D example: 5 x 3 x 2 (30)
  _______
 /      /|
+-----+ x|
|XXXXX|xx|
|XXXXX|xx/
|XXXXX|x/
+-----+/
################################################################# */
textcolor yellow
puts "Array size is ${dim_x}x * ${dim_y}y (15)"
textcolor
newline

# FIX improve args
# New version allows x,y,z values to be defined for 2D and 3D dims
set h [carray create $dim_x $dim_y]		# FIX does not fill
#set h [carray create [* $dim_x $dim_y] $dim_x $dim_y]		# FIX does not fill
puts "carray size is [carray count $h]"

# Make re-entrant in the CLI
undef ix_get -nocomplain
undef ix_get_rvalue -nocomplain
undef ix_get_tuple -nocomplain
undef ix_set -nocomplain

# Example procs. Now built-in to ticol_carray.dll
proc ix_set {handle x y value {rvalue}} {
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y)"
		return
	}
	carray set $handle  [+ [* $y $::dim_x] $x] $value $rvalue		2D
}

proc ix_get {handle x y} {
	# Return Lvalue only
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y)"
		return
	}
	return [carray get $handle [+ [* $y $::dim_x] $x]]
}

proc ix_get_rvalue {handle x y} {
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get_rvalue x > $::dim_x accessing element($x,$y)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get_rvalue y > $::dim_y accessing element($x,$y)"
		return
	}
	return [carray get $handle [+ [* $y $::dim_x] $x] -rvalue]
}

proc ix_get_tuple {handle x y} {
	# Return Lvalue and Rvalue pair
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y)"
		return
	}
	return [list [carray get $handle [+ [* $y $::dim_x] $x]]  [carray get $handle [+ [* $y $::dim_x] $x] -rvalue]  ]
}

# Index values are array base 0
# ** FIRST ROW **
ix_set $h 0 0 "0,0" First
ix_set $h 1 0 "1,0" Second
#ix_set $h 2 0 "2,0"
#ix_set $h 3 0 "3,0"
#ix_set $h 4 0 "4,0"

# ** SECOND ROW **
ix_set $h 0 1 "0,1" Sixth
ix_set $h 1 1 "1,1" Seventh
#ix_set $h 1 1 "1,1"

# ** THIRD ROW **
ix_set $h 0 2 "0,2"
ix_set $h 1 2 "1,2"
ix_set $h 2 2 "2,2"
ix_set $h 3 2 "3,2"
ix_set $h 4 2 "4,2" "Omega"

catch {ix_set $h 4 3 "4,3"}	# error
# */
#ix_set $h 0 3 "0,1"
#ix_set $h 0 4 "0,1"
#ix_set $h 1 1 "1,1"
#ix_set $h 2 1 "1,1"
#ix_set $h 4 2 "1,1"
#ix_set $h 5 5 "5,5"
#ix_set $h 1 3 "1,3"
#ix_set $h 9 4 "10-5"

carray walk $h
newline

#################################################################
# Read the array as a 2-dimensional one
#################################################################
textcolor magenta
puts "1 0 -> '[ix_get_tuple $h 1 0]'"
puts "2 0 -> '[ix_get $h 2 0]'"
puts "2 2 -> '[ix_get $h 2 2]'"
puts "4 2 -> '[ix_get_tuple $h 4 2]'"
textcolor

#################################################################
# Re-set using a nested loop
#################################################################
lib ticol_numword -nocomplain
#puts "Testing local proc version of ix_set ..."
puts "Testing lib version of carray set_2d ..."
loop j 0 3 { 
	loop i 0 5 {
		# set handle x y lvalue rvalue
		# Local proc version
		#ix_set $h $i $j ($i,$j) [integer_to_words [+ [* $j $::dim_x] $i]]
		# Built-in lib version
		carray set_2d $h $i $j ($i,$j) [integer_to_words [+ [* $j $::dim_x] $i]]
	}
}
newline
carray walk $h
newline

# Built-in lib version
textcolor yellow
puts "Testing lib version of carray get_2d ..."
loop j 0 3 { 
	loop i 0 5 {
		#puts "carray get_2d->'[carray get_2d $h $i $j]'"
		#puts "carray get_2d->'[carray get_2d $h $i $j -rvalue]'"
		puts "carray get_2d->'[carray get_2d $h $i $j -tuple]'"
	}
}
textcolor

#################################################################
# Print selected item(s)
#################################################################
textcolor magenta
puts "Item 4,1 is '[ix_get $h 4 1]', rvalue='[ix_get_rvalue $h 4 1]'"
textcolor

# Built-in lib version
assert [carray get_2d $h 0 0] {eq $_ "(0,0)"} -v -line #__LINE__
assert [carray get_2d $h 1 1] {eq $_ "(1,1)"} -v -line #__LINE__
assert [carray get_2d $h 4 1] {eq $_ "(4,1)"} -v -line #__LINE__
assert [carray get_2d $h 4 2] {eq $_ "(4,2)"} -v -line #__LINE__

/*
# Local proc version
assert [ix_get $h 0 0] {eq $_ "(0,0)"} -v -line #__LINE__
assert [ix_get $h 1 1] {eq $_ "(1,1)"} -v -line #__LINE__
assert [ix_get $h 4 1] {eq $_ "(4,1)"} -v -line #__LINE__
assert [ix_get $h 4 2] {eq $_ "(4,2)"} -v -line #__LINE__
# */


/* Trace
 0 '(0,0)' 'zero'
 1 '(1,0)' 'one'
 2 '(2,0)' 'two'
 3 '(3,0)' 'three'
 4 '(4,0)' 'four'
 5 '(0,1)' 'five'
 6 '(1,1)' 'six' 7 '(2,1)' 'seven'
 8 '(3,1)' 'eight'
 9 '(4,1)' 'nine'
10 '(0,2)' 'ten'
11 '(1,2)' 'eleven'
12 '(2,2)' 'twelve'
13 '(3,2)' 'thirteen'
14 '(4,2)' 'fourteen' */

carray unset $h
unset h i j dim_x dim_y -nocomplain
lib ticol_carray -unload
lib ticol_numword -unload

vars
newline 

textcolor green
puts "* Done *"
textcolor

# EOF #
########################################################################################