########################################################################################
# File:				test_3d_carray.tcl
# Purpose:			3 dimensional indexing of a 1 dimensional carray space
# See also:			test_2D_carray.tcl
# Language:			Ticol Tcl v1.26+
# Author:			M Shaw
#
# Versions:
# 1.00 17/Dec/2021	First version
# 1.01 07/Jun/2022	Revised for changes to [loop] bounds
# 1.02 28/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
########################################################################################
# Notes:
# A future release of ticol_carray may include the basic procs below
# Demonstrates using a proc to perform indexing arithmetic to handle a 1-dimensional
# carray as a multi-dimensional (N-dimensional) one
# Multiple dimensions means further multiplication in addition to the Y axis Y Z etc.
# Additional dimensions should be added from left to right -> (x, y, z) etc.
#
# FIX Check for memory leaks on loop reassign test
# FIX carray set can't 'add' only change existing elements
########################################################################################

option expression off

textcolor cyan
lib ticol_carray -unload -nocomplain
lib ticol_carray -verbose -nocomplain
textcolor
newline

# row-major order (x==row)
# x 10, y 5 == 10 x 5 matrix
set dim_x 5		# Major row
set dim_y 3
set dim_z 2
set dim_xy [* $::dim_x $::dim_y]	# Math shortcut

/* 
	3 (1D layout)
	+-+
	|x|
	|x|
	|x|
	+-+

	5 x 3 (2D Layout)
	+-----+  
	|XXXXX| 12345
	|XXXXX| 6789A
	|XXXXX| BCDEF
	+-----+
	5 x 3 x 2 = 30 (3D Layout)
	  _______
	 /      /|
	+-----+ x|
	|XXXXX|xx|
	|XXXXX|xx/
	|XXXXX|x/
	+-----+/
################################################################# */
textcolor yellow
puts "Array size is ${dim_x}x * ${dim_y}y * ${dim_z}z ([* $dim_z $dim_xy])"
textcolor

set h [carray create $dim_x $dim_y $dim_z]	# New code, 2021
#set h [carray create [* [* $dim_x $dim_y] $dim_z]]
puts "Allocated carray size is [carray count $h]"

# Allow CLI re-entrancy
undef ix_get -nocomplain
undef ix_get_rvalue -nocomplain
undef ix_get_tuple -nocomplain
undef ix_set -nocomplain

proc ix_set {handle x y z value {rvalue}} {
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y,$z)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y,$z)"
		return
	}
	if {>= $z $::dim_z} {
		error "Carray bounds error: ix_get z > $::dim_z accessing element($x,$y,$z)"
		return
	}
	carray set $handle  [+ [+ [* $y $::dim_x] [* $z $::dim_xy]] $x] $value $rvalue
}

proc ix_get {handle x y z} {
	# Return Lvalue only
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y,$z)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y,$z)"
		return
	}
	if {>= $z $::dim_z} {
		error "Carray bounds error: ix_get z > $::dim_z accessing element($x,$y,$z)"
		return
	}
	#puts "ix_get returning"
	return [carray get $handle [+ [+ [* $y $::dim_x] [* $z $::dim_xy]] $x]]
}

proc ix_get_rvalue {handle x y z} {
	# Return Rvalue only
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get_rvalue x > $::dim_x accessing element($x,$y,$z)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get_rvalue y > $::dim_y accessing element($x,$y,$z)"
		return
	}
	if {>= $z $::dim_z} {
		error "Carray bounds error: ix_get_rvalue z > $::dim_z accessing element($x,$y,$z)"
		return
	}
	return [carray get $handle [+ [+ [* $y $::dim_x] [* $z $::dim_xy]] $x] -rvalue]
}

proc ix_get_tuple {handle x y z} {
	# Return Lvalue and Rvalue pair
	# [in] x and y should be array-base 0
	if {>= $x $::dim_x} {
		error "Carray bounds error: ix_get x > $::dim_x accessing element($x,$y,$z)"
		return
	}
	if {>= $y $::dim_y} {
		error "Carray bounds error: ix_get y > $::dim_y accessing element($x,$y,$z)"
		return
	}
	if {>= $z $::dim_z} {
		error "Carray bounds error: ix_get z > $::dim_z accessing element($x,$y,$z)"
		return
	}
	set offset [+ [+ [* $y $::dim_x] [* $z $::dim_xy]] $x]
	return [list [carray get $handle $offset\
		[carray get $handle $offset -rvalue]  ]]	# [+ [+ [* $y $::dim_x] [* $z $::dim_xy]] $x]
}


#################################################################
# Re-set using a nested loop
#################################################################
# Note changes to [loop] boundings 2022
lib ticol_numword -nocomplain
set ck 0
loop k 0 $dim_z {
# loop k 0 [- $dim_z 1] {
	loop j 0 $dim_y { 
	# loop j 0 [- $dim_y 1] { 
		loop i 0 $dim_x {
		# loop i 0 [- $dim_x 1] {
			#puts "Assigning ($i,$j,$k)"
			ix_set $h $i $j $k ($i,$j,$k) [integer_to_words [+ [+ [* $j $::dim_x] [* $k $dim_xy]] $i]]
			++ ck
		}
	}

}
puts "Assignent iters checksum $ck"
newline
carray walk $h
newline

#################################################################
# Print selected item(s)
#################################################################
textcolor magenta
puts "Item 0,0,0 is '[ix_get $h 0 0 0]', rvalue='[ix_get_rvalue $h 0 0 0]'"	# 0
puts "Item 1,1,0 is '[ix_get $h 1 1 0]', rvalue='[ix_get_rvalue $h 1 1 0]'"	# 6
puts "Item 3,2,0 is '[ix_get $h 3 2 0]', rvalue='[ix_get_rvalue $h 3 2 0]'"	# 13
puts "Item 1,1,1 is '[ix_get $h 1 1 1]', rvalue='[ix_get_rvalue $h 1 1 1]'"	# 21
puts "Item 4,2,1 is '[ix_get $h 4 2 1]', rvalue='[ix_get_rvalue $h 4 2 1]'"	# 29
textcolor

#catch {puts [ix_get $h 4 2 2]}	# Error
#puts [ix_get $h 4 2 2]			# Error

try {
	ix_get $h 4 2 2
} catch {
	puts "try-catch safely caught a bounds error"
	newline
}

#################################################################
# A few tests...
#################################################################
assert [ix_get $h 0 0 0] {eq $_ "(0,0,0)"} -v -line #__LINE__
assert [ix_get_rvalue $h 0 0 0] {eq $_ "zero"} -v -line #__LINE__
assert [ix_get $h 1 1 0] {eq $_ "(1,1,0)"} -v -line #__LINE__
assert [ix_get_rvalue $h 1 1 0] {eq $_ "six"} -v -line #__LINE__
assert [ix_get $h 3 2 0] {eq $_ "(3,2,0)"} -v -line #__LINE__
assert [ix_get_rvalue $h 3 2 0] {eq $_ "thirteen"} -v -line #__LINE__
assert [ix_get $h 1 1 1] {eq $_ "(1,1,1)"} -v -line #__LINE__
assert [ix_get_rvalue $h 1 1 1] {eq $_ "twenty one"} -v -line #__LINE__
assert [ix_get $h 4 2 1] {eq $_ "(4,2,1)"} -v -line #__LINE__
assert [ix_get_rvalue $h 4 2 1] {eq $_ "twenty nine"} -v -line #__LINE__


textcolor
newline

########################################################################################

carray unset $h
unset ck dim_x dim_y dim_z dim_xy h i j k  -nocomplain

undef ix_get -nocomplain
undef ix_get_rvalue -nocomplain
undef ix_get_tuple -nocomplain
undef ix_set -nocomplain

lib ticol_carray -unload
lib ticol_numword -unload

vars
newline 

textcolor green
puts "* Done *"
textcolor
########################################################################################