#######################################################################################
# File: 			test_alias.tcl
# Purpose:			Test [alias] and proc+command orphaning after [undef]
# Language:			Ticol Tcl v1.27
#
# Versions:
# 1.00 21/Oct/2019	First version using [assert]
# 1.01 06/Jan/2023	Added -nocomplain cases
#######################################################################################
# Notes:
# ADD -nocomplain tests
# DONE We need to ensure that, particularly for [proc] that any aliases do not trigger
# the destruction of the master proc copy. The proc code block should be deleted only
# when the last valid alias is being deleted
#######################################################################################
option expression off
option echo on

newline
textcolor yellow
puts "Testing \[alias\] ..."
textcolor
newline

undef bar foo -nocomplain
undef unknown -nocomplain	# We *MUST* undef [unknown] if we intend to run this script a 2nd time from tests.tcl in the CLI or IDE

set i 0
proc foo {} {
	textcolor magenta
	puts "In proc foo"
	textcolor
	set ::i 1
}

alias bar foo

assert [defined foo] {== $_ 1} -v -line #__LINE__
assert [defined bar] {== $_ 1} -v -line #__LINE__

set i 0
foo 
assert $i {== $_ 1} -v -line #__LINE__

set i 0
bar
assert $i {== $_ 1} -v -line #__LINE__

######################################################################################
newline
textcolor yellow
puts "Testing safe alias deletion without leaving orphaned proc ..."
puts "Undefining parent proc 'foo' ..."
textcolor
newline

puts "'[procs]' @"#__LINE__
undef foo
puts "'[procs]' @"#__LINE__
assert [defined foo] {== $_ 0} -v -line #__LINE__
# FIX bar is defined but the proc it points to no longer exists
assert [defined bar] {== $_ 1} -v -line #__LINE__

newline
textcolor yellow
puts "Calling alias 'bar' after \[undef foo\] ..."
textcolor
newline

set i 0
bar
assert $i {== $_ 1} -v -line #__LINE__

undef bar
assert [defined foo] {== $_ 0} -v -line #__LINE__
assert [defined bar] {== $_ 0} -v -line #__LINE__


# This line fails in WinTicol if tests.tcl is run a 2nd time without exiting
# This is because state is preserved between calls as with the Ticol CLI
# and currently [clearprocs] is not called before running
option autoexec off
set i 0
trace add bar
try {
	puts "Attempting to call proc bar after undef"
	bar
	puts "After attempting to call bar (we should not be here)"
} catch {
	puts "Attempt to call proc bar failed"
	set i 1
}
if {== $i 0} {
	puts "procs b* '[procs b*]'"
	puts "Checking for presence of proc 'unknown' -> '[procs un*]'"
}
trace remove bar
assert $i {== $_ 1} -v -line #__LINE__

######################################################################################
# We never destroy the compiled-in command (unless it is an unloaded [lib])
newline
textcolor yellow
puts "Test alias and undef command, checking for orphans ..."
textcolor
newline

# Create an alias
alias tim timer
assert [defined timer] {== $_ 1} -v -line #__LINE__
assert [defined tim] {== $_ 1} -v -line #__LINE__

# Call the alias
assert [tim] {> $_ 0} -v -line #__LINE__

# Undefine the parent
undef timer 
assert [defined timer] {== $_ 0} -v -line #__LINE__
assert [defined tim] {== $_ 1} -v -line #__LINE__
assert [tim] {> $_ 0} -v -line #__LINE__

# Rename the alias as the old parent
rename tim timer
assert [defined timer] {== $_ 1} -v -line #__LINE__
assert [defined tim] {== $_ 0} -v -line #__LINE__
assert [timer] {> $_ 0} -v -line #__LINE__

######################################################################################
# Alias to a [lib] command
newline
textcolor yellow
puts "Test unloading a \[lib\] with command aliases ..."
puts "Current dir is [getcwd]"
textcolor
newline

textcolor cyan
lib ticol_r2d -verbose -nocomplain
textcolor
newline

undef foo -nocomplain
alias foo roman_to_decimal
assert [defined foo] {== $_ 1} -v -line #__LINE__
assert [defined roman_to_decimal] {== $_ 1} -v -line #__LINE__
assert [foo MDCCLXXVI] {== $_ 1776} -v -line #__LINE__
undef roman_to_decimal
assert [defined roman_to_decimal] {== $_ 0} -v -line #__LINE__
assert [foo MDCCLXXVI] {== $_ 1776} -v -line #__LINE__
set i 0
try {
	roman_to_decimal IV
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [foo MDCCLXXVI] {== $_ 1776} -v -line #__LINE__

textcolor yellow
puts "Unloading ticol_r2d @"#__LINE__
textcolor
textcolor cyan

# FIX ticol_r2d: Module (null) already loaded
# ADD [aliases commandname]
lib ticol_r2d -unload -verbose		# This should trigger unload of the [foo] alias
textcolor

# [foo] should no longer exist
assert [defined foo] {== $_ $::false} -v -line #__LINE__
set i 0
try {
	foo IV
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

####################################################################################################

undef foo -nocomplain
undef bar -nocomplain

# Check assign non-existent alias source 'bar'
# alias newname existingname
puts "Line "#__LINE__
set i 0
set r 1
try {
	set r [alias foo bar]		# Should return 0 but set won't be called due to error being raised
} catch {
	set i 1
}
assert [defined foo] {== $_ $::false} -v -line #__LINE__	#
assert $i {== $_ 1} -v -line #__LINE__	#
# assert $r {== $_ 0} -v -line #__LINE__	#

# Check assign non-existent alias source 'bar' with -nocomplain
assert [defined bar] {== $_ $::false} -v -line #__LINE__	#
set i 0
set r 1
try {
	set r [alias foo bar -nocomplain]		# r will be set
} catch {
	set i 1
}
assert [defined foo] {== $_ $::false} -v -line #__LINE__	#
assert $i {== $_ 0} -v -line #__LINE__	#
assert $r {== $_ 0} -v -line #__LINE__	#

proc bar {} {}
alias foo bar		# Create a valid alias

# Test trying to re-create a current alias foo->bar
set i 0
try {
	set r [alias foo bar]		# Set won't be called due to error being raised
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__	#

# Test trying to re-create a current alias  foo->bar with -nocomplain
set i 0
set r 1
try {
	set r [alias foo bar -nocomplain]		#
} catch {
	set i 1
}
assert $i {== $_ 0} -v -line #__LINE__	# */
assert $r {== $_ 0} -v -line #__LINE__	#

undef bar -nocomplain
undef foo -nocomplain
unset i -nocomplain

#######################################################################################
option expression off
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #