############################################################################################
# File:				test_array_find_seq.tcl
# Author:			M Shaw
# Purpose:			Test using [array find N] to find the Nth sequential matching item
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Aug/2019 First version
###########################################################################################
# Notes:
# Reminder that arrays are not stored in sorted/ordred sequence so finding means iterating
# through all items which are effectively stored in random order
# Thus results might occasionally appear inconsistent between find with case and nocase
###########################################################################################

option expression off

array unset a -nocomplain

/*
set s [readfile ticol.man]
set lines [string toarray $s a]
puts $lines
# */

array set a {			# We use a power of 2 integer index so we can generate a valid checksum ID
	1 Hello
	2 there
	4 fox				# 1 (4)
	8 how 
	16 are
	32 fox				# 2 (32)
	64 you
	128 The
	256 quick
	512 brown
	1024 fox			# 3 (1024)
	2048 jumps
	4096 over
	8192 the
	16384 lazy
	32768 fox			# 4 (32768)
	65536 end
	131072 FOX			# 5 (131072)		# Differnt case
}

set findstr fox

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

#/*

textcolor yellow
puts "Case significant comparison ..."
textcolor
puts "0->'[array find a $findstr 0]'"
puts "1->'[array find a $findstr 1]'"
puts "2->'[array find a $findstr 2]'"
puts "3->'[array find a $findstr 3]'"
puts "4->'[array find a $findstr 4]'"
puts "5->'[array find a $findstr 5]'"
puts "5->'[array find a $findstr 5 -nocase]'"
puts "1->'[array find a $findstr 1 -nocase]'"
puts "6->'[array find a $findstr 6]'"
newline
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

puts "Line: "#__LINE__
textcolor yellow
puts "No case comparison ..."
textcolor

puts "NC 1->'[array find a $findstr 1 -nocase]'"
puts "NC 2->'[array find a $findstr 2 -nocase]'"
puts "NC 3->'[array find a $findstr 3 -nocase]'"
puts "NC 4->'[array find a $findstr 4 -nocase]'"
puts "NC 5->'[array find a $findstr 5 -nocase]'"
puts "NC 6->'[array find a $findstr 6 -nocase]'"
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

puts "Line: "#__LINE__
textcolor yellow
puts "Searching in iterative loop to find all matching cases"
textcolor
#set found [array find a $findstr]
set found ""
set sequence 1			
set total 0				# Checksum for [assert]

do {													# Case-sensitive search
	set found [array find a $findstr $sequence]
	+= total $found										# Assert checksum
	puts "'$found'\t'[array item a $found {<not found>}]'"
	++ sequence
} while {ne $found ""} 

puts "\nSearch checksum total is $total"
assert $total {== $_ 33828} -v -line #__LINE__
newline
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

set found ""
set sequence 1			
set total 0				# Checksum for [assert]

do {													# Case-sensitive search
	set found [array find a $findstr $sequence -nocase]
	+= total $found										# Assert checksum
	printf "'%7i'\t'[array item a $found {<not found>}]'\n" $found
	++ sequence
} while {ne $found ""} 

puts "\nSearch checksum total is $total"
assert $total {== $_ 164900} -v -line #__LINE__

newline
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

set findstr foo
set found ""
set sequence 1			
set total 0				# Checksum for [assert]

do {													# Case-sensitive search
	set found [array find a $findstr $sequence -nocase]
	+= total $found										# Assert checksum
	printf "'%7i'\t'[array item a $found {<not found>}]'\n" $found
	++ sequence
} while {ne $found ""} 

puts "\nSearch checksum total is $total"
assert $total {== $_ 0} -v -line #__LINE__

newline
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
array unset a -nocomplain
set findstr ".ticol"
set found ""
set sequence 1			
set total 0				# Checksum for [assert]

puts "Reading ticol.man ..."
set start_time [timer]
readfile ticol.man a -array
puts "Read took [- [timer] $start_time] ms"				# 376 ms

puts "Searching for $findstr, 6  ...\n"

set start_time [timer]
do {													# Case-sensitive search
	set found [array find a $findstr $sequence -nocase -length 6]
	if {ne $found ""} {
		++ total 
		printf "%7i  $a($found) sequence: $sequence\n" $found
	}
	++ sequence
} while {ne $found ""} 

puts "Find took [- [timer] $start_time] ms"				# 130 ms
puts "\nTotal instances of $findstr is $total"
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
array unset a
unset total found sequence findstr

vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #