###################################################################################
# File: 			test_array_of_array.tcl
# Purpose:			Test arrays of arrays
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
# See also:			test_double_deref_adt, test_dereference
#
# Versions:
# 1.00 10/Sep/2019	First version
# 1.01 26/Oct/2019	Updated linear iterator and cleanup
# 1.02 13/Feb/2022	Fixed for modifications to [static] functionality
# 1.03 07/Jun/2022	Revised for changes to [loop] bounds
# 1.04 27/Dec/2022	Revised again for reversion to [loop] bounds
###################################################################################
# Notes:
# Member of tests.tcl
# Iterate through two dimensional arrays (arrays of true arrays)
#
# DONE FIX This could be extended to N dimensional if we test the array element 
# payload against "<array>" and if this is the case then iter for this array name
# A recursive solution would probably be required which would call itself for
# each instance of an array element being fount to be <array> and then 
# iter that array by name
###################################################################################
option echo on

## Create the test array ##########################################################
/*
                  a					# Parent
                 / \
                b   c				# Child
               /\    \
                 d					# Grandchild
                 |
*/
###################################################################################

# Init array a
set a(0) b							# Point child array a to child array b by name
set a(1) c							# Point child array a to child array c by name

# Init some non array elements
set a(2) "a.Hello from a(2)"
set a(3) "a.Hello from a(3)"

# Init child array b
set b(0) $qbf						# A non-array value from variable
set b(1) "b element 1"
set b(2) "b element 2"
set b(3) d							# Point child array b to grandchild array d by name

# Init child array c (make size different to b)
set c(0) "Hello world"
set c(1) "c element 1"
set c(2) "c element 2"
set c(3) "c element 3"
set c(4) "c element 4"

set d(0) "d element 0"
set d(1) "d element 1"

# 12 leaf nodes, 3 array child nodes

###################################################################################
# Some asserts
###################################################################################

assert ${$a(0)} {eq $_ "<array>"} -v -line #__LINE__

assert ${$a(0)(0)} {eq $_ $::qbf} -v -line #__LINE__
assert ${$a(0)(1)} {eq $_ "b element 1"} -v -line #__LINE__

assert ${$a(1)(0)} {eq $_ "Hello world"} -v -line #__LINE__
assert ${$a(1)(1)} {eq $_ "c element 1"} -v -line #__LINE__

assert $a(0) {eq $_ b} -v -line #__LINE__

#########################################################################################################
# Non-recursive printout
# Non recursive referencing solutions can quicky get very unweidly and recursive ones are recommended!
# Linear solutions might appeal to mathematicians but are likely to make non-mathematician's
# brains hurt badly after 2 or more levels of nesting!
#########################################################################################################
puts "Line: "#__LINE__

textcolor yellow
puts "Linear listing array of array (parent a, children b, c and d)"
textcolor
newline

 loop i 0 [array size a] {			# Mods 27/Dec/2022
# loop i 0 [- [array size a] 1] {
	if {is_array $a($i)} {
		loop j 0 [array size $a($i)] {
		# loop j 0 [- [array size $a($i)] 1] {
			if {is_array ${$a($i)($j)}} {					# 'd'
				newline
				loop k 0 [array size ${$a($i)($j)}] {
				# loop k 0 [- [array size ${$a($i)($j)}] 1] {
					#puts "${$a($i)($j)}($k) == ${${$a($i)($j)}($k)}"			# OK
					puts "a(b(${$a($i)($j)}($k))) -> ${${$a($i)($j)}($k)}"
					# Explanation...
					#                                ${             ($k)}		# Outer/final array dereference for d
					#                                  ${$a($i)($j)}			# Inner reference == d
					#                                  ${      ($j)}			# Dereference for b
					#                                    $a($i)                 # Outer reference == b
					#puts " -> '${$a($i)($a($j))c($k)}'"			# a(b(3)) a(b(c(x)))
				}
			} else {
				puts "a($a($i)($j)) -> '${$a($i)($j)}'"
			}
		}
		newline
	} else {
		puts "a($i) -> '$a($i)'"
	}
}

###################################################################################
# Recursive proc to print an array of any depth of an integer indexed array
# An alternate could be non-integer indexed using [foreach]
# Extra points for iterating the subtree nodes before the leaf nodes
# FIX how can we find the name of an unknown array var?
# 	We can just get <array>
###################################################################################

set trace_printed_nodes 0						# For debugging assert only

proc print_array_recursive {_a} {
	textcolor magenta
	puts "Enter print_array_recursive @"#__LINE__
	textcolor
	static indent_level 0
	puts "Line: "#__LINE__
	+= indent_level 2
	upvar $_a a									# Local var is always called 'a' but _a has true varname
	puts "Line: "#__LINE__
	#puts "ENTER print_array '$a' '[is_array a]' '$_a'" # [array name $_a]'"
	if {is_array a} {
		#puts "Debug [array size a]"
		loop i 0 [array size a] {
		# loop i 0 [- [array size a] 1] {
			#puts "Debug: is_array $a($i) -> '[bool [is_array ::$a($i)]]'"
			if {is_array ::$a($i)} {
				puts [string repeat " " $indent_level] -nonewline	# Pretty indenting
				textcolor white
				puts "${_a}($i) -> '$a($i)'"
				textcolor
				#puts "Debug: call print_array recursively for $a($i)"
				print_array_recursive ::$a($i)
			} else {
				puts [string repeat " " $indent_level] -nonewline	# Pretty indenting
				puts "${_a}($i) -> '$a($i)'"
				++ ::trace_printed_nodes		# For debugging assert only
			}
		}
	} /* else {
		puts "** element -> '$a'"
	} # */
	-- indent_level
}

static indent_level -unset -nocomplain
assert [is_set indent_level] {== $_ $false} -v -line #__LINE__
vars

newline
puts "Line: "#__LINE__
textcolor yellow
puts "Calling print_array_recursive ..."
puts "Parent nodes are white, leaf nodes grey"
textcolor
newline

puts "Line: "#__LINE__
print_array_recursive a
puts "We printed $trace_printed_nodes node(s)"
newline
assert $trace_printed_nodes {== $_ 12} -v -line #__LINE__

###################################################################################
# Use this RECURSIVE method to clean up
# This could be improved upon to use relative rather than global scope
# Poss using [uplevel @N array unset var]
# But this code works well enough to demonstrate
###################################################################################

#puts [is_array b]
#puts [is_array b(0)]
#puts [uplevel @0 "is_array b(0)"]
#puts [uplevel @0 "is_array b"]

puts "Line: "#__LINE__
unset QUOTE -const -nocomplain
const QUOTE [chr 34]							# Sneaky method of embedding dquotes

proc delete_array_recursive {_q} {
	puts "ENTER proc delete_array_recursive _q -> '$_q'"
	upvar $_q q									# Local var is always called 'a' but _a has true varname
	if {is_array q} {
		loop i 0 [array size q] {
		# loop i 0 [- [array size q] 1] {				# Check each element for child arrays
			# FIX if array var resolves to a value with spaces then each verb may be used (incorrectly)
			# So set b(1) "b element 1" will -> [is_array b element 1] and ignore the remaining values
			# Catch 22 here. Can't brace as this would block proc-local var resolution
			# cannot have nested quotes so we must use [chr 34] to insert a quote character
			# We could also use a var
			#puts "Testing 'is_array '$q($i)'' a:[is_array ::$q($i)] b:"[uplevel @0 "is_array [chr 34]$q($i)[chr 34]"]
			if {[uplevel @0 "is_array $::QUOTE$q($i)$::QUOTE"]} {
			# if {[uplevel @0 "is_array [chr 34]$q($i)[chr 34]"]} {
				delete_array_recursive ::$q($i)
			}
		}
		textcolor darkyellow
		puts "Unsetting array $_q"
		textcolor
		uplevel @0 "array unset $_q"			# Use [uplevel 1 array unset]
	}
} 
delete_array_recursive a
# */

newline
textcolor yellow
puts "Recursive deletion of array and all array children ..."
textcolor
newline

delete_array_recursive ::a								# We must call with the root node using global scope (::)
 # */

###################################################################################
/*
# Simple solution: Use this ITERATIVE method to clean up (top level)
newline
textcolor yellow
puts "Iter array of array and delete all children ..."
textcolor
newline
puts "Line: "#__LINE__
loop i 0 [array size a] {
# loop i 0 [- [array size a] 1] {
	if {is_array $a($i)} {
		puts "Unsetting array $a($i)"
		array unset $a($i)
	}
} # */

###################################################################################
# Post delete tests and cleanup
###################################################################################

puts [vars -list]
newline

assert [array exists a] {== $_ $::false} -v -line #__LINE__
assert [array exists b] {== $_ $::false} -v -line #__LINE__
assert [array exists c] {== $_ $::false} -v -line #__LINE__
assert [array exists d] {== $_ $::false} -v -line #__LINE__

puts "Undefining print_array_recursive"
undef print_array_recursive
puts "Undefining delete_array_recursive"
undef delete_array_recursive

puts "Cleanup variables..."
vars
array unset a -nocomplain
unset i j k trace_printed_nodes
static print_array_recursive::indent_level -unset # Statics require special cleanup

###################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #