###############################################################################################
# File:				test_array_of_struct_method.tcl
# Author:			M Shaw
# Purpose:			Test 'method' calling syntax for elements in arrays of struct
# 					This is NOT a unit test (yet)
# Language:			Ticol Tcl v1.27+
#
# Version:
# 1.00 20/Dec/2022	First version
###############################################################################################
# Notes:
# This demonstrates calling methods as a pseudo-class instance via arrays of struct and using 
# a single array element only
#
# EXECUTION/CALLING
# The {*} operator operates only on well-formed Tcl lists, not Tcl script blocks
# Notes: In Tcl, dereferences which result in a command are executed
# This applies to single commands. Scripts require expansion (parsing) into a command array
# Dereferences which resolve into a single Tcl "word" will execute okay and directly via $
# and without needing expansion
# It is therefore useful for methods to be a simple reference to a proc or sub if direct
# dollar based command dereferences are required
###############################################################################################
option expression off

struct template {				# Our template struct used for instantiation
	a 100						# Needs to be large enough to store the demo script
	b 30						# Needs to be large enough to store a proc name reference
}

puts "Line "#__LINE__
# Instantiation
new template foo_1				# Duplicate template struct into a new one
new template foo_2				# Duplicate template struct into a new one

puts "Line "#__LINE__
# Script/method definition
set script {static i 1; textcolor magenta; puts "Hello $i"; textcolor; ++ i}
set test {puts "Hello from test"}
undef foo -nocomplain
sub foo $script					# Assign our scriptlet to a [sub] called 'foo'

puts "Line "#__LINE__
# Binding the array element to a struct instance
# This is a simple demo and we use only one array element
# array set a 0 foo_1			# Alternate method of setting (1)
set a(0) foo_1					# Alternate method of setting (2)
array walk a					# Confirmation of object link via array walk
puts $a(0)						# Visual confirmation of object link
assert $a(0) {eq $_ foo_1} -v -line #__LINE__

puts "Line "#__LINE__
# Assigning member functionality
struct set ${a(0)}.a $script	# Assign a scriptlet as a "method" (not recommended)
struct set ${a(0)}.b foo		# Point to a sub/proc aka a "method" (single Tcl word)


################################################################################################
# Calling Methods are:
# expand, {*}, eval, @>, =>>
################################################################################################
puts "Line "#__LINE__
puts ${$a(0).a}					# Will resolve to, and thus show, the method source
assert ${$a(0).a} {eq $_ $::script} -v -line #__LINE__

puts "Line "#__LINE__
# Various examples of calling our 'methods'
puts "Calling..."

# Calling member 'a' ...
puts "Line "#__LINE__
expand ${$a(0).a}				# Method 1 (Call with expand command)
assert $i {== $_ 2} -v -line #__LINE__

puts "Line "#__LINE__
{*}$test						# Works OK because it's simple

puts "Line "#__LINE__
eval $script					# FAIL (ILLEGAL. Not a well formed Tcl list)
assert $i {== $_ 3} -v -line #__LINE__

puts "Line "#__LINE__
# [{*}$script]					# FAIL (ILLEGAL. Not a well formed Tcl list)

# ILLEGAL
puts "Line "#__LINE__
{*}${$a(0).a}					# Method 2 (Call via 'wart' (expand) {*} operator)

puts "Line "#__LINE__
eval ${$a(0).a}					# Method 3 (Call with eval)
assert $i {== $_ 4} -v -line #__LINE__

puts "Line "#__LINE__
@> ${$a(0).a}					# Method 4 (Call with the experimental 'eval subst' command)
assert $i {== $_ 5} -v -line #__LINE__

puts "Line "#__LINE__
=>> ${a(0)}.a					# Eval struct command (evals foo_1.a)

# ${$a(0).a}					# This won't work because the scriptlet in '.a' isn't expanded into a command array
# Calling member 'b' ...
${$a(0).b}						# This will work since it calls a single Tcl "word" (command) via a sub/proc name

##############################################################################
# Side-issue... looking at {*} problems
# Example of no error or output being generated
##############################################################################
puts "Line "#__LINE__
option echo on
set math {/ 22}
# {*}$math 0 # Traps correctly (error state)
# {*}$math 7.0 # (ActiveState = Invalid command name /)
puts "{*}$math 7.0"		# Does not work as expected (ActiveState = '{*}/ 22 7.0') (same)
puts {{*}$math 7.0}		# Interpretation is delayed
newline

puts "Line "#__LINE__
# FIX ? {*} internal parsing will brace $x thus result in eval as 0 and DBZ error
# c/w set square {lmap {x {expr $x * $x}}}
set math {/ 22 $x}
set x 7.0
# {*}$math

puts "Line "#__LINE__
set square {lmap {x {expr $x * $x}}}
assert [expand "$square 42"] {== $_ 1764} -v -line #__LINE__
assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
set x 6
# {*}$square 6			# ActiveState = wrong args (same)
# "{*}$square 6"		# ActiveState = Invalid Command (same)
puts "{*}$square 6"
puts [{*}$square 6]		# Ticol works OK, ActivState fails with 'wrong # args: should be "lmap varList list ?varList list ...? command"'

# Note: ActiveState does not call {*} to resolve within quoted strings (expected)
##############################################################################################
# FIX outputs 0. Is parsing as div ok as DBZ can be triggered
# Again, looks to  be due to {*} wrapping $vars in {braces}
# DONE PASS Change picolEval temp to remove braces, run tests.tcl
# CHECK Does ActiveState {*} resolve expanded vars? (hard to say, doesn't look like it)
#	Test suggests that {*} in ActiveState does not expand variables
#	Viz:
#		set test {$p $q}
#		set p set
#		set q Hello
#		{*}$test
#		invalid command name "$p"
#		{*}[subst $test]
#		23
##############################################################################################
# Test simple expansion
puts "Line "#__LINE__
set p 10
set q 23
# vars
set test {/ $p $q}
puts "{*}$test"			# Both echo "{*}* $p $q"
puts "Line "#__LINE__
# {*}$test				# ActiveState: 'invalid command name "*"', Ticol = no output
puts "{*}$test"			# Both complain about bad puts args
echo "{*}$test"
puts "Line "#__LINE__
# puts [{*}$test]		# ActiveState returns the correct result, Ticol return 0 (
puts "Line "#__LINE__
puts [subst $test]		# ActivState and Ticol prints '/ 10 23'
puts "Line "#__LINE__
# puts [subst {*}$test]	# INVALID this is malformed. [subst] takes only one non-switch argument
puts "Line "#__LINE__
# puts {*}[subst $test]	# Works OK in ActiveState. Not handled in Ticol since it receives back x3 args
{*}[subst $test]		# No output even with option echo on
puts "Line "#__LINE__
echo {*}[subst $test]	# Safely displays 1st argument "/" (correctly expands the list into argv but requires usage)

puts "Line "#__LINE__
puts "Showing $test expanded into an argument array"
textcolor magenta
# Safely displays all expanded arguments
assert [dummy {*}[subst $test]] {== $_ 4} -v -line #__LINE__}
textcolor

puts "Line "#__LINE__
undef foo -nocomplain
puts "Showing $test expanded into an argument array in foo as 'args'"
proc foo {args} {
	textcolor cyan
	puts $args
	assert $args {eq $_ "/ 10 23"} -v -line #__LINE__			# ActiveState validated
	textcolor
	newline
}
foo {*}[subst $test]		# Will break into individual args then pass all via $args in one list

puts "Line "#__LINE__
puts [list {*}[subst $test]]	# Safely displays 1st argument "/"
newline

puts "Line "#__LINE__
assert [list {*}$test] {eq $_ {/ $p $q}} -v -line #__LINE__	# ActiveState validated
assert [list {*}[subst $test]] {eq $_ "/ 10 23"} -v -line #__LINE__	# ActiveState validated
assert [subst $test] {eq $_ "/ 10 23"} -v -line #__LINE__			# ActiveState validated
assert [subst {*}$test] {eq $_ "/"} -v -line #__LINE__				# MALFORMED
# assert {*}[subst $test] {eq $_ "*"} -v -debug -trace -line #__LINE__	# MALFORMED input to [assert]
# assert [list {*}[subst $test]] {eq $_ "*"} -v -line #__LINE__


##############################################################################
# Set stuct field with simple values
# This is something of a kludge still since we dereference to set, 
# (which is not intuitive)
##############################################################################
vars
# struct set $a(0).a 23		# This is incorrect since it will dereference the array and member
struct set ${a(0)}.a 23		# Dereferences to 'foo.a'
struct set ${a(0)}.b $pi
assert $a(0).a {== $_ 23} -v -line #__LINE__
assert $a(0).b {== $_ $::pi} -v -line #__LINE__

###############################################################################################
unset a p q math script square test x -nocomplain
unset foo_1
unset foo_2
unset template
static i -unset
static foo::i -unset
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #