############################################################################################
# File:				test_class.tcl (NOT A UNIT TEST)
# Author:			M Shaw
# Purpose:			Experiments in emulating classes
# Language:			Ticol Tcl v1.20+, v1.26+
#
# Versions:
# 1.00 10/Nov/2016	First version
# 1.01 21/Jan/2022	Updated with added complex dereferences
############################################################################################
# Notes:
# This idea is purely for amusement. There is no serious intention to implement classes
# and the Ticol [struct] wasn't designed to support common class-like behaviour
# To do so would either require a rewrite of the interpreter core or the provision of a
# plugin module of some sort.
#
# Ticol structs are mainly designed to interact with lib calldll_* but can also be
# persuaded to be used as a structured type within scripts
#
# The syntax has been 'tortured' somewhat to use [->] etc.
# [-> struct.field] is shorthand for [struct item $x] which borrows from the Ticol 
# [item] paradigm
#	-> ?
# e.g. [-> ${x}.f1]
#
# Note that [<command>] will dereference and call a command similar to C/C++ in the
# form funct() -> *funct. Also {*} may be used to execute a dereferenced struct member
# Some extremely complex expressions can be created
# Note that [eval] will perform variable substituion and command execution whereas 
# a simple struct field dereference will not
############################################################################################
# Drawbacks:
# * No inheritance
# * No data hiding / encapsulation
# * No automatic true constructor on instantiation or destructor on scope exit
# * Procs are not inherently 'bound' to a class or instance other than via the
# * pseudo-constructor. The constructor must 'mangle-in' the relevant proc names
# * In all, this is all horribly 'kludgy'
#########################################################################################
# FIX implement test of struct get/set
#####################################################################################

struct class {											# Fields must be large enough to hold 32-bit NUM_MAX
	f1 20
	f2 20
	s1 256
	r1 20
	fx 256
}

new class c												# Create an instance of our 'class'
new class d												# Another instance

# Add some member definitions
struct set c.f1 puts									# Emulates pointer to function - '(long*)(LPSTR)'
struct set c.f2 date									# Ditto
struct set c.s1 "Hello world"							# Simple, static data string
struct set c.r1 pi										# Variable name reference
#struct set c.fx {puts {Hello from complex $::date}}	# Inner braces also work but will mask contents
struct set c.fx {puts "Hello from complex $::date"}		# Outer braces only give access to vars and commands

############################################################################################

proc call_class {x} {
	textcolor yellow
	puts "Enter proc: [info procname] '[info args [info procname]]'"
	textcolor
	upvar ${x}											# We get only a name so must upvar
	upvar pi											# Required only for [eval] use below
	puts "DEBUG ref: '${x}.f1' @"#__LINE__				# Var dereference translates to c.f1
	puts "DEBUG '[-> ${x}.f1]' @"#__LINE__				# Struct item translates c.f1 to 'puts'
	#puts "DEBUG '[struct item ${x}.f1]'"				# Alternate syntax to [->] in the above
	[-> ${x}.f1] "Hello from inside proc @"#__LINE__	# Dereference to command [puts] "Hello from member"
	#puts [eval ::$[-> ${x}.r1]]						# Display using [eval] (requires upvar)
	puts "pi=[set ::[-> ${x}.r1]]"						# Display var using [set] (pi)
	#[struct item ${x} ${x}.f1] "Hello from member"		# Dereference to 'function call'
	puts "Date is: '[eval {[-> ${x}.f2] now}]'"			# As VB double
	puts "Date is: '[date [eval {[-> ${x}.f2] now}]]'"	# As a date, proper
	[-> ${x}.f1] [eval {[struct item ${x}.f2] now} ]	# Equiv to 'puts [date now] ([date] becomes an argument to [puts])
}

############################################################################################

# // Call from ROOT context //
textcolor darkyellow
puts "Line: "#__LINE__
[-> c.f1] "Hello from main()\n"	# [puts]
puts "Line: "#__LINE__
[-> c.f1] $c.s1					# Will print the address stored in the var
puts "Line: "#__LINE__
[-> c.f1] [subst $c.s1]			# Will print the address stored in the var
puts "Line: "#__LINE__
#[-> c.f1] -> c.s1				# INVALID Will print the struct field name
puts "Line: "#__LINE__
#[-> c.f1] [set [-> c.s1]]		# INVALID (BUG "c.s1}")
#[-> c.f1] ${-> c.s1}			# INVALID (BUG "c.s1}")
puts "Line: "#__LINE__
#[-> c.f1] '[set {-> c.s1}]'	# FIX "" (set does not eval args?)
puts "Line: "#__LINE__
[-> c.f1] '[eval {-> c.s1}]'	# Will print "Hello world"
puts "Line: "#__LINE__
[-> c.f1] '[-> c.s1]'			# == puts "Hello world"
puts "Line: "#__LINE__
[-> c.f1] $c.s1					# OK
puts "Line: "#__LINE__
puts [-> c.s1]
puts "Line: "#__LINE__

# Execute struct field using {*} wart operator
#{*}-> c.fx						# Does nothing
#-> c.fx						# Does nothing
#[-> c.fx]						# Invalid
{*}[-> c.fx]					# Works OK
puts "Line: "#__LINE__
{*}$c.fx						# This works OK

# Dereference an indirect/complex variable reference and execute using the {*} wart operator
puts "Line: "#__LINE__
set foo c
{*}[-> ${foo}.fx]				# Works OK (complex dereference)
{*}${${foo}.fx}					# Works OK (complex dereference)

# Simple struct dereference performs no embedded command execution or variable substitution
set date [date]
puts "Line: "#__LINE__
puts $c.fx						# Prints out the entire field, literally
#${$c.fx}						# Double-dereferencing here won't execute!
puts "Line: "#__LINE__
eval $c.fx						# Will execute AND perform variable substitution
puts "Line: "#__LINE__
{*}$c.fx						# Will execute but does no variable substitution

puts "Line: "#__LINE__
textcolor
newline

# [-> c.s1] == [struct item c.s]
assert [-> c.s1] {eq $_ "Hello world"} -v -line #__LINE__
assert [set [-> c.r1]] {eq $_ $pi} -v -line #__LINE__
#assert ${-> c.r1} {eq $_ $pi} -v -line #__LINE__		# INVALID
#assert [-> c.r1] {eq $_ $pi} -v -line #__LINE__		# INVALID
newline

call_class c											# Call the class

############################################################################################
unset date foo -nocomplain
struct unset c -nocomplain
struct unset d -nocomplain
struct unset class  -nocomplain
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #