############################################################################################
# File: 			test_deref_proc.tcl
# Purpose:			Test file for [proc] dereferencing via vars etc. 
# Author: 			M Shaw
# Language:			Ticol Tcl v1.19+..v1.26+
#
# Versions:
# 1.00 16/Aug/2016	First version
# 1.01 18/Jul/2017	Added tests, replace [checkex] with [assert]
# 1.02 09/Sep/2019	Added various 'wart' dereference tests
############################################################################################
# Notes: 
# A var can store a command name which can be called by dereferencing
# Arguments can also be passed via dereferenced vars
# Rapid [if] switching can be achieved using arrays
############################################################################################
option expression off
option echo on

# Call proc with arguments via an intermediate variable
undef foo -nocomplain
set i 0
proc foo {x} {
	textcolor darkyellow
	puts "Enter foo 'x:$x'"
	textcolor
	set ::i $x
	return $x
}
set q {foo}
set r 23
$q $r											# Dereference
assert $i {== $_ 23} -v -line #__LINE__

# FIX ? Does not like dquoted strings
# FIX array set does not strip enclosing dquotes
# Array rvalue must be in the form "<commandname> ?args?"
array set a {						# Mimic pointers to functions
	0 {foo "Hello 0"}
	1 "puts {Hello 1a}"
	2 {echo {Hello 2}}
	3 "foo 23"						# A/S allows this, we do also but don't allow multi line dquotes
}

array walk a -integer
newline
puts "a(3) '$a(3)'"
puts "a(0) '$a(0)'"
puts "a(1) '$a(1)'"
puts "a(2) '$a(2)'"
puts "a(3) '$a(3)'"

# Note: {*} must immediately prefix any deferferenceable command (no intervening whitespace allowed)
puts "call a(0) '$a(0)'"
{*}$a(0)
assert $i {eq $_ {Hello 0}} -v -line #__LINE__
puts "call a(1) '$a(1)'"
{*}$a(1)
puts "eval '$a(2)'"
{*}$a(2)
set i 0
{*}$a(3)
assert $i {eq $_ 23} -v -line #__LINE__
#die

{*}$a([rnd 0 2])
{*}$a([rnd 0 2])
{*}$a([rnd 0 2])

set q {foo "hello at line __LINE__" }
{*}$q

############################################################################################
# WART TESTS ({*})
# FIX Expand operator has no return value and [assert] will test for its first argument
# Check ActiveState
############################################################################################

{*}$a(3)
# puts {*}$a(3)			# Won't work. Invalid stream handle (ActiveState is same - can't find channel named foo
# puts {{*}$a(3)}		# => {*}$a(3)	ActiveState is same '{*}$f'
puts "{*}$a(3)"			#		ActiveState -> {*}foo 23

puts "Line: "#__LINE__
puts [expand $a(3)]		# PASS

puts "Line: "#__LINE__
assert [expand $a(3)] {eq $_ 23} -v -line #__LINE__		# NOT 23

puts "Line: "#__LINE__
textcolor cyan
puts "'[call {*}$a(3)]'"					# DONE FIX [call] exits script (was returning code return (2) back to parent script)
textcolor
assert [call {*}$a(3)] {== $_ 23} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor cyan
puts "'[gosub {*}$a(3)]'"					# [gosub] works OK but obviously, passes no arguments so we get no return as we have no default arg
textcolor

puts "Line: "#__LINE__
textcolor cyan
puts [dummy {*}$a(3)]						# [dummy] works OK
assert [dummy {*}$a(3)] {== $_ 3} -v -line #__LINE__
textcolor

puts "Line: "#__LINE__
textcolor cyan
[{*}$a(3)]									# Command call [] brackets work OK
textcolor

puts "Line: "#__LINE__
textcolor cyan
puts [expr {*}$a(3)]						# [eval] no output without print command, we then get literal 'foo 23'
textcolor

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
# FIX returns: 'assert: Line 65: Assertion failed: 'foo' != '23'' no matter what rvalue!
# This is because the array expands to 2 arg terms, one of which is captured by [assert]'s rvalue (argv[2])
#assert {*}$a(3) {} -trace -v -line #__LINE__		#
# The correct method of calling is this...
assert [{*}$a(3)] {== $_ 23} -trace -v -line #__LINE__
# Or to use [expand]
assert [expand $a(3)] {== $_ 23} -trace -v -line #__LINE__

array unset a -nocomplain
undef foo -nocomplain
unset i q r -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #	