###########################################################################################
# File:				test_dereference.tcl
# Author:			M Shaw
# Purpose:			Test PERL-style multi-level variable dereferencing
#
# Versions:
# 1.00 04/Dec/2016 First version
# 1.01 06/Dec/2016 Can now resolve top-level array (entry) var along the chain
# 1.02 15/Jun/2017 Replaced [checkex] with [assert]
###########################################################################################
# Notes:
# This is an experimental attempt to implement a clean and simple, PERL-style multi-level
# dereferencing which will handle arrays etc. as final or intermediate nodes
# A new funct calls [set] internally to resolve all but the final stage of dereferencing
#
# References:
# http://wiki.tcl.tk/4589
###########################################################################################
# DONE FIX checkex $$$$$$d {eq $x $$$$$d}		# Final array FIX set: Unhandled error
# DONE FIX $$$$$d Should not resolve and should raise error (should be $$$$d max)
# DONE FIX Add VNF error messages
# FIX Refactor and compress code in picolEval
###########################################################################################
/* /////////////////////////////////////////////////////////////////////
	EXAMPLE from http://wiki.tcl.tk/4589

This can be done more than one time by nesting set. Compare the following C and Tcl programs, that perform 
the same trivial task, and exhibit remarkable similarity:
#include <stdio.h>
int main(void) {
    int    i =      42;
    int *  ip =     &i;
    int ** ipp =   &ip;
    int ***ippp = &ipp;
    printf("hello, %d\n", ***ippp);
    return 0;
}
...and Tcl:
set i    42
set ip   i
set ipp  ip
set ippp ipp
puts "hello, [set [set [set [set ippp]]]]"
assert [set [set [set [set ippp]]]] {== $_ 42}
////////////////////////////////////////////////////////////////////// */
#define ALLOW_ARRAY_PROMOTE			# for now we will allow standard vars to be promoted to arrays by reassignment

option expression off
option echo on

textcolor yellow
puts "Testing dereference variables ..."
textcolor
newline

#ifndef ALLOW_ARRAY_PROMOTE
puts "Check block promoting non array type to array ... @"#__LINE__
puts "Line: "#__LINE__
set i 0
set c hello
try {
	set c(1) b
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
#endif
# #exit

###########################################################################################

textcolor yellow
puts "Testing multi-level dereferencing ... @"#__LINE__
textcolor
newline

# Examples per: http://wiki.tcl.tk/4589
set line #__LINE__
puts "Line: $line	"
set i    42
set ip   i
set ipp  ip
set ippp ipp
assert [set [set [set [set ippp]]]] {== $_ 42} -v -line #__LINE__

# Test Ticol multi-dereference feature ...
assert $i {eq $_ 42} -v -line #__LINE__
assert $ip {eq $_ i} -v -line #__LINE__
assert $ipp {eq $_ ip} -v -line #__LINE__
assert $ippp {eq $_ ipp} -v -line #__LINE__
assert $$ip {eq $_ 42} -v -line #__LINE__
assert $$ipp {eq $_ i} -v -line #__LINE__
assert $$ippp {eq $_ ip} -v -line #__LINE__
assert $$$ippp {eq $_ i} -v -line #__LINE__
assert $$$$ippp {eq $_ 42} -v -line #__LINE__
newline

unset ip ipp ippp -nocomplain

# DONE picolParseVar - change to while($) and accumulate all prefixes. Then call picolGetVarEx to call itself recursively for each $ prefix?
set line #__LINE__
#puts "test: '$$$$ippp'"

textcolor yellow
puts "Testing multi-level array dereferencing ... @"#__LINE__
textcolor
newline

unset b -nocomplain
array unset b -nocomplain
unset c -nocomplain
unset d -nocomplain
set a(1) "one"
set b a(1)
set c b
set d c
# Multi-level dereferencing ...
# $d => c
# $$d => b
# $$$d => a(1)
# $$$$d => "one"
# $$$$$d => $"one" (ERR)

# Resolve for 'd'
puts "Resolve for 'd' - Start a(1) is $a(1) @"#__LINE__
assert $d {eq $_ c} -v -line #__LINE__
assert $$d {eq $_ b} -v -line #__LINE__
assert $$$d {eq $_ a(1)} -v -line #__LINE__		# new dereferencing can't dereference arrays yet 2017/May/07
assert $$$$d {eq $_ one} -v -line #__LINE__					# Final string

newline
textcolor yellow
puts "Ignore catch errors @"#__LINE__ 
textcolor
newline

# checkex $$$$$$d {eq $x $$$$$d}			# Final array FIX set: Unhandled error
set line #__LINE__
catch {
	puts "Line $line '$$$$$d'"				# x5 FAIL
}
catch {
	puts "Line $line '$$$$$$d'"				# x6 FAIL
}
set line #__LINE__

if {
	[catch {
			puts "$line: \$\$\$\$\$d resolves to '$$$$$d'"		# FIX $$$$$d Should not resolve!! (should be $$$$d max)
		}
	]
} {														# n.b. keyword "then" is not yet supported
	textcolor red
	puts "Caught exception at line $line resolving \$\$\$\$\$d"
	textcolor
}

newline

textcolor yellow
puts "Testing multi-level array dereferencing ... @"#__LINE__
textcolor
newline

# Test with intermediate array variables
puts "Line : "#__LINE__
set a(1) one
puts "Line : "#__LINE__
set b a(1)
puts "Line : "#__LINE__

vars c*
unset c -nocomplain
set c(1) b
puts "Line : "#__LINE__
set d  c(1)
puts "Line : "#__LINE__
set line #__LINE__
assert $$$$d {eq $_ one} -v -line #__LINE__		# Intermediate array 
puts "$line: x4 \$\$\$\$d resolves to '$$$$d' (should resolve)"

option echo on
puts "Line : "#__LINE__
set line #__LINE__
catch {
	puts "$line: x5 \$\$\$\$\$d resolves to '$$$$$d' (should not resolve)"
}
newline

textcolor yellow
puts "Testing multi-level array dereferencing ... @"#__LINE__
textcolor
newline

# Test top level dereference as array
# FIX cannot resolve top level array reference. Not yet implemented
# c(1) -> b -> a(1) -> "one"

set a(1) "one"
set b a(1)
set c(1) b

# Multi-dereferencing ...
# $c(1) -> b
# $$c(1) -> a(1)
# $$$c(1) -> "one"
# $$$$c(1) -> $"one" (ERR)

puts "\$c(1) resolves to '$c(1)'\t(Should be [set c(1)])"					# b
assert $c(1) {eq $_ b} -v -line #__LINE__
puts "\$\$c(1) resolves to '$$c(1)'\t(Should be a(1))" #[set $c(1)])			# a(1)
assert $$c(1) {eq $_ a(1)} -v -line #__LINE__
puts "\$\$\$c(1) resolves to '$$$c(1)'\t(Should be one)" #[set [set $c(1)]])	# one
assert $$$c(1) {eq $_ one} -v -line #__LINE__

newline
puts "\$\$\$\$c(1) should not resolve and should raise an exception ... @"#__LINE__

set line #__LINE__
if { 
	[catch { 
			puts "$line: try resolve invalid \$\$\$\$c(1) '$$$$c(1)'"		# Should not resolve
		} 
	] 
} {																	# n.b. keyword "then" is not yet supported
	textcolor red
	puts "catch: Caught exception at line $line resolving \$\$\$\$c(1)"
	textcolor
}
newline

array unset a -nocomplain
unset b -nocomplain
array unset c -nocomplain
unset d -nocomplain

# Array with struct reference ###################################

textcolor yellow
puts "Testing struct dereferencing ... @"#__LINE__
textcolor
newline

puts "Line :"#__LINE__

struct st {
	a 10
	b 20
	c 30
}
set a(0) st

# We can't yet do:  new st a(0)  :and get an array var
puts "Line :"#__LINE__
struct set st.a Hello
puts "Line :"#__LINE__
assert $st.a {eq $_ Hello} -v -line #__LINE__
assert ${st.a} {eq $_ Hello} -v -line #__LINE__
assert ${$a(0).a} {eq $_ Hello} -v -line #__LINE__
assert ${${a(0)}.a} {eq $_ Hello} -v -line #__LINE__
assert [-> st.a] {eq $_ Hello} -v -line #__LINE__
assert [struct item st.a] {eq $_ Hello} -v -line #__LINE__
assert [-> ${a(0)}.a] {eq $_ Hello} -v -line #__LINE__
#assert $$a(0).a {eq $_ Hello} -v -line #__LINE__		# Will fail due to array resolution having precedence over std. var

puts "Line :"#__LINE__
set a(1) Hello
puts "Line :"#__LINE__
set ss 1
puts "Line :"#__LINE__
puts ${a($ss)}text
assert ${a($ss)}text {eq $_ Hellotext} -v -line #__LINE__

# Array with struct reference ###################################

newline
textcolor yellow
puts "Testing proc upvar complex struct dereference ...\r\n"
textcolor

undef foo -nocomplain
struct set st.a 23
proc foo {x} {
	upvar $x
	assert ${${x}.a} {eq $_ 23} -v -line #__LINE__
}
foo st
undef foo -nocomplain
newline

puts "Line :"#__LINE__
unset a(1) ss st -nocomplain
#unset * -glob -nocomplain

struct unset st -nocomplain
array unset a -nocomplain
unset i line ss -nocomplain

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #