############################################################################################
# File: 			test_dict.tcl
# Purpose:			Test [dict]
# Language:			Ticol Tcl v1.26+
# See also:			test_dict_map.tcl
#
# Versions:
# 1.00 01/Oct/2019	First version
# 1.01 03/Oct/2019	Added more tests
############################################################################################
# Notes:
# DESIGN DEF [dict] will store rvalues unbraced
#	Reason: the purpose of dict is to rapidly reference rvalues and this is the most
#	frequent use. Return of the whole or part dictionaries is less frequent
#	if rvalues are braced then removing the braces will be less efficient
#	Thus, anything which returns the whole dict must brace the dict rvalues
#	However, nested dicts will still need *internal* braces stored, but not the
#	outer ones, so that picolGetDictStringF can be consistent with tuples
############################################################################################
# TODO:
# ADD dict filter
# ADD dict incr
# FIX [dict set] should set multiple keys with only one value
# FIX !!!! dict hashtables are un-ordred. Dicts need to be ordered
#	This is a bit of a problem
#	What can we use as an alternate to hashtables?
#		stack?, linkedlist?
#	Maybe we could store an lvalue+rvalue struct in the hashtable as a* -> long?
#	Then increment the string-cast index counter to the end of table_size ?
#	Would require these structs to be assigned by [dict set] and freed by [unset]
#	Would be basically an integer indexed hash array -> {dictstruct lvalue, rvalue}
# DONE ADD [unset dictvar] leaktest loop
# See: http://www.wellho.net/mouth/3614_Tcl-dicts-a-tutorial-and-examples.html
# DONE FIX Overrun in dict get $mydic foo bar spong phiew => 'poo'
############################################################################################
/*
# Commands
dict append dictionaryVariable key ?string ...? 
dict create ?key value ...? 
dict exists dictionaryValue key ?key ...? 
dict filter dictionaryValue filterType arg ?arg ...? 
dict filter dictionaryValue key ?globPattern ...? 
dict filter dictionaryValue script {keyVariable valueVariable} script 
dict filter dictionaryValue value ?globPattern ...? 
dict for {keyVariable valueVariable} dictionaryValue body 
dict get dictionaryValue ?key ...? 
dict incr dictionaryVariable key ?increment? 
dict info dictionaryValue 
dict keys dictionaryValue ?globPattern? 
dict lappend dictionaryVariable key ?value ...? 
dict map {keyVariable valueVariable} dictionaryValue body
dict merge ?dictionaryValue ...?
dict remove dictionaryValue ?key ...?
dict replace dictionaryValue ?key value ...?
dict set dictionaryVariable key ?key ...? value 
dict size dictionaryValue 
dict unset dictionaryVariable key ?key ...? 
dict update dictionaryVariable key varName ?key varName ...? body 
dict values dictionaryValue ?globPattern? 
dict with dictionaryVariable ?key ...? body

dict filter dictionaryValue filterType arg ?arg ...? 
	dict filter dictionaryValue key ?globPattern ...? 
	dict filter dictionaryValue script {keyVariable valueVariable} script 
	dict filter dictionaryValue value ?globPattern ...? 

DONE ADD [dict clear] (non standard)
DONE ADD [dict append]
DONE FIX [dict create <literal>]
DONE FIX [dict set <dictvar> key ?key ...? value]
DONE FIX [dict set] if exists then append args to incremented sequence number
DONE FIX [dict size <dictvar|string>]
DONE FIX [dict walk <dictvar>] non standard
DONE FIX [dict info <dictvar>] see array info
DONE ADD [dict merge]
DONE ADD [dict for]
DONE FIX [dict get]
DONE FIX [dict exists <dictvar>]
DONE ADD $dictvar dereference
DONE ADD [set] cast to dictionary
DONE ADD [dict lappend]
DONE ADD [dict replace]
DONE ADD [dict remove]
DONE FIX [dict unset <dictvar> value] is wrong
DONE ADD [dict map]

ADD [dict filter]
ADD [dict update]	# Poss N/R
ADD [dict with]		# Poss N/R
ADD [dict incr]		# Poss N/R

Useful Test Data
A3
4470 Great Northern
4471 Sir Frederick Banbury
4472 Flying Scotsman
4473 Solario
4474 Victor Wild
4475 Flying Fox
4476 Royal Lancer
4477 Gay Crusader
4478 Hermit
4479 Robert the Devil
4480 Enterprise
4481 St Simon
2543 Melton
2544 Lemberg
2545 Diamond Jubilee
2546 Donovan
2547 Doncaster
2548 Galtee More
2549 Persimmon
2550 Blink Bonny
2551 Prince Palatine

A4
2510 Quicksilver
2511 Silver King
2512 Silver Fox
4482 Golden Eagle
4483 Kingfisher
4484 Falcon
4485 Kestrel
4486 Merlin
4487 Sea Eagle

60014 2509 Silver Link
60015 2510 Quicksilver
60016 2511 Silver King
60017 2512 Silver Fox
60023 4482 Golden Eagle
60024 4483 Kingfisher
60025 4484 Falcon
60026 4485 Kestrel
60027 4486 Merlin
60028 4487 Sea Eagle
*/
############################################################################################

textcolor yellow
puts "Test \[dict\] ..."
newline

############################################################################################
# DICT CLEAR
# Clear a dictionary of all keys and values but retain the type information and stucture
puts "Line: "#__LINE__
unset mydict -nocomplain
puts "Line: "#__LINE__
set mydict [dict create a b c d]
puts "Line: "#__LINE__
assert $mydict {eq $_ {c d a b}} -v -line #__LINE__		# A/S 8.6
puts "Line: "#__LINE__
puts "[dict clear mydict] @"#__LINE__
puts "Line: "#__LINE__
assert [type mydict] {eq $_ dictionary} -v -line #__LINE__
puts "Line: "#__LINE__
assert $mydict {eq $_ {}} -v -line #__LINE__

############################################################################################
# DICT SET
# Create a dictionary variable
# Bracing args is not allowed in std. Tcl
# FIX - add nested cases
# FIX We can now accumulate in between [dict set] calls
textcolor yellow
puts "Calling \[set mydict\] ... @"#__LINE__
textcolor
unset mydict -nocomplain
assert [dict set mydict a b] {eq $_ {a b}} -v -line #__LINE__		# A/S 8.6

#dict set course Tcl Programming 3 4 5
#Tcl -> {Programming {3 {4 5}}}
assert [  dict set course Tcl Programming 3 4 5] {eq $_ {Tcl {Programming {3 {4 5}}}}} -v -line #__LINE__		# A/S 8.6
unset course mydict

puts "Line: "#__LINE__
puts "Create dict: '[dict create Tcl {Programming 3} 4 4]'"
assert [dict set course Tcl {Programming 3} 1 2] {eq $_ {Tcl {{Programming 3} {1 2}}}} -v -line #__LINE__		# A/S 8.6 BUG!
unset course course -nocomplain

############################################################################################
# $dict (DOLLAR) DEREFERENCE
# FIX does not recast return to <dictionary> type (return is string and a TCL list)
textcolor yellow
puts "Calling \$dict defeference... @"#__LINE__
textcolor

unset foo -nocomplain
assert [dict set foo Tcl {Programming 3} 1 2] {eq $_ {Tcl {{Programming 3} {1 2}}}} -v -line #__LINE__		# A/S 8.6 BUG!
puts "\$foo is '$foo' @"#__LINE__
assert $foo {eq $_ {Tcl {{Programming 3} {1 2}}}} -v -line #__LINE__
# FIX should stored rvalue in dict be braced if it contains spaces?
unset foo

############################################################################################
# $dict (DOLLAR) DEREFERENCE ROUND TRIP 
# Set cast return to var and then set back to var via set
set foo [dict create a b c d]			# b {c d}
puts "Foo -> '$foo'"
set newdict [set foo]
puts "newdict -> '$newdict'"
# FIX requires ordered dictionaries...
assert $newdict {eq $_ {c d a b}} -v -line #__LINE__

unset foo newdict

############################################################################################
# DICT WALK (non standard)
#
textcolor yellow
puts "Calling \[dict walk\] ... @"#__LINE__
textcolor
set mydict [dict create a b c d]			# b {c d}
#assert [dict exists mydict] {eq $_ 1} -v -line #__LINE__
puts [dict walk mydict]
unset mydict

############################################################################################
# DICT INFO (non standard)
#
textcolor yellow
puts "Calling \[dict info\] ... @"#__LINE__
textcolor
set mydict [dict create a b c d]			# b {c d}
puts [dict info mydict]
unset mydict


############################################################################################
# UNSET <dictvar>
# Delete a dictionary variable
textcolor yellow
puts "Calling \[unset\] ... @"#__LINE__
textcolor
set mydict [dict create a b c d]			# b {c d}
assert [is_set mydict] {== $_ $::true} -v -line #__LINE__		# A/S 8.6 
unset mydict
assert [is_set mydict] {== $_ $::false} -v -line #__LINE__		# A/S 8.6 
unset mydict -nocomplain

# NOT VALID
#puts "size is [dict size mydic]"
#assert [dict size mydic] {== $_ 1} -v -line #__LINE__		# A/S 8.6

############################################################################################
# DICT GET
#
textcolor yellow
puts "Calling \[dict get\] ... @"#__LINE__
textcolor
# dict get with  no key
# assert [dict get {a b c d}] 
assert [dict get {}] {eq $_ {}} -v -line #__LINE__						# A/S 8.6 return ''
assert [dict get {a b c d}] {eq $_ {a b c d}} -v -line #__LINE__		# A/S 8.6 return 'a b c d'
assert [dict get {a {b c}}] {eq $_ {a {b c}}} -v -line #__LINE__		# A/S 8.6 return 'a {b c}'

# dict get with key
assert [dict get {a b} a] {eq $_ {b}} -v -line #__LINE__				# A/S 8.6 return 'b'
assert [dict get {a {b c}} a] {eq $_ {b c}} -v -line #__LINE__			# A/S 8.6 return 'b c'

# Multiple keys
assert [dict get {a b c d} a] {eq $_ b} -v -line #__LINE__		# A/S 8.6 return 'a b c d'
set i 0
try {
	# Invalid as nested dict is not present so 2nd key is not valid
	assert [dict get {a b c d} a c] {eq $_ {b d}} -v -line #__LINE__		# A/S 8.6 return 'a b c d'
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
unset i

assert [dict get {Tcl {Learning {3 {4 5}}}} Tcl] {eq $_ {Learning {3 {4 5}}}} -v -line #__LINE__			# A/S 8.6 return 'b c'
catch {
assert [dict get {Tcl {Learning {3 {4 5}}}} Learning] {eq $_ {b c}} -v -line #__LINE__			# A/S 8.6 return 'b c'
}
puts "Line: "#__LINE__

# Use the dict command to retrieve elements from a standard Tcl list passed as a literal
set demo {Anne Jones Bill Brown Charlotte Steeple Dennis {The Menace} Enid Blyton
        Fred Dibnah Gwynn Grimm Henry Eighth Isobel Quietly}
set surname [dict get $demo Enid]
puts $surname
assert $surname {eq $_ Blyton} -v -line #__LINE__

unset demo surname


############################################################################################
# MORE DICT GET
# mydict -> 'foo bar spong poo'
# REQUIRES $<dict>

textcolor yellow
puts "Calling \[dict get\] ... @"#__LINE__
textcolor

puts "Line: "#__LINE__
dict set mydic foo bar spong poo
#die "$dict NYI @"#__LINE__
puts "Line: "#__LINE__
textcolor cyan
puts "exists foo? [dict exists mydic]"
#puts "dict get: '[dict get $mydic]'"		$ NYI
# FIX failing on wrong vartype AUTO
assert [dict exists mydic] {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor yellow
puts "dict get \$mydic foo => '[dict get $mydic foo]'"
puts "Should be:             'bar {spong poo}'"

#assert [dict get $mydic] {eq $_ {}} -v -line #__LINE__		# A/S 8.6
assert [dict get $mydic foo] {eq $_ {bar {spong poo}}} -v -line #__LINE__		# A/S 8.6

unset mydic

############################################################################################
# DICT UNSET
# dict unset dictionaryVariable key ?key ...? 
# dict exists dictionaryValue key ?key ...? 
textcolor yellow
puts "Calling \[dict unset\] ... @"#__LINE__
textcolor

set dict [dict create a foo b bar c moo d par]
puts "dict is '$dict', deleting items b,d"
dict unset dict b d
puts "dict is now: '$dict'"			# dict is now: 'c moo a foo'
assert [dict get $dict a] {eq $_ foo} -v -line #__LINE__
# FIX implement ordering so we can check the whole dict string!!!!
#assert [dict get $dict b] {eq $_ {}} -v -line #__LINE__			# FIX should we err or just return {}
assert [dict exists dict a] {== $_ 1} -v -line #__LINE__
assert [dict exists dict b] {== $_ 0} -v -line #__LINE__
assert [dict exists dict c] {== $_ 1} -v -line #__LINE__
assert [dict exists dict d] {== $_ 0} -v -line #__LINE__
unset dict

############################################################################################
# DICT FOR 
set i 0
dict for {k v} {a b c d} {
	puts "$k $v"
	if {== $i 0} {
		assert "$k $v" {eq $_ {a b}} -v -line #__LINE__
	} elseif {== $i 1} {
		assert "$k $v" {eq $_ {c d}} -v -line #__LINE__
	}
	++ i
}
# a b
# c d

unset k v i
#stop
############################################################################################
/*
# FIX syntax!
puts "Line: "#__LINE__
textcolor cyan 
puts "'[dict exists $mydic foo bar spong]'"
assert [dict exists $mydic foo bar spong] {== $_ 1} -v -line #__LINE__		# A/S 8.6
textcolor yellow
puts "dict get \$mydic foo bar spong => '[dict get $mydic foo bar spong]'"
puts "Should be:                       'poo'"
#dict get $mydic foo bar spong
textcolor
newline
*/

############################################################################################
# dict set mydic foo bar spong poo
/*
puts "Line: "#__LINE__
assert [dict exists fred] {== $_ 0} -v -line #__LINE__		# A/S 8.6

# FIX *************************************
puts "Line: "#__LINE__
puts "\[dict get foo\] -> '[dict get foo]'"

puts "Line: "#__LINE__
# Q no [dict get name] ?
unset mydic -nocomplain
set mydic foo
puts "Get $mydic ..."
puts "dict get $mydic spong => '[dict get $mydic spong]'"
assert [dict get $mydic spong] {== $_ poo} -v -line #__LINE__		# A/S 8.6
puts "Line: "#__LINE__

puts "Line: "#__LINE__
puts "Should be:              ''"
puts "dict get \$mydic foo bar spong phiew => '[dict get $mydic foo bar spong phiew]'"

puts "Line: "#__LINE__
puts "Should be:                             ''"
puts "dict get \$mydic foo bar spong blah blah blah blah blah => '[dict get $mydic foo bar spong blah blah blah blah blah]'"


puts "Line: "#__LINE__
textcolor red
puts "Tests: block 2"
textcolor
newline
dict set q  foo bar
#foo bar
dict set q  poo wah
#foo bar poo wahz
dict set q ooh yah
#foo bar poo wah ooh yah

puts "Line: "#__LINE__
puts "Dictionary \$q is now '$q'"
puts "Should be:           'foo bar poo wah ooh yah'"
#dict get foo
#missing value to go with key

puts "Line: "#__LINE__
puts "dict get \$q foo '[dict get $q foo]'"

puts "Line: "#__LINE__
puts "Should be       'bar'"
puts "dict get \$q poo '[dict get $q poo]'"

puts "Line: "#__LINE__
puts "Should be:      'wah'"
puts "dict get \$q bah '[dict get $q bah]'"

puts "Line: "#__LINE__
puts "Should be:      ''"
dict set q blah wah fah
puts "Dict is: '$q'"
puts "dict get \$q blah '[dict get $q blah]'"

# */

puts "Line: "#__LINE__

###########################################
# Split off to dict timetable
undef timetable -nocomplain
proc timetable {type schedule} {
	textcolor white
	puts "Enter proc timetable"
	puts "\n----------> $type"
	dict for {where when} $schedule {
		puts [format "%-25s %11s" $where $when]
	}
	textcolor
}
  
puts "Line: "#__LINE__
############################################################################################
# DESIGN ISSUE for [dict create]
# The only way to create a dict from multiple key+value pairs without casting to an intermediate var via [set] 
# is using [dict create], [dict set] is no use as it is designed to handle only x1 pair
# To make the standard syntax work we would have to add [dict set] ability to [set] and pass back type info from [dict create]
# Changed from dict create with [set] -> dict type, to dict set
# This creates a flat dictionary (list)
# [dict create] seems pointless because all it does is duplicate the function of [list] except casting the return data type to [dict] via [set]


puts "Line: "#__LINE__
set mydata [dict create "Cardiff Central" 06:28 Newport 06:42 "Severn Tunnel Junction" 06:53 Pilning 07:03 Patchway 07:06]
textcolor green
puts "mydata -> '$mydata' @"#__LINE__
textcolor
dict walk mydata

# [dict set] creates a nested dictionary (list) if there are more than 2 items (key+value pair)
# so, at least adds extra functionality over [list] + casting
#dict create mydata "Cardiff Central" 06:28 Newport 06:42 "Severn Tunnel Junction" 06:53 Pilning 07:03 Patchway 07:06

puts "type='[type mydata]'"
assert [type mydata] {eq $_ "dictionary"} -v -line #__LINE__

# FIX (ordering fails!!!)
textcolor red
puts "FIX ordering issues in dicts"
textcolor
puts "\$mydata '$mydata' @"#__LINE__
#assert $mydata {eq $_ {{Cardiff Central} 06:28 Newport 06:42 {Severn Tunnel Junction} 06:53 Pilning 07:03 Patchway 07:06}} -v -line #__LINE__
puts "type='[type mydata]'"

puts "Line: "#__LINE__

# FIX won't work dict create does not create a dict var
# set mydata must convert the var type to dictionary, we can't do that!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# We would have to add more complexity to the interpreter core by passing back i->type for [set] to use
#dict lappend mydata {Filton Abbey Wood} 07:09		
# This won't work  because [set] (above) cannot 'promote' a return to dict type!
#set mydata [dict lappend mydata {Filton Abbey Wood} 07:09]		# mydata must be a dict

puts "Line: "#__LINE__
puts "\$mydata '$mydata' @"#__LINE__
set moredata [dict create "Bristol Temple Meads" dunno Bath\ Spa 07:35 Bradford-on-Avon 07:47 Trowbridge 07:53 Westbury 08:00]
textcolor green
puts "moredata='$moredata'"
textcolor
puts "type='[type moredata]' @"#__LINE__


textcolor red
puts "\[dict merge\] ... @"#__LINE__
textcolor

# Merge two dictionaries
textcolor magenta
puts "mydata '$mydata'"
textcolor blue
puts "moredata '$moredata'"
textcolor
#set allway [dict create $mydata $moredata]
set allway [dict merge $mydata $moredata]
textcolor darkyellow
puts "allway is '$allway' @"#__LINE__
textcolor

puts "Line: "#__LINE__
timetable original $allway
puts "type='[type allway]' @"#__LINE__


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
# DICT LAPPEND
textcolor yellow
puts "Calling \[dict lappend\] ... @"#__LINE__
textcolor
 
puts "Line: "#__LINE__
# lappend only adds ONE item - so this has to be 2 separate commands
dict lappend allway Warminster 08:09
dict lappend allway Salisbury 08:32

# [lappend] to non-existent varible
unset foo -nocomplain
dict lappend foo hi there
assert [dict exists foo] {eq $_ 1} -v -line #__LINE__
assert $foo {eq $_ {hi there}} -v -line #__LINE__
unset foo 
assert [dict exists foo] {eq $_ 0} -v -line #__LINE__
  
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
# DICT APPEND

# Move to full example
puts "Line: "#__LINE__
# Add to existing value within the dictionary
dict append allway Westbury -08:01

# Remove an item from the dictionary
set allway [dict remove $allway Pilning]
  
# Replace a value
set allway [dict replace $allway {Bristol Temple Meads} 07:22]

puts "Line: "#__LINE__
timetable modified $allway

############################################################################################
/*
TODO add [dict filter]
stop
# We probably cannot implement [dict filter] as it requires grep
# search by value and return a shorter dict
set before_8 [dict filter $allway value {0[0-7]*} ]
timetable "before 8 a.m." $before_8

# How many items?
puts "Line: "#__LINE__
set sz [dict size $before_8]
puts "There are $sz items in that dict"
*/

unset allway

/*
--------> original
  Cardiff Central 06:28
  Newport 06:42
  Severn Tunnel Junction 06:53
  Pilning 07:03
  Patchway 07:06
  Filton Abbey Wood 07:09
  Bristol Temple Meads dunno
  Bath Spa 07:35
  Bradford-on-Avon 07:47
  Trowbridge 07:53
  Westbury 08:00
  
  ----------> modified
  Cardiff Central 06:28
  Newport 06:42
  Severn Tunnel Junction 06:53
  Patchway 07:06
  Filton Abbey Wood 07:09
  Bristol Temple Meads 07:22
  Bath Spa 07:35
  Bradford-on-Avon 07:47
  Trowbridge 07:53
  Westbury 08:00-08:01
  Warminster 08:09
  Salisbury 08:32
  
  ----------> before 8 a.m.
  Cardiff Central 06:28
  Newport 06:42
  Severn Tunnel Junction 06:53
  Patchway 07:06
  Filton Abbey Wood 07:09
  Bristol Temple Meads 07:22
  Bath Spa 07:35
  Bradford-on-Avon 07:47
  Trowbridge 07:53
  
  There are 9 items in that dict
  wizard:textra graham$ 
 */
############################################################################################
/*
dict set course Tcl Learning 4
  dict set course Tcl Programming 3
  dict set course Perl Learning 5
  dict set course Perl Programming 5
  dict set course Perl Advanced 3
  dict set course Python Learning 4
  dict set course Python Programming 3
  
  puts $course
  
  dict for {language levels} $course {
    puts "$language courses"
    dict with levels {
      puts "--- First level $Learning"
      puts "--- Second level $Programming"
      if {[dict exists $course $language Advanced]} {
        puts "--- Third level $Advanced"
      } else {
        puts "--- Third level not offered"
      }
    }
  }
#*/

puts "Line: "#__LINE__
############################################################################################
# [set <dictvar] etc.
puts "Line: "#__LINE__
unset mydic -nocomplain
set mydic [dict create 4470 {Great Northern} 4471 "Sir Frederic Banbury"]
puts "mydic -> '$mydic' @"#__LINE__
assert $mydic {eq $_ "4470 {Great Northern} 4471 {Sir Frederic Banbury}"} -v -line #__LINE__

puts "Line: "#__LINE__
dict lappend mydic 4472 "Flying Scotsman"
dict walk mydic

# FIX Requires ordred <dict>
puts "Line: "#__LINE__
#assert $mydic {eq $_ "4470 {Great Northern} 4471 {Sir Frederic Banbury}"} -v -line #__LINE__
dict lappend mydic 4473 "Solario"
dict lappend mydic 4474 "Victor Wild"
dict walk mydic
dict lappend mydic 4475 "Flying Fox"
dict lappend mydic 4476 "Royal Lancer"
dict walk mydic

puts "Line: "#__LINE__
assert [dict size $mydic] {== $_ 7} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor darkyellow
puts "mydic -> '$mydic'"
textcolor
newline

puts "Line: "#__LINE__
textcolor magenta
puts "\[set mydic\] (set echo <dict>) -> '[set mydic]' @"#__LINE__
textcolor
newline

option echo on
puts "Line: "#__LINE__
dict keys $mydic
puts "Line: "#__LINE__
set k [dict keys $mydic]
puts "Line: "#__LINE__
textcolor darkyellow
puts "keys-> '$k'"
textcolor
newline

# FIX Requires ordred <dict>
assert $k {eq $_ {4475 4476 4470 4472 4474 4473 4471}} -v -line #__LINE__

puts "Line: "#__LINE__
set v [dict values $mydic]
puts "Line: "#__LINE__
textcolor darkyellow
puts "values-> '$v'"
textcolor
newline

# FIX Requires ordred <dict>
puts "Line: "#__LINE__
assert $v {ne $_ {}} -v -line #__LINE__

puts "Line: "#__LINE__
assert [dict get $mydic 4470] {eq $_ {Great Northern}} -v -line #__LINE__
assert [dict get $mydic 4471] {eq $_ {Sir Frederic Banbury}} -v -line #__LINE__
assert [dict get $mydic 4472] {eq $_ {{Flying Scotsman}}} -v -line #__LINE__	# Double-bracing is correct
assert [dict get $mydic 4473] {eq $_ {Solario}} -v -line #__LINE__

dict walk mydic

puts "Line: "#__LINE__
undef timetable -nocomplain
unset a b i k mydic mydict v -nocomplain
unset course demo surname -nocomplain
unset moredata mydata -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #