###############################################################################################
# File:				test_expand.tcl
# Author:			M Shaw
# Purpose:			Test file for Tcl {*} expand syntax
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 23/Dec/2016	First version. {*} needs more work
# 1.01 21/Jul/2017	Replace [checkex] with [assert]
# 1.02 15/Dec/2017	Added expand string literal
# 1.03 16/Dec/2017	Added [expand] command
# 1.04 29/Jun/2018	Fixed {*} to work within flow-control commands
# 1.05 10/Sep/2018	Added expr tests
# 1.06 25/Sep/2019	Added list join comparisons
# 1.07 21/Nov/2021	A few more tests
###############################################################################################
# Notes
# DONE ADD within flow control
# FIX cannot make eq work here. Requires [unescape]
# Mull over whether might be worth slowing down [eq] by unescaping internally
###############################################################################################

option expression off
option echo on
#option stacktrace on
#option breakpoint on

textcolor yellow
puts "Running test_expand.tcl ..."
textcolor
newline

vars
puts [procs]
array unset a -nocomplain
unset a -nocomplain

textcolor cyan
puts "Line: "#__LINE__
textcolor
halt

# 2022 - Is this the correct usage? We propose early and complete evaluation of the wart tail
# Activestate merely expands the wart tail and stuffs it into the arg accumulator
# Command in variable
set e {info exists}
puts "Line "#__LINE__
puts [expand "$e a"]
puts "Line "#__LINE__
puts [{*}$e a]
set a Hello
puts [expand "$e a"]
assert [expand "$e a"] {== $_ 1} -v -line #__LINE__
puts [eval "$e a"]
assert [eval "$e a"] {== $_ 1} -v -line #__LINE__
unset a e -nocomplain

/*
# (INVALID)
# Complex command in variable 
# Checked with ActiveState. {*} works with list, semi colons not allowed?
puts "Line "#__LINE__
set q -1
set e {puts "In snippet"; set q [/ 22 7.0]}		# ActiveState will not parse this! (expects a list)
# list element in quotes followed by ";" instead of space
eval {*}$e
assert $q {== $_ 3.142857142857143} -v -line #__LINE__
unset e q -nocomplain
#  */

## expand within flow control ##

# /*
textcolor yellow
puts "Testing expand in flow control ..."
textcolor
newline

# Check the code is correct
set square {lmap {x {expr $x * $x}}}
assert [expand "$square 42"] {== $_ 1764} -v -line #__LINE__
assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__

textcolor cyan
puts "Line: "#__LINE__
textcolor
if {1} {
	set square {lmap {x {expr $x * $x}}}
	textcolor magenta
	puts "RESULT:[expand "$square 42"]"
	#puts [ [expand $square] 42]
	textcolor
	assert [expand "$square 42"] {== $_ 1764} -v -line #__LINE__
}
newline

# {*} is parsed by [if], should be delayed ##########################################
# Single level if{}
textcolor cyan
puts "Line: "#__LINE__
textcolor
puts "Line: "#__LINE__
if {1} {					# FAILS
	puts "Line: "#__LINE__
	set square {lmap {x {expr $x * $x}}}
	textcolor magenta
	puts "Line: "#__LINE__
	puts "RESULT:[{*}$square 42]"
	#puts "RESULT:{*}$square 42"
	textcolor
	puts "Line: "#__LINE__
	assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
}
puts "Line: "#__LINE__
newline
# exit

# Nested if{} #######################################################################
textcolor cyan
puts "Line: "#__LINE__
textcolor
if {1} {	# FAILS
	if {2} {
		set square {lmap {x {expr $x * $x}}}
		textcolor magenta
		puts "RESULT:[{*}$square 42]"
		textcolor
		assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
	}
}
newline

# while{} loop #####################################################################
textcolor cyan
puts "Line: "#__LINE__
textcolor
if {1} {
	while {1} {
		set square {lmap {x {expr $x * $x}}}
		textcolor magenta
		puts "RESULT:[{*}$square 42]"
		textcolor
		assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
		break
	}
}
newline


# for{} loop ######################################################################
textcolor cyan
puts "Line: "#__LINE__
textcolor
if {1} {
	for {set i 0} {< $i 1} {++ i} {
		set square {lmap {x {expr $x * $x}}}
		textcolor magenta
		puts "RESULT:[{*}$square 42]"
		textcolor
		assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
		break
	}
}
newline


# [if] within [for ] ##############################################################
for {set i 0} {< $i 1} {++ i} {
	if {1} {
		set square {lmap {x {expr $x * $x}}}
		textcolor magenta
		puts "RESULT:[{*}$square 42]"
		textcolor
		assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
		break
	}
}
newline


#########################################################
# Original tests
#########################################################
# ActiveState a b {[c]} d 1 f {g h} a b {[c]} d 1 f {g h}
textcolor cyan
puts "Line: "#__LINE__
textcolor
#puts "Should be:\ta b \{\[c\]\} d 1 f \{g h\} a b \{\[c\]\} d 1 f \{g h\}"
#puts "$line:\t\t" -nonewline
#puts [list a {*}{b [c]} d {*}{1 f {g h}} a {*}{b [c]} d {*}{1 f {g h}}] 		# Funny things happen if [] are escaped

#puts [list a {*}{b [c]} d {*}{1 f {g h}}]

# FIX double unescaping here. Revisit [assert] unescaping design - 29/Aug/2019
puts "Line: "#__LINE__
echo [escape [list a {*}{b [c]} d {*}{1 f {g h}}]]
puts "Line: "#__LINE__
assert [list a {*}{b [c]} d {*}{1 f {g h}}] {eq $_ [unescape "a b \{\[c\]\} d 1 f \{g h\}"]} -nounescape -v -line #__LINE__
puts "Line: "#__LINE__
echo [escape [list a {*}{b [c]} d {*}{1 f {g h}} a {*}{b [c]} d {*}{1 f {g h}}]]
assert [list a {*}{b [c]} d {*}{1 f {g h}} a {*}{b [c]} d {*}{1 f {g h}}] {eq $_ [unescape "a b {\[c\]} d 1 f {g h} a b {\[c\]} d 1 f {g h}"]} -nounescape -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # #
# Test expand string literal with command
# # # # # # # # # # # # # # # # # # # # # #
set i 0
set s "+= i 23"
{*}$s
assert $i {== $_ 23} -v -line #__LINE__
unset i -nocomplain

# # # # # # # # # # # # # # # # # # # # # #
# Test [expand] command
# # # # # # # # # # # # # # # # # # # # # #

set args [list range foobar 2 4]
puts [string {*}$args]
assert [string {*}$args] {eq $_ oba} -v -line #__LINE__
puts [expand "string $args"]
assert [expand "string $args"] {eq $_ oba} -v -line #__LINE__
puts [eval "string $args"]
assert [eval "string $args"] {eq $_ oba} -v -line #__LINE__
unset args -nocomplain

unset a e -nocomplain


# Lambada funct by variable
set square {lmap {x {expr $x * $x}}}
puts "Line "#__LINE__
puts '[{*}$square 42]'
puts [expand "$square 42"]
assert [expand "$square 42"] {== $_ 1764} -v -line #__LINE__
puts [eval "$square 42"]
assert [eval "$square 42"] {== $_ 1764} -v -line #__LINE__
unset square -nocomplain

# Lambada funct inline
assert [expand [lmap {x {expr $x * $x}} 42]] {== $_ 1764} -v -line #__LINE__
assert [eval [lmap {x {expr $x * $x}} 42]] {== $_ 1764} -v -line #__LINE__

unset s square -nocomplain

###############################################################################################
# FIX listHello' is not recognized
# If {*} followed by dquote then arg gets appended to current argv[argc]
# Detection of {*} in parseBrace leaves parse point +1 too advanced
puts "Test quoted subst expand list {*}\"{\$e f {g h}}\""
set e Hello
puts [list {*}"{$e f {g h}}"]
assert [list {*}"{$e f {g h}}"] {eq $_ "{Hello f {g h}}"} -v -line #__LINE__

assert [dummy [list {*}"{$e f {g h}}" Fred]] {== $_ 2} -v -line #__LINE__
assert [dummy {*}{$e f {g h}} Fred] {== $_ 5} -v -line #__LINE__

unset e -nocomplain
newline

puts "Line: "#__LINE__
textcolor yellow
puts "Test stored \[expr\] ..."
textcolor
unset a -nocomplain

set a {1 + 2 * 3}
assert [expr $a] {== $_ 7} -v -line #__LINE__

set a {expr {1 + 2 * 3}}
puts [{*}$a]					# OK
assert [{*}$a] {== $_ 7} -v -line #__LINE__

puts [expr [{*}$a]]
assert [expr [{*}$a]] {== $_ 7} -v -line #__LINE__

unset a -nocomplain

###############################################################################################
# Compare {*} to other methods of list joining...
unset a b -nocomplain
set a {a b c}; set b {d e f}
assert [list $a $b] {eq $_ {{a b c} {d e f}}} -v -line #__LINE__
assert [concat $a $b] {eq $_ {a b c d e f}} -v -line #__LINE__
assert [list {*}$a {*}$b] {eq $_ {a b c d e f}} -v -line #__LINE__
unset a b -nocomplain

###############################################################################################
# Test simple expansion
###############################################################################################
puts "Line "#__LINE__
set p 10
set q 23
# vars
set test {/ $p $q}
puts "{*}$test"			# Both echo "{*}* $p $q"
puts "Line "#__LINE__
# {*}$test				# ActiveState: 'invalid command name "*"', Ticol = no output
puts "{*}$test"			# Both complain about bad puts args
echo "{*}$test"
puts "Line "#__LINE__
# puts [{*}$test]		# ActiveState returns the correct result, Ticol return 0 (
puts "Line "#__LINE__
puts [subst $test]		# ActivState and Ticol prints '/ 10 23'
puts "Line "#__LINE__
# puts [subst {*}$test]	# INVALID this is malformed. [subst] takes only one non-switch argument
puts "Line "#__LINE__
# puts {*}[subst $test]	# Works OK in ActiveState. Not handled in Ticol since it receives back x3 args
{*}[subst $test]		# No output even with option echo on
puts "Line "#__LINE__
echo {*}[subst $test]	# Safely displays 1st argument "/" (correctly expands the list into argv but requires usage)
puts "Line "#__LINE__
dummy {*}[subst $test]	# Safely displays all expanded arguments
puts "Line "#__LINE__
puts [list {*}[subst $test]]	# Safely displays 1st argument "/"
newline

puts "Line "#__LINE__
assert [list {*}$test] {eq $_ {/ $p $q}} -v -line #__LINE__	# ActiveState validated
assert [list {*}[subst $test]] {eq $_ "/ 10 23"} -v -line #__LINE__	# ActiveState validated
assert [subst $test] {eq $_ "/ 10 23"} -v -line #__LINE__			# ActiveState validated
assert [subst {*}$test] {eq $_ "/"} -v -line #__LINE__				# MALFORMED
# assert {*}[subst $test] {eq $_ "*"} -v -debug -trace -line #__LINE__	# MALFORMED input to [assert]
# assert [list {*}[subst $test]] {eq $_ "*"} -v -line #__LINE__



###############################################################################################
set t [timer]
assert $t {> $_ 0} -v -line #__LINE__
unset t

set t [info tclversion]
assert $t {> $_ 0} -v -line #__LINE__
unset t


resume
option breakpoint off
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #