#############################################################################
# File:				test_file.tcl
# Author:			M Shaw
# Purpose:			Test file for various file commands
# Language:			Ticol Tcl v1.23..v1.26+
#
# Versions:
# 1.00 16/Jan/2017	First version
# 1.01 21/Jul/2017	Replace [checkex] with [assert]
# 1.02 12/Dec/2019	Add pathed [diskfree] test(s)
# 1.03 26/Feb/2022	Added more [file] tests and *nix path handling checks
#############################################################################
# Notes:
# Complete full range of [file] commands for both Win and *nix path types
#############################################################################

textcolor yellow
puts "Testing file related commands ..."
textcolor
newline

# diskfree
textcolor yellow
puts "Testing \[diskfree\]"
textcolor
assert [diskfree c:] {> $_ 0} -v -line #__LINE__

set windir [array item env WINDIR]
if {ne $windir ""} {
	# is_dir
	puts "Checking WINDIR $windir"
	assert [is_dir $windir] -v -line #__LINE__
	assert [diskfree $windir] {> $_ 0} -v -line #__LINE__
}
# FIX [is_dir c:\\] fails
#assert [is_dir $env(WINDIR)] {== $_ 1} -v -line #__LINE__
unset windir

set appdata [array item env APPDATA]
if {ne $appdata ""} {
	puts "Checking APPDATA $appdata"
	assert [is_dir $appdata] -v -line #__LINE__
	assert [diskfree $appdata] {> $_ 0} -v -line #__LINE__
}
unset appdata

# disktype
textcolor yellow
puts "Testing \[disktype\]"
textcolor

# Assume C: is a HDD
assert [disktype C:] {eq $_ 3} -v -line #__LINE__
if {eq [disktype C:] 3} {
	assert [diskfree c:] {> $_ 0} -v -line #__LINE__
}

# eof
textcolor yellow
puts "Testing \[eof\]"
textcolor

# Should be false
assert [eof stdin] {== $_ 0} -v -line #__LINE__		# ActiveState==0

# Should be false
assert [eof stdout] {== $_ 0} -v -line #__LINE__		# ActiveState==0
assert [eof stderr] {== $_ 0} -v -line #__LINE__		# ActiveState==0

# Invalid handle is EOF
assert [eof 0] {== $_ 1} -v -line #__LINE__

# filesize #
textcolor yellow
puts "Checking \[filesize\]"
textcolor
if {[file exists ticol.exe]} {
	assert [filesize ticol.exe] {> $_ 0} -v -line #__LINE__
}


# pwd
option expression off
textcolor yellow
puts "Checking \[pwd\]"
textcolor

if {[ [catch {
	puts "'pwd [escape [pwd]]'" 
	}]] } {
	die "Tests failed at line $line"
}

puts "pwd [escape [pwd]]"

puts '[ls tests.tcl]'

# pwd
textcolor yellow
puts "Testing \[pwd\]"
textcolor
assert [escape [pwd]] {ne $_ ""} -v -line #__LINE__

# FIX after above, cannot find scripts
#puts [escape [pwd]]
textcolor yellow
puts "\nSTART Testing \[ls\] test_*.tcl"
textcolor

#puts [ls test_*.tcl]
assert [ls test_*.tcl] {ne $_ ""} -v -line #__LINE__
textcolor yellow
puts "END Testing \[ls\] test_*.tcl"
textcolor

#############################################################################
# [file]
#############################################################################

textcolor yellow
puts "\nTesting \[file\] commands ..."
textcolor
newline
puts [file exists test_if.tcl]

# [exists]
assert [file exists tests.tcl] {== $_ 1} -v -line #__LINE__

# [isroot]
puts "file isroot {/} [file isroot "/"]"
assert [file isroot "/"] {== $_ $true} -v -line #__LINE__
puts "file isroot {c:/} [file isroot "c:/"]"
assert [file isroot "c:/"] {== $_ $true} -v -line #__LINE__
# puts "file isroot {c:\} [file isroot "c:\"]"	# Not valid
puts "file isroot {c:\\} [file isroot "c:\\"]"
assert [file isroot "c:\\"] {== $_ $true} -v -line #__LINE__

# Local version may have had attribs changed
assert [llength [file attributes tests.tcl]] {> $_ 0} -v -line #__LINE__

# This script
set path [file normalize [info script]]
assert $path {ne $_ ""} -v -line #__LINE__
assert [file writeable $path] {== $_ $true} -v -line #__LINE__

set path "c:\\path\\to\\some\\file.tcl"

textcolor red
puts "FIX file readable $path"
# FAILS
#assert [file readable $path] {== $_ $true} -v -line #__LINE__
textcolor

assert [file extension $path] {[|| [eqi $_ ".tcl"] [eqi $_ ".tcx"]]} -v -line #__LINE__
assert [file dirname $path] {ne $_ ""} -v -line #__LINE__
assert [file pathcount $path] {== $_ 5} -v -line #__LINE__
puts "file split $path = [file split $path]"
assert [lcount [file split $path]] {== $_ 4} -v -line #__LINE__
puts "\[file name $path]=[file name $path]"
assert [file name $path] {eq $_ "file.tcl"} -v -line #__LINE__

newline

if {file exists ticol.exe} {
	puts "\[file type \"ticol.exe\"]=[file type $path]"
	assert [file type "ticol.exe"] {eq $_ "file"} -v -line #__LINE__
	puts "\[file exepath \"ticol.exe\"\]=[escape [file exepath "ticol.exe"]]"
	assert [file exepath "ticol.exe"] {ne $_ ""} -v -line #__LINE__
	puts "\[file runtype \"ticol.exe\"\]=[file runtype "ticol.exe"]"
	assert [file runtype "ticol.exe"] {== $_ 0} -v -line #__LINE__		# 0 == exefile
	assert [file stat "ticol.exe" arr] {== $_ 1} -v -line #__LINE__
	assert [array size arr] {> $_ 0} -v -line #__LINE__
	puts "\[file md5 \"ticol.exe\"\]=[file md5 "ticol.exe"]"
	assert [file md5 "ticol.exe"] {> $_ 0} -v -line #__LINE__
	assert [string length [file md5 "ticol.exe"]] {== $_ 32} -v -line #__LINE__
	array unset arr -nocomplain
}
newline

# *nix paths
textcolor yellow
puts "Checking *nix paths"
textcolor
newline
set path "c:/path/to/some/file.tcl"
assert [file extension $path] {[|| [eqi $_ ".tcl"] [eqi $_ ".tcx"]]} -v -line #__LINE__
assert [file dirname $path] {ne $_ ""} -v -line #__LINE__
assert [file pathcount $path] {== $_ 5} -v -line #__LINE__
puts "file split $path = [file split $path]"
assert [lcount [file split $path]] {== $_ 4} -v -line #__LINE__
newline

puts "\[file name $path]=[file name $path]"
assert [file name $path] {eq $_ "file.tcl"} -v -line #__LINE__


/*
# [isdirectory] can't test
	resolve
	FAIL executable 
#*/

unset path -nocomplain

#############################################################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #