######################################################################################
# File:			test_funct.tcl
# Author:		M Shaw
# Purpose:		Test for [funct]
# Language:		Ticol Tcl v1.19..v1.23
#
# Versions:
# 1.00 03/Sep/2016	First version
# 1.01 17/Jul/2017	Added more tests
# 1.02 21/Jul/2017	Added full range of tests
# 1.03 07/Nov/2019	Added exception checks to acos, asin etc. now we raise exceptions
# 1.04 21/Feb/2021	Added more descriptive prompts
######################################################################################
# Notes:
######################################################################################
/*
Function Arity 0
----------------
srand                      Initialise the random number generator
rand                       Return double random value from 0 to 1
                           The rand() and srand() functions are not 
						   cryptographically secure, and should not be used
						   to generate one-time passwords or session keys
						   Link to the Windows API using [calldll] instead

Function Arity 1
----------------
++       value             Increment value (this is a prefix-only operator)
--       value             Decrement value (this is a prefix-only operator)
abs      value             Return the absolute value of a number
acos     value             Return the arc cosine of value (arccos)
asin     value             Return the arc sin of value (arcsin)
atan     value             Return the arc tangent of value (arctan)
bool     value             Evaluates a string and returns 1 or 0
ceil     value             Round to the next-highest integer value
cos/cosh value             Return the cosine of value
decr     value             Decrement value
degrees  value             Convert radians to degrees
double   value             Cast value to double
exp      value             Return the exponent of value
fib      value             Return the fibonacci of value
fix      value             Return integer value rounded up
floor    value             Round to the next-lowest integer value
fraction value             Return the fraction part of a real/double
incr     value             Increment value
int      value             Cast value to integer
integral value             Return the integer part of a real/double
log      value             Return the logarithm of value
log10    value             Return the log10 of value
round    value (0 places)  Round value to 0 places using standard rounding
sgn      value             Return the sign as an integer (1, 0, -1)
sin/sinh value             Return the sine of value
sqr      value             Return the square of value
sqrt     value             Return the square-root of value
srand    seed              Seed the random number generator
tan/tanh value             Return the tangent of value
wide     value             Force a value to 64 bit signed integer

Function Arity 2
----------------
div      a b               Functional division of a / b
fmod     a b               Return the remainder of a / b
hypot    a b               Return the hypotenuse of a and b
max      a b               Return the maximum of a and b
min      a b               Return the minimum of a and b
mul      a b               Functional multiplication of a * b
pow      a b               Return a to the power of b
rnd      a b               Return a random number between a and b inclusive
round    a b (b places)    Round a to b decimal places
*/
######################################################################################

textcolor yellow
puts "Testing \[funct\] ..."
textcolor
newline

unset a -nocomplain
array unset a -nocomplain


# Test invalid calls
textcolor yellow
puts "Test invalid call catch (noarg) at line: "#__LINE__
textcolor
if {! [catch {funct} ]}  {								# Invalid arg count
	die "Failed to catch error at line "#__LINE__
}

textcolor yellow
puts "Test invalid call catch (emptyarg) at line: "#__LINE__
textcolor
if {! [catch {funct {}} ]} {  							# Invalid arg count
	die "Failed to catch error at line "#__LINE__
}

textcolor yellow
puts "Test invalid call catch (emptyarg x 2) at line: "#__LINE__
textcolor
if {! [catch {funct {} {}} ]}  {						# Invalid arg count
	die "Failed to catch error at line "#__LINE__
}

textcolor yellow
puts "Test invalid call catch (emptyarg x 3) at line: "#__LINE__
textcolor
if {! [catch {funct {} {} {}} ]}  {						# Invalid arg count
	die "Failed to catch error at line "#__LINE__
}

# Test valid calls # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

# Arity 0
newline
textcolor yellow
puts "Testing \[funct\] (Arity 0) ..."
textcolor
newline
assert [funct srand] {== $_ 0} -v -line #__LINE__
assert [funct rand] {> $_ 0} -v -line #__LINE__

# Arity 1
#assert [funct ++ 1] {> $_ 0} -v -line #__LINE__					# FIX
#assert [funct -- 1] {> $_ 0} -v -line #__LINE__					# FIX
newline
textcolor yellow
puts "Testing \[funct\] (Arity 1)"
textcolor
newline
assert [funct abs -0] {== $_ 0} -v -line #__LINE__								#
assert [funct abs -1.23] {== $_ 1.23} -v -line #__LINE__						#

# Test invalid calls
textcolor yellow
puts "Test invalid calls to asin() at line: "#__LINE__
textcolor
set i 0
try {
	funct acos 3
} catch {
	set i 1
}
assert $i 1 -v -line #__LINE__
#assert [funct acos 3] {== $_ "-1.#IND00000000000"} -v -line #__LINE__			# Now raises an exception

set i 0
try {
	funct acos .5
} catch {
	set i 1
}
assert $i 1 -v -line #__LINE__
#assert [funct acos .5] {== $_ 1.047197551196598} -v -line #__LINE__			# Now raises an exception

set i 0
try {
	funct asin 3
} catch {
	set i 1
}
assert $i 1 -v -line #__LINE__

#assert [funct asin 3] {== $_ "-1.#IND00000000000"} -v -line #__LINE__			# Now raises an exception
assert [funct asin .5] {== $_ 0.523598775598299} -v -line #__LINE__			#
assert [funct atan 0] {== $_ 0} -v -line #__LINE__								#
assert [funct atan -3] {== $_ -1.249045772398254} -v -line #__LINE__			#
assert [funct atan 3] {== $_ 1.249045772398254} -v -line #__LINE__			#
assert [funct atan .5] {== $_ 0.463647609000806} -v -line #__LINE__			#
assert [* [funct atan 1] 4] {== $_ 3.141592653589792} -v -line #__LINE__		#
assert [funct bool 0] {== $_ 0} -v -line #__LINE__								#
assert [funct bool 1] {== $_ 1} -v -line #__LINE__								#
assert [funct bool true] {== $_ 1} -v -line #__LINE__							#
assert [funct bool false] {== $_ 0} -v -line #__LINE__							#
assert [funct bool foo] {== $_ 0} -v -line #__LINE__							#
assert [funct bool foo] {!= $_ 1} -v -line #__LINE__							#
assert [funct bool ""] {== $_ 0} -v -line #__LINE__								#
assert [funct bool {}] {== $_ 0} -v -line #__LINE__								#
assert [funct ceil 0] {== $_ 0} -v -line #__LINE__								#
assert [funct ceil .1] {== $_ 1} -v -line #__LINE__								#
assert [funct ceil .5] {== $_ 1} -v -line #__LINE__								#
assert [funct ceil .7] {== $_ 1} -v -line #__LINE__								#
assert [funct ceil 1] {== $_ 1} -v -line #__LINE__								#
assert [funct ceil 1.1] {== $_ 2} -v -line #__LINE__							#
assert [funct ceil -.1] {== $_ 0} -v -line #__LINE__							#
assert [funct ceil -.5] {== $_ 0} -v -line #__LINE__							#
assert [funct ceil -.7] {== $_ 0} -v -line #__LINE__							#
assert [funct ceil -1] {== $_ -1} -v -line #__LINE__							#
assert [funct ceil -1.1] {== $_ -1} -v -line #__LINE__							#
assert [funct cos 0] {== $_ 1} -v -line #__LINE__								#
assert [funct cos 3] {== $_ -0.989992496600445} -v -line #__LINE__			#
assert [funct cosh 0] {== $_ 1} -v -line #__LINE__								#
assert [funct cosh 3] {== $_ 10.067661995777765} -v -line #__LINE__			#
assert [funct decr 2] {== $_ 1} -v -line #__LINE__								#
set a 2
assert [funct decr $a] {== $_ 1} -v -line #__LINE__								#
assert [funct degrees 0] {== $_ 0} -v -line #__LINE__							#
assert [funct degrees 23] {== $_ 1317.802928838559000} -v -line #__LINE__		#
assert [funct double 23] {eq $_ 23.000000000000000} -v -line #__LINE__		#
assert [funct exp 0] {== $_ 1} -v -line #__LINE__								#
assert [funct exp 1] {== $_ 2.718281828459045} -v -line #__LINE__				#
assert [round [funct exp 10] 5] {== $_ [round 22026.465794806725000 5]} -v -line #__LINE__		#
assert [funct fib 0] {== $_ 0} -v -line #__LINE__								#
assert [funct fib 1] {== $_ 1} -v -line #__LINE__								#
assert [funct fib 3] {== $_ 2} -v -line #__LINE__								#
assert [funct fib 23] {== $_ 28657} -v -line #__LINE__							#
assert [funct fib 77] {== $_ 5527939700884757} -v -line #__LINE__				#
assert [funct fix 0] {== $_ 0} -v -line #__LINE__								#
assert [funct fix $pi] {== $_ 3} -v -line #__LINE__								#
assert [funct floor 0] {== $_ 0} -v -line #__LINE__								#
assert [funct floor $pi] {== $_ 3} -v -line #__LINE__							#
assert [funct fraction 0] {== $_ 0} -v -line #__LINE__							#
assert [funct fraction $pi] {== $_ 0.141592653589793} -v -line #__LINE__		#
assert [funct incr 1] {== $_ 2} -v -line #__LINE__								#
set a 1
assert [funct incr $a] {== $_ 2} -v -line #__LINE__								#
assert [funct int $pi] {== $_ 3} -v -line #__LINE__								#
assert [funct integral $pi] {== $_ 3} -v -line #__LINE__						#
if {gt [info winver -vernum] 5.0} {
	assert [funct log 0] {eq $_ "-1.#INF00000000000"} -v -line #__LINE__		# NT version differs
}
assert [funct log 10] {== $_ 2.302585092994046} -v -line #__LINE__			#
assert [funct log $pi] {== $_ 1.144729885849400} -v -line #__LINE__			#
if {gt [info winver -vernum] 5.0} {
	assert [funct log10 0] {eq $_ "-1.#INF00000000000"} -v -line #__LINE__			#
}
assert [funct log10 2] {== $_ 0.301029995663981} -v -line #__LINE__			#
assert [funct log10 10] {== $_ 1} -v -line #__LINE__							#
assert [funct log10 $pi] {== $_ 0.497149872694134} -v -line #__LINE__			#
assert [funct round $pi] {== $_ 3} -v -line #__LINE__							# Round to 0 places
assert [funct sgn 0] {== $_ 0} -v -line #__LINE__								# VB Sgn()
assert [funct sgn 12] {== $_ 1} -v -line #__LINE__								# VB Sgn()
assert [funct sgn -2.4] {== $_ -1} -v -line #__LINE__							# VB Sgn()
assert [funct sinh 0] {== $_ 0} -v -line #__LINE__								#
assert [funct sinh 3] {== $_ 10.017874927409903} -v -line #__LINE__			#
assert [funct sqr 0] {== $_ 0} -v -line #__LINE__								#
assert [funct sqr $pi] {== $_ 9.869604401089358} -v -line #__LINE__			#
assert [funct sqrt 0] {== $_ 0} -v -line #__LINE__								#
assert [funct sqrt $pi] {== $_ 1.772453850905516} -v -line #__LINE__			#
assert [funct tan 0] {== $_ 0} -v -line #__LINE__								#
assert [funct tan -3] {== $_ 0.142546543074278} -v -line #__LINE__			#
assert [funct tan 3] {== $_ -0.142546543074278} -v -line #__LINE__			#
assert [funct tan .5] {== $_ 0.546302489843790} -v -line #__LINE__			#
assert [funct tanh 0] {== $_ 0} -v -line #__LINE__								#
assert [funct tanh -3] {== $_ -0.995054753686730} -v -line #__LINE__			#
assert [funct tanh 3] {== $_ 0.995054753686730} -v -line #__LINE__			#
assert [funct tanh .5] {== $_ 0.462117157260010} -v -line #__LINE__			#


# Arity 2
newline
textcolor yellow
puts "Testing \[funct\] (Arity 2)"
textcolor
newline
assert [funct div $pi 0] {== $_ 0} -v -line #__LINE__							# Safe div
assert [funct div $pi 3] {== $_ 1.047197551196598} -v -line #__LINE__			#
assert [funct hypot 0 0] {== $_ 0} -v -line #__LINE__							# 
assert [funct hypot 3 4] {== $_ 5} -v -line #__LINE__							# 
assert [funct max 3 4] {== $_ 4} -v -line #__LINE__								# 
assert [funct min 3 4] {== $_ 3} -v -line #__LINE__								# 
assert [funct mul 3 4] {== $_ 12} -v -line #__LINE__							# 
assert [funct pow 3 4] {== $_ 81} -v -line #__LINE__							# 
assert [funct rnd 3 4] {|| [== $_ 3] [== $_ 4]} -v -line #__LINE__							# 
assert [funct round $pi 2] {== $_ 3.14} -v -line #__LINE__

unset a i -nocomplain

#####################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF # 

