#################################################################################
# File: 			test_goto.tcl
# Purpose:			Tests for [goto_block]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 07/Dec/2016	First version
# 1.01 10/Dec/2016	Added brace-free block
# 1.02 03/Jun/2017	Repaired 'goto label' and added tests
#					Changed test routine from [checkex] to [assert]
# 1.02 06/Sep/2018	Added goto and gosub call of proc with assert
# 1.03 29/Jul/2019	Pre-undef procs so file can be called in a loop
# 1.04 15/Jan/2021	Addded more tests
#################################################################################
# Notes:  Called from tests.tcl
# goto/gosub tests - full list:
# test_gosub.tcl
# test_goto.tcl
# test_goto_block.tcl
# test_goto_gosub.tcl
# test_goto_return.tcl
#################################################################################
option echo on
option expression off

puts "Line: "#__LINE__
catch {stop}			# Should not exit
catch {exit}			# Should not exit
catch {return}			# Should not exit
catch {break}			# Should not exit

/* #define NDEBUG */

#ifdef NDEBUG
	die "ndebug is defined in [info script]"
#endif

#################################################################################
# Sequential block execution with return test
puts "Line: "#__LINE__
set i 0
goto_block {
	10 {
		+= i 1
	}
	30 {
		+= i 2
	}
	40 {
		+= i 4
		return
	}
	50 {
		+= i 8
	}
}
assert $i {== $_ 7} -v -line #__LINE__

#################################################################################
# Will execute all blocks after (any jump)
puts "Line: "#__LINE__
set var ""
puts "Line: "#__LINE__
textcolor cyan
goto_block {
	init {                # Initialisation must be within a dummy block
		set var [rnd 1 3]
		#goto $var      # Jump to a [goto] entry and print all entries after
	}
	1 {
		puts "One"
	}
	2 {
		puts "Two"
	}
	3 {
		puts "Three"
	}
	4 {
		set var 4
	}
}

textcolor
#checkex $var {== $x 4}
assert $var {== $_ 4} -v -line #__LINE__
newline

#################################################################################
# goto_block without braces ...
#################################################################################
puts "Line: "#__LINE__
undef one -nocomplain
undef two -nocomplain
undef three -nocomplain
undef four -nocomplain

proc one {} {
	puts "One"
}

proc two {} {
	puts "Two"
}

proc three {} {
	puts "Three"
}

proc four {} {
	puts "Four"
}
newline

puts "Line: "#__LINE__
textcolor magenta
goto_block {
	init {                	# Initialisation must be within a dummy block
		set var [rnd 1 4]
		goto $var      		# Jump to a [goto] entry and print all entries after
	}
	1 one					# Unbraced, simple (naked) commands
	2 two
	3 three
	4 four
}
newline

#################################################################################
puts "Line: "#__LINE__
textcolor darkyellow
# Execute one selected block then [break]
goto_block {
	0 {                # Initialisation must be within a dummy block
		set var [rnd 1 3]
		goto $var      # Jump to a [goto] entry
	}
	1 {
		puts "One"
		break;
	}
	2 {
		puts "Two"
		break;
	}
	3 {
		puts "Three"
		break;
	}
}
textcolor

#################################################################################
puts "Line: "#__LINE__
puts "var is $var"
#checkex [&& [> $var 0] [< $var 4]] {== $x 1}
assert [&& [> $var 0] [< $var 4]] {== $_ 1} -v -line #__LINE__

#################################################################################
puts "Line: "#__LINE__
set s 1
goto_block {
	# goto_block used to comment out a block of code
	1 {goto end}		# Broken
	2 {set s 0}
	end {return}
}
#checkex $s {== $x 1}
assert $s {== $_ 1} -v -line #__LINE__
newline

#################################################################################
puts "Line: "#__LINE__
set s 1
goto_block {
	# goto_block used to comment out a block of code
	goto end
	2 {set s 0}
	end return			# Unbraced form
}
#checkex $s {== $x 1}
assert $s {== $_ 1} -v -line #__LINE__
newline

#################################################################################
puts "Line: "#__LINE__
set s 1
goto_block {
	# goto_block used to comment out a block of code
	goto end
	2 {set s 0}
	end {}
}
#checkex $s {== $x 1}
assert $s {== $_ 1} -v -line #__LINE__
newline

#################################################################################
puts "Line: "#__LINE__
set s 1
goto_block {
	# goto_block used to comment out a block of code
	goto end
	2 {set s 0}
	end break		# Unbraced form
}
#checkex $s {== $x 1}
assert $s {== $_ 1} -v -line #__LINE__
newline

#################################################################################
puts "Line: "#__LINE__
set s 1
goto_block {
	# goto_block used to comment out a block of code
	goto end
	2 {set s 0}
	end {break}		# Braced form
}
#checkex $s {== $x 1}
assert $s {== $_ 1} -v -line #__LINE__
newline

#################################################################################
puts "Line: "#__LINE__
option expression off
set s 0
goto_block {
	start {
		++ s
		if {> $s 9} {
			goto end
		}
		goto start
	}
	#	goto start
	end {
		textcolor magenta
		puts "\[goto_block\] exited OK at label :end @line:"#__LINE__
		textcolor
	}
}
#checkex $s {== $x 10}
assert $s {== $_ 10} -v -line #__LINE__
newline

#################################################################################
# DONE FIX should [goto] or stack pointer set $this?
#################################################################################
goto_block {
	one {
		puts "this is $this"
		assert $this {eq $_ one} -v -line #__LINE__
	}
	two {
		puts "this is $this"
		assert $this {eq $_ two} -v -line #__LINE__
	}
	goto four
	three {puts "this is $this"}
	four  {
		puts "this is $this"
		assert $this {eq $_ four} -v -line #__LINE__
	}
}

#################################################################################
newline
textcolor yellow
puts "Testing proc call from goto with \$this"
textcolor
set i 0
set j 0
undef foo -nocomplain

proc foo {} {
	upvar i
	puts "Called by $::this"
	if {== $i 1} {
		assert $::this {eq $_ one} -v -line #__LINE__
	} elseif {== $i 2} {
		assert $::this {eq $_ two} -v -line #__LINE__
	} elseif {== $i 3} {
		assert $::this {eq $_ three} -v -line #__LINE__
	} else {
		die "Unexpected value $i at line "#__LINE__
	}
}

goto_block {
	one {
		set i 1
		+= j 2
		foo
	}
	goto three
	two {
		set i 2
		+= j 4
		foo
	}
	three  {
		set i 3
		+= j 8
		foo
	}
}
assert $j {eq $_ 10} -v -line #__LINE__

#################################################################################
newline
textcolor yellow
puts "Testing proc call from gosub with \$this"
textcolor

set i 0
set j 0
goto_block {
	goto skip
	one {
		set i 1
		+= j 2
		foo
		return -code ok
	}
	two {
		set i 2
		+= j 4
		foo
		return -code ok
	}
	three  {
		set i 3
		+= j 8
		foo
		return -code ok		
	}
	skip {}
	gosub one
	gosub three
}
assert $j {eq $_ 10} -v -line #__LINE__

#################################################################################
# Random jumps with return block exit
#################################################################################
set i 0
goto_block {
	init {
		puts "In label init"
		set i 1
		goto run
	}
	10 {
		puts "In label 10"
		+= i 8}
	20 {
		puts "In label 20"
		+= i 16
		goto end
	}
	30 {
		puts "In label 30"
		+= i 2
	}
	40 {
		puts "In label 40"
		+= i 4
		goto 10
	}
	run {
		puts "In label run"
		goto 30
	}
	end {
		puts "In label end"
		puts "i is $i"
		return		# Ensure that return works (exit will exit the script)
	}
	
	dummy {			# Ensure that exit works (if not then will fall thru and execute)
		+= i 32
	}
}
assert $i {eq $_ 31} -v -line #__LINE__

#################################################################################
# Jump backwards and then return from the block
#################################################################################
set i 0
goto_block {
	#init {goto 60}
	goto 60			# Raw (unbraced) goto
	10 {
		puts "In label 10"
		+= i 1
		return
	}
	20 {
		puts "In label 20"
		+= i 2
		goto 10
	}
	30 {
		puts "In label 30"
		+= i 4
		goto 20
	}
	40 {
		puts "In label 40"
		+= i 8
		goto 30
	}
	50 {
		puts "In label 50"
		+= i 16
		goto 40
	}
	60 {
		puts "In label 60"
		+= i 32
		goto 50
	}
}
assert $i {== $_ 63} -v -line #__LINE__

#################################################################################
# Nested, within braced arguments, confirming that label scope is local to each
# nested goto (no clashes)
#################################################################################

set i 0
goto_block {
	#init {goto 20}
	goto 20			# Raw (unbraced) goto
	10 {
		puts "In outer 10"
		+= i 1
		goto 30
	}
	20 {
		puts "In outer 20"
		+= i 2
		goto 10
	}
	30 {
		puts "In outer 30"
	}
	40 {
		goto_block {
			init {
				puts "In inner 10"
				goto 20}
			10 {
				+= i 4
				break
			}
			20 {
				+= i 8
				goto 10
			}
		}
	}
}
puts $i
assert $i {== $_ 15} -v -line #__LINE__

#################################################################################
# Loop using goto
set i 0
goto_block {
	goto 10		# Not nesc. but why not?
	10 {
		++ i
		if {>= $i 1000} {
			puts "Break at line 10 "#__LINE__
			break		# This will exit the goto block not the sub clause
		}
		goto 10
	}
	20 {	# Should not execute
		puts "In line 20 at "#__LINE__
		die
	}
}
puts "i is $i"
assert $i {== $_ 1000} -v -line #__LINE__

#################################################################################
# Clean up
undef foo -nocomplain
unset i j s this var -nocomplain
undef one -nocomplain
undef two -nocomplain
undef three -nocomplain
undef four -nocomplain

vars

#################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #