###############################################################################################
# File:				test_goto_exit_loops.tcl
# Author:			M Shaw
# Purpose:			Test file for safe [goto] jump out of loop constucts
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 08/Dec/2019	First version
# 1.01 17/Dec/2021	Bugfix for regression change in [carray create]
# 1.02 07/Jun/2022	Revised for changes to [loop] bounds
# 1.03 27/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
###############################################################################################
# Notes:
# Note: Vars declared within [goto_block] clauses are visible to the whole [goto_block],
# which means other clauses. There is no need to re-declare from clause to clause
#
# Tests:
# [while], [for], [loop], [foreach], [array foreach], [do], [stack foreach], [carray foreach],
# [linkedlist foreach], [varray foreach]
#
# The Ticol loop constructs will not exit unless the correct condition code is encountered
# [goto] -> TICOL_GOTO will safely terminate these loops and cascade the operation back to the
# [goto_block] command
###############################################################################################

option expression off

goto_block {

	###############################################################################################
	# [while] #

	10	{
		set i 0
		while {< $i 100} {
			puts "Loop $i in block $this (while)"
			if {>= $i 10} {goto 20}
			++ i
		}
	}

	20 {
		assert $i {== $_ 10} -v -line #__LINE__
		puts "In block $this"
		set i 0
		while {< $i 100} {
			++ i
		}
		assert $i {== $_ 100} -v -line #__LINE__
	}

	###############################################################################################
	# [for] #

	30 {
		for {set i 0} {< $i 100} {++ i} {
			puts "Loop $i in block $this (for)"
			if {>= $i 10} {goto 40}
		}
	}

	40 {
		assert $i {== $_ 10} -v -line #__LINE__
		puts "In block $this"
		set i 0
		for {set i 0} {< $i 100} {++ i} {
		}
		assert $i {== $_ 100} -v -line #__LINE__
	}

	###############################################################################################
	# [loop] #

	50 {
		loop i 0 100 {
			puts "Loop $i in block $this (loop)"
			if {>= $i 10} {goto 60}
		}
	}

	60 {
		assert $i {== $_ 10} -v -line #__LINE__
		puts "In block $this"
		loop i 0 100 {		# 0..99
		
		}
		assert $i {== $_ 100} -v -line #__LINE__
	}

	###############################################################################################
	# [foreach] #

	70 {
		set i 0
		unset x -nocomplain
		foreach x $qbf {
			puts "Loop $i in block $this (foreach) -> '$x'"
			if {>= $i 5} {goto 80}
			puts "Loop $this incrementing"
			++ i
		}
	}

	80 {
		assert $i {== $_ 5} -v -line #__LINE__
		puts "In block $this"
		set i 0
		unset x -nocomplain
		foreach x $qbf {
			++ i
		}
		assert $i {== $_ 9} -v -line #__LINE__
	}

	###############################################################################################
	# [array foreach] #

	90 {
		set s [split $qbf " " -array a]
		set i 0
		unset x -nocomplain
		array foreach a x {
			puts "Loop $i in block $this (array foreach) -> '$x'"
			if {>= $i 5} {goto 100}
			puts "Loop $this incrementing"
			++ i
		}
	}

	100 {
		assert $i {== $_ 5} -v -line #__LINE__
		puts "In block $this"
		set i 0
		unset x -nocomplain
		array foreach a x {
			++ i
		}
		assert $i {== $_ 9} -v -line #__LINE__
	}

	###############################################################################################
	# [do] #

	110	{
		set i 0
		do {
			puts "Loop $i in block $this (do)"
			if {>= $i 10} {goto 120}
			++ i
		} while {< $i 100}
	}

	120 {
		assert $i {== $_ 10} -v -line #__LINE__
		puts "In block $this"
		set i 0
		do {
			++ i
		} while {< $i 100}
		assert $i {== $_ 100} -v -line #__LINE__
	}

	###############################################################################################
	# [stack foreach] #

	130	{
		stack create stk 20
		unset x -nocomplain
		foreach x $qbf {								# Populate the object with qbf words
			stack push stk $x
		}
		assert [stack count stk] {== $_ 9} -v -line #__LINE__
		set i 0
		unset x -nocomplain
		stack foreach stk x {
			puts "Loop $i in block $this (stack foreach)"
			if {>= $i 5} {goto 140}
			++ i
		}
	}

	140 {
		assert $i {== $_ 5} -v -line #__LINE__
		puts "In block $this"
		set i 0
		unset x -nocomplain
		stack foreach stk x {
			++ i
		}
		assert $i {== $_ 9} -v -line #__LINE__
		stack unset stk
	}

	###############################################################################################
	# [carray foreach] #

	150 {
		lib ticol_carray
		set h [carray create]	# Don't specify a size or we will ADD to this count
		set i 0
		unset x -nocomplain
		foreach x $qbf {								# Populate the object
			carray add $h $i $x							# Index $i -> value
			++ i
		}
		assert [carray count $h] {== $_ 9} -v -line #__LINE__
		unset x -nocomplain
		carray foreach $h x {
			puts "Loop $x in block $this (carray foreach)"
			if {>= $x 5} {goto 160}
		}
	}

	160 {
		assert $x {== $_ 5} -v -line #__LINE__
		unset x -nocomplain
		carray foreach $h x {
			puts "carray foreach: Iter check $x"
		}
		assert $x {== $_ 8} -v -line #__LINE__
		lib ticol_carray -unload
	}

	###############################################################################################
	# [linkedlist foreach] #

	170 {
		lib ticol_list
		set h [linkedlist create 20]
		unset x -nocomplain
		set i 0
		foreach x $qbf {								# Populate the object
			linkedlist add $h $i $x						# Index $i -> value
			++ i
		}
		assert [linkedlist count $h] {== $_ 9} -v -line #__LINE__
		unset x -nocomplain
		linkedlist foreach $h x {
			puts "Loop $x in block $this (linkedlist foreach)"
			if {>= $x 5} {puts "max reached at $x in $this"}
			if {>= $x 5} {goto 180}	
		}
	}

	180 {
		textcolor magenta
		puts "Enter block '$this'"
		textcolor
		assert $x {== $_ 5} -v -line #__LINE__
		unset x -nocomplain
		linkedlist foreach $h x { 
		}
		vars
		assert $x {== $_ 8} -v -line #__LINE__
		unset x -nocomplain
		lib ticol_list -unload
		# assert object no longer exists
	}

	###############################################################################################
	# [varray foreach] #

	190 {
		textcolor magenta
		puts "Enter block '$this'"
		textcolor
		#lib ticol_variant -unload -nocomplain
		lib ticol_variant -nocomplain	# If still loaded in IDE this can cause issues
		set h [varray create 20]
		set i 0
		unset x -nocomplain
		foreach x $qbf {								# Populate the object
			varray set $h $i $x							# Index $i -> value
			++ i
		}
		# DOC amend no create and give tested example since updates
		assert [varray size $h] {== $_ 20} -v -line #__LINE__
		unset x -nocomplain
		set i 0
#puts "Line: "#__LINE__
		varray foreach $h x {
			puts "Loop $i in block $this varray foreach -> '$x'"
			if {>= $i 5} {goto 200}
			++ i
		}	# */
	}
	
	200 {
		textcolor magenta
		puts "Enter block '$this'"
		textcolor
		assert $i {== $_ 5} -v -line #__LINE__
		unset x -nocomplain
		set i 0
		puts "varray size is [varray size $h]"

		varray foreach $h x {						# Varray iters whole declared array not occupied elements			
			++ i
			puts "$i $x"
		}
		assert $i {== $_ 20} -v -line #__LINE__
# */		
		lib ticol_variant -unload -nocomplain
	}

}

array unset a -nocomplain
unset h i s this x -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #