######################################################################################
# File: 			test_lib_carray.tcl
# Purpose:			Test for C/C++ arrays - [lib ticol_carray]
# Language:			Ticol Tcl v1.23+..v1.26
# Lib:				ticol_carray.dll v1.05+
#
# Versions:
# 1.00 12/Jan/2017	First version
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
# 1.01 11/Dec/2019	Added [carray get] updated tests
######################################################################################
# Notes: 
# See also: carray_leaktest.tcl
# Implemented as struct tuples of x2 LPSTR strings than as an array of LPSTR
# This makes [carray] more useful and flexible
######################################################################################
/*
	handle [carray create ?size?]
	integer [carray add handle lval ?rval?]
	carray delete handle index
	carray count handle
	carray get handle index ?-rval? ?-list?
	carray item handle index ?defaultval? ?-rval? ?-list?
	carray foreach handle lvalue ?rvalue? {code}
	carray sort handle ?-reverse?
	carray walk handle
	carray unset handle ?-nocomplain?
*/
######################################################################################

option echo on
option expression off

textcolor cyan
lib ticol_carray -unload -nocomplain
lib ticol_carray -verbose
puts "Library v[lib ticol_carray -info]"
textcolor
newline

#/*

set h [carray create]

puts "DEBUG: handle is $h"

puts "Line: "#__LINE__
set r [carray add $h Hello World]
assert $r {== $_ 1} -v -line #__LINE__
assert [carray count $h] {== $_ 1} -v -line #__LINE__
set r [carray add $h Quick Brown]
assert $r {== $_ 2} -v -line #__LINE__
assert [carray count $h] {== $_ 2} -v -line #__LINE__
set r [carray add $h Fox Jumps]
assert $r {== $_ 3} -v -line #__LINE__
assert [carray count $h] {== $_ 3} -v -line #__LINE__
set r [carray add $h Lazy Dog]
assert $r {== $_ 4} -v -line #__LINE__
assert [carray count $h] {== $_ 4} -v -line #__LINE__
set r [carray add $h LValue RValue]
assert $r {== $_ 5} -v -line #__LINE__
assert [carray count $h] {== $_ 5} -v -line #__LINE__
set r [carray add $h {} RValue]
assert $r {== $_ 6} -v -line #__LINE__
assert [carray count $h] {== $_ 6} -v -line #__LINE__


carray sort $h
textcolor darkyellow
carray foreach $h lv rv {
	puts "lvalue:'$lv'\tlvalue:'$rv'"
}
assert [carray count $h] {== $_ 6} -v -line #__LINE__
assert [carray item $h 0] {eq $_ ""} -v -line #__LINE__
assert [carray item $h 1] {eq $_ Fox} -v -line #__LINE__
assert [carray item $h 2] {eq $_ Hello} -v -line #__LINE__
assert [carray item $h 3] {eq $_ LValue} -v -line #__LINE__
assert [carray item $h 3 -rvalue] {eq $_ RValue} -v -line #__LINE__
assert [carray item $h 4] {eq $_ Lazy} -v -line #__LINE__
assert [carray item $h 5] {eq $_ Quick} -v -line #__LINE__

# Ensure we return "" when using [carray item]
assert [carray item $h 999] {eq $_ {}} -v -line #__LINE__
# Ensure we return a specified default when using [carray item {default}]
assert [carray item $h 999 foo] {eq $_ foo} -v -line #__LINE__

# Ensure we raise an error when using [carray get]
set i 0
try {
	carray get $h 999
} catch {
	set i 1
}
assert $i  -v -line #__LINE__

puts "Line: "#__LINE__
textcolor
newline
carray delete $h 2
textcolor darkyellow
carray foreach $h lv rv {
	puts "lvalue:'$lv'\tlvalue:'$rv'"
}
assert [carray count $h] {== $_ 5} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor
newline
carray sort $h -reverse
textcolor darkyellow
carray foreach $h lv rv {
	puts "lvalue:'$lv'\tlvalue:'$rv'"
}
assert [carray item $h 0] {eq $_ Quick} -v -line #__LINE__
assert [carray item $h 1] {eq $_ Lazy} -v -line #__LINE__
assert [carray item $h 2] {eq $_ LValue} -v -line #__LINE__
assert [carray item $h 3] {eq $_ Fox} -v -line #__LINE__
assert [carray item $h 4] {eq $_ ""} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor
newline
set count [carray count $h]
textcolor darkyellow
for {set i 0} {[< $i $count]} {++ i} {
	puts "item $i: '[carray item $h $i]'\t'[carray item $h $i -rvalue]' '[carray item $h $i -list]'"
}
textcolor

######################################################################################
# Deletion
# carrays are not 'sparse' so they will resize downwards and remove any intermediate
# records will be removed. Thus record ordering will change on deletion
puts "Line: "#__LINE__
puts "Record deletion test. Start count is [carray count $h]"
set i 0
/*
# Correct method
puts "Delete records using \[while\] ..."
while {[> [carray count $h] 0]} {
	carray delete $h 0
} # */

puts "Delete records in reverse order ..."
for {set i [- [carray count $h] 1]} {>= $i 0} {-- i} {
	puts "Deleting record # $i"
	carray delete $h $i
} # */

/*
# This method won't work
set count [carray count $h]
for {set i 0} {< $i $count} {++ i} {
	puts "Delete $i of $count"
	carray delete $h $i
} # */

/*
# This method won't work either
# Resultant record count is 2 of 5
for {set i 0} {[< $i [carray count $h]]} {++ i} {
	puts "Deleting record # $i"
	carray delete $h $i
} # */

#carray add $h Misty "Green and Blue"
puts "Line: "#__LINE__
puts "Count is now [carray count $h]"
assert [carray count $h] {== $_ 0} -v -line #__LINE__

newline
textcolor darkyellow
carray foreach $h lv rv {
	puts "lvalue:'$lv'\tlvalue:'$rv'"
}
textcolor



#carray walk $h 
#carray unset 12345
carray unset $h		# Dies
#*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

lib ticol_carray -unload

unset count h i lv r rv -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #