######################################################################################
# File: 			test_lib_numword.tcl
# Purpose:			Test [integer_to_words]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 24/Oct/2019	First version with conversion each way
# 1.01 15/Nov/2019	Disabled NT4 as test VM runs on QEMU which appears to have FP
#					math issues at float number borderlines
######################################################################################
# Notes:
# Loop tested 0 .. 5012 0000
# Round trip performance 10000 per second on i5
# Internal code is limited by 64 bit integers. If we can support 128 bit we can expand
# ADD -noand tests
# DONE ADD -hyphenate tests when implemented
# Sextillion, Septillion, Octillion, Nonillion, Decillion
# https://en.wikipedia.org/wiki/Names_of_large_numbers
# https://www.calculatorsoup.com/calculators/conversions/numberstowords.php
######################################################################################

if {< [info winver -vernum] 5.0} {
	textcolor red
	puts "Not running in NT4 as test platform runs on QEMU which has some FP issues"
	textcolor
	return 
}

######################################################################################
/*
nine quintillion 
two hundred and twenty three quadrillion 
three hundred and seventy two trillion 
thirty six billion 
eight hundred and fifty four million 
seven hundred and seventy five thousand 
eight hundred and seven 	# */

unset INT_MIN INT_MAX UNSIGNED_MIN UNSIGNED_MAX -const -nocomplain

const INT_MIN   -9223372036854775808
const INT_MAX    9223372036854775807	nine quintillion two hundred twenty-three quadrillion three hundred seventy-two trillion thirty-six billion eight hundred fifty-four million seven hundred seventy-five thousand eight hundred seven
const UNSIGNED_MIN 0
const UNSIGNED_MAX 18446744073709551615

textcolor cyan
lib ticol_numword -verbose
puts [lib ticol_numword -info]
textcolor

/*
# 7060000
loop i 50120000 1000000001 {
	if {is_mod $i 10000} {puts $i}
	#integer_to_words $i		# No leaks
	# No leaks ...
	#words_to_integer {one billion nine hundred and ninety nine million nine hundred and ninety nine thousand nine hundred and ninety nine}
	assert [words_to_integer [integer_to_words $i]] {== $_ $i} -line #__LINE__
} # */

######################################################################################
newline
textcolor yellow
puts "Converting from words to integer using \[words_to_integer\] ..."
textcolor
newline

# Positive
assert [words_to_integer one hundred and twenty three] {== $_ 123} -v -line #__LINE__
assert [words_to_integer nine hundred and eighty six] {== $_ 986} -v -line #__LINE__
assert [words_to_integer two thousand one hundred and twenty three] {== $_ 2123} -v -line #__LINE__
assert [words_to_integer ninety two thousand one hundred and twenty three] {== $_ 92123} -v -line #__LINE__
assert [words_to_integer one hundred and ninety two thousand one hundred and twenty three] {== $_ 192123} -v -line #__LINE__
assert [words_to_integer two million one hundred and ninety two thousand one hundred and twenty three] {== $_ 2192123} -v -line #__LINE__
assert [words_to_integer nine hundred and one million one hundred and ninety two thousand one hundred and twenty three] {== $_ 901192123} -v -line #__LINE__
assert [words_to_integer four billion nine hundred and one million one hundred and ninety two thousand one hundred and twenty three] {== $_ 4901192123} -v -line #__LINE__
assert [words_to_integer one billion two hundred and thirty-four million five hundred and sixty-seven thousand eight hundred and ninety] {== $_ 1234567890} -v -line #__LINE__
#9223371892713013247
# nine quintillion 
# two hundred and twenty three quadrillion 
# three hundred and seventy two trillion 
# thirty six billion 
# eight hundred and fifty four million 
# seven hundred and seventy five thousand 
# eight hundred and seven
#		9 223 372 036 854 775 807
# GOT  9 223 371 892 713 013 247
assert [words_to_integer nine quintillion two hundred and twenty three quadrillion three hundred and seventy two trillion thirty six billion eight hundred and fifty four million seven hundred and seventy five thousand eight hundred and seven]  {== $_ $INT_MAX} -v -line #__LINE__

# Negative
assert [words_to_integer minus one hundred and twenty three] {== $_ -123} -v -line #__LINE__
assert [words_to_integer minus nine hundred and eighty six] {== $_ -986} -v -line #__LINE__
assert [words_to_integer minus one billion two hundred and thirty-four million five hundred and sixty-seven thousand eight hundred and ninety] {== $_ -1234567890} -v -line #__LINE__
newline
##exit


textcolor yellow
puts "Converting from integer to words using \[integer_to_words\] ..."
textcolor
newline

######################################################################################
# Test ranges
#                  9,223,372,036,854,775,807
#                  9 223 372 036 854 775 807
# LLONG_MAX              9223372036854775807
#                        9223372036854775807 - INT_MAX
# INT64_MIN             -9223372036854775808
#                  9 292 929 299 929 292 923
                      
# Maxima
assert [integer_to_words $INT_MAX] {eq $_ {nine quintillion two hundred and twenty three quadrillion three hundred and seventy two trillion thirty six billion eight hundred and fifty four million seven hundred and seventy five thousand eight hundred and seven}} -v -line #__LINE__
assert [integer_to_words $INT_MIN] {eq $_ {overflow}} -v -line #__LINE__
# Move +1 closer to zero...
assert [integer_to_words [+ $INT_MIN 1]] {eq $_ {minus nine quintillion two hundred and twenty three quadrillion three hundred and seventy two trillion thirty six billion eight hundred and fifty four million seven hundred and seventy five thousand eight hundred and seven}} -v -line #__LINE__

puts [integer_to_words 9292929299929292923]
assert [integer_to_words 9292929299929292923] {eq $_ overflow} -v -line #__LINE__
# FAILS
/*                     18446744073709551615 */
#puts [integer_to_words -9292929299929292923]
#assert [integer_to_words -9292929299929292923] {eq $_ overflow} -v -line #__LINE__
# OB1
assert [integer_to_words 9223372036854775808] {eq $_ overflow} -v -line #__LINE__
assert [integer_to_words -9223372036854775808] {eq $_ overflow} -v -line #__LINE__	# is OK
#puts [integer_to_words 987654321]
assert [integer_to_words 999999999] {eq $_ {nine hundred and ninety nine million nine hundred and ninety nine thousand nine hundred and ninety nine}} -v -line #__LINE__

######################################################################################
# Random
assert [integer_to_words {}] {eq $_ zero} -v -line #__LINE__
set i 1
assert [integer_to_words $i] {eq $_ one} -v -line #__LINE__
set i 12
assert [integer_to_words $i] {eq $_ twelve} -v -line #__LINE__
set i 123
assert [integer_to_words $i] {eq $_ {one hundred and twenty three}} -v -line #__LINE__
set i 1234
assert [integer_to_words $i] {eq $_ {one thousand two hundred and thirty four}} -v -line #__LINE__
set i 12345
assert [integer_to_words $i] {eq $_ {twelve thousand three hundred and forty five}} -v -line #__LINE__
set i 123456
assert [integer_to_words $i] {eq $_ {one hundred and twenty three thousand four hundred and fifty six}} -v -line #__LINE__
set i 1234567
assert [integer_to_words $i] {eq $_ {one million two hundred and thirty four thousand five hundred and sixty seven}} -v -line #__LINE__
set i 12345678
assert [integer_to_words $i] {eq $_ {twelve million three hundred and forty five thousand six hundred and seventy eight}} -v -line #__LINE__
set i 123456789
assert [integer_to_words $i] {eq $_ {one hundred and twenty three million four hundred and fifty six thousand seven hundred and eighty nine}} -v -line #__LINE__
set i 1234567890
assert [integer_to_words $i] {eq $_ {one billion two hundred and thirty four million five hundred and sixty seven thousand eight hundred and ninety}} -v -line #__LINE__

assert [comma [words_to_integer sixty seven million nine hundred and twenty seven thousand four hundred and ninety one]] {eq $_ "67,927,491"} -v -line #__LINE__

######################################################################################
# Sequences
assert [integer_to_words 0] {eq $_ zero} -v -line #__LINE__
assert [integer_to_words 1] {eq $_ one} -v -line #__LINE__
assert [integer_to_words 2] {eq $_ two} -v -line #__LINE__
assert [integer_to_words 3] {eq $_ three} -v -line #__LINE__
assert [integer_to_words 4] {eq $_ four} -v -line #__LINE__
assert [integer_to_words 5] {eq $_ five} -v -line #__LINE__
assert [integer_to_words 6] {eq $_ six} -v -line #__LINE__
assert [integer_to_words 7] {eq $_ seven} -v -line #__LINE__
assert [integer_to_words 8] {eq $_ eight} -v -line #__LINE__
assert [integer_to_words 9] {eq $_ nine} -v -line #__LINE__
assert [integer_to_words 10] {eq $_ ten} -v -line #__LINE__
assert [integer_to_words 11] {eq $_ eleven} -v -line #__LINE__
assert [integer_to_words 12] {eq $_ twelve} -v -line #__LINE__
assert [integer_to_words 13] {eq $_ thirteen} -v -line #__LINE__
assert [integer_to_words 14] {eq $_ fourteen} -v -line #__LINE__
assert [integer_to_words 15] {eq $_ fifteen} -v -line #__LINE__
assert [integer_to_words 16] {eq $_ sixteen} -v -line #__LINE__
assert [integer_to_words 17] {eq $_ seventeen} -v -line #__LINE__
assert [integer_to_words 18] {eq $_ eighteen} -v -line #__LINE__
assert [integer_to_words 19] {eq $_ nineteen} -v -line #__LINE__
assert [integer_to_words 20] {eq $_ twenty} -v -line #__LINE__
assert [integer_to_words 21] {eq $_ {twenty one}} -v -line #__LINE__
assert [integer_to_words 22] {eq $_ {twenty two}} -v -line #__LINE__
assert [integer_to_words 23] {eq $_ {twenty three}} -v -line #__LINE__
assert [integer_to_words 24] {eq $_ {twenty four}} -v -line #__LINE__
assert [integer_to_words 25] {eq $_ {twenty five}} -v -line #__LINE__
assert [integer_to_words 26] {eq $_ {twenty six}} -v -line #__LINE__
assert [integer_to_words 27] {eq $_ {twenty seven}} -v -line #__LINE__
assert [integer_to_words 28] {eq $_ {twenty eight}} -v -line #__LINE__
assert [integer_to_words 29] {eq $_ {twenty nine}} -v -line #__LINE__
assert [integer_to_words 30] {eq $_ {thirty}} -v -line #__LINE__
assert [integer_to_words 31] {eq $_ {thirty one}} -v -line #__LINE__
assert [integer_to_words 39] {eq $_ {thirty nine}} -v -line #__LINE__
assert [integer_to_words 40] {eq $_ {forty}} -v -line #__LINE__
assert [integer_to_words 41] {eq $_ {forty one}} -v -line #__LINE__
assert [integer_to_words 90] {eq $_ {ninety}} -v -line #__LINE__
assert [integer_to_words 91] {eq $_ {ninety one}} -v -line #__LINE__
assert [integer_to_words 99] {eq $_ {ninety nine}} -v -line #__LINE__
assert [integer_to_words 100] {eq $_ {one hundred}} -v -line #__LINE__
assert [integer_to_words 101] {eq $_ {one hundred and one}} -v -line #__LINE__
assert [integer_to_words 112] {eq $_ {one hundred and twelve}} -v -line #__LINE__
assert [integer_to_words 120] {eq $_ {one hundred and twenty}} -v -line #__LINE__
assert [integer_to_words 121] {eq $_ {one hundred and twenty one}} -v -line #__LINE__
assert [integer_to_words 129] {eq $_ {one hundred and twenty nine}} -v -line #__LINE__
assert [integer_to_words 199] {eq $_ {one hundred and ninety nine}} -v -line #__LINE__
assert [integer_to_words 900] {eq $_ {nine hundred}} -v -line #__LINE__
assert [integer_to_words 901] {eq $_ {nine hundred and one}} -v -line #__LINE__
assert [integer_to_words 920] {eq $_ {nine hundred and twenty}} -v -line #__LINE__
assert [integer_to_words 999] {eq $_ {nine hundred and ninety nine}} -v -line #__LINE__
assert [integer_to_words 1000] {eq $_ {one thousand}} -v -line #__LINE__
assert [integer_to_words 1001] {eq $_ {one thousand and one}} -v -line #__LINE__
assert [integer_to_words 9000] {eq $_ {nine thousand}} -v -line #__LINE__
assert [integer_to_words 9001] {eq $_ {nine thousand and one}} -v -line #__LINE__
assert [integer_to_words 10000] {eq $_ {ten thousand}} -v -line #__LINE__
assert [integer_to_words 10001] {eq $_ {ten thousand and one}} -v -line #__LINE__
assert [integer_to_words 12500] {eq $_ {twelve thousand five hundred}} -v -line #__LINE__
assert [integer_to_words 12345] {eq $_ {twelve thousand three hundred and forty five}} -v -line #__LINE__
assert [integer_to_words 99345] {eq $_ {ninety nine thousand three hundred and forty five}} -v -line #__LINE__
assert [integer_to_words 100000] {eq $_ {one hundred thousand}} -v -line #__LINE__
assert [integer_to_words 100001] {eq $_ {one hundred thousand and one}} -v -line #__LINE__
assert [integer_to_words 100100] {eq $_ {one hundred thousand one hundred}} -v -line #__LINE__
assert [integer_to_words 100101] {eq $_ {one hundred thousand one hundred and one}} -v -line #__LINE__
assert [integer_to_words 199345] {eq $_ {one hundred and ninety nine thousand three hundred and forty five}} -v -line #__LINE__
assert [integer_to_words 999345] {eq $_ {nine hundred and ninety nine thousand three hundred and forty five}} -v -line #__LINE__
assert [integer_to_words 1000000] {eq $_ {one million}} -v -line #__LINE__
assert [integer_to_words 1000001] {eq $_ {one million and one}} -v -line #__LINE__
assert [integer_to_words 1000010] {eq $_ {one million and ten}} -v -line #__LINE__
assert [integer_to_words 1000100] {eq $_ {one million one hundred}} -v -line #__LINE__
assert [integer_to_words 1001000] {eq $_ {one million one thousand}} -v -line #__LINE__
assert [integer_to_words 1010000] {eq $_ {one million ten thousand}} -v -line #__LINE__
assert [integer_to_words 1100000] {eq $_ {one million one hundred thousand}} -v -line #__LINE__
assert [integer_to_words 900100000] {eq $_ {nine hundred million one hundred thousand}} -v -line #__LINE__
assert [integer_to_words 910000000] {eq $_ {nine hundred and ten million}} -v -line #__LINE__
assert [integer_to_words 999999999] {eq $_ {nine hundred and ninety nine million nine hundred and ninety nine thousand nine hundred and ninety nine}} -v -line #__LINE__
assert [integer_to_words 1000000000] {eq $_ {one billion}} -v -line #__LINE__
assert [integer_to_words 1999999999] {eq $_ {one billion nine hundred and ninety nine million nine hundred and ninety nine thousand nine hundred and ninety nine}} -v -line #__LINE__
assert [integer_to_words 1000000000000] {eq $_ {one trillion}} -v -line #__LINE__
assert [integer_to_words 1000000000000000] {eq $_ {one quadrillion}} -v -line #__LINE__
assert [integer_to_words 1000000000000000000] {eq $_ {one quintillion}} -v -line #__LINE__
assert [integer_to_words 9000000000000000000] {eq $_ {nine quintillion}} -v -line #__LINE__

# const INT_MAX    9223372036854775807	nine quintillion two hundred twenty-three quadrillion
# FAIL OVERFLOW PASS u:6766279631452241920 s:6766279631452241920 (should overflow)
assert [integer_to_words  99000000000000000000] {eq $_ {overflow}} -v -line #__LINE__
# FAIl OVERFLOW PASS OVERFLOW PASS u:2875820019684212736 s:2875820019684212736 (should overflow)
assert [integer_to_words 999000000000000000000] {eq $_ {overflow}} -v -line #__LINE__

######################################################################################
# Hyphenated
set i 123
assert [integer_to_words $i -hyphen] {eq $_ {one hundred and twenty-three}} -v -line #__LINE__
set i 1234
assert [integer_to_words $i -hyphen] {eq $_ {one thousand two hundred and thirty-four}} -v -line #__LINE__
set i 12345
assert [integer_to_words $i -hyphen] {eq $_ {twelve thousand three hundred and forty-five}} -v -line #__LINE__
set i 123456
assert [integer_to_words $i -hyphen] {eq $_ {one hundred and twenty-three thousand four hundred and fifty-six}} -v -line #__LINE__
set i 1234567
assert [integer_to_words $i -hyphen] {eq $_ {one million two hundred and thirty-four thousand five hundred and sixty-seven}} -v -line #__LINE__
set i 12345678
assert [integer_to_words $i -hyphen] {eq $_ {twelve million three hundred and forty-five thousand six hundred and seventy-eight}} -v -line #__LINE__
set i 123456789
assert [integer_to_words $i -hyphen] {eq $_ {one hundred and twenty-three million four hundred and fifty-six thousand seven hundred and eighty-nine}} -v -line #__LINE__
set i 1234567890
assert [integer_to_words $i -hyphen] {eq $_ {one billion two hundred and thirty-four million five hundred and sixty-seven thousand eight hundred and ninety}} -v -line #__LINE__

######################################################################################
# Negative numbers
assert [integer_to_words -0] {eq $_ {zero}} -v -line #__LINE__
assert [integer_to_words -1] {eq $_ {minus one}} -v -line #__LINE__
assert [integer_to_words -19] {eq $_ {minus nineteen}} -v -line #__LINE__
assert [integer_to_words -20] {eq $_ {minus twenty}} -v -line #__LINE__
assert [integer_to_words -21] {eq $_ {minus twenty one}} -v -line #__LINE__
assert [integer_to_words -99] {eq $_ {minus ninety nine}} -v -line #__LINE__
assert [integer_to_words -100] {eq $_ {minus one hundred}} -v -line #__LINE__
assert [integer_to_words -101] {eq $_ {minus one hundred and one}} -v -line #__LINE__
assert [integer_to_words -123] {eq $_ {minus one hundred and twenty three}} -v -line #__LINE__
assert [integer_to_words -999] {eq $_ {minus nine hundred and ninety nine}} -v -line #__LINE__
# */

###########################################################################################
# Round trip
newline
textcolor yellow
puts "Round-trip converting from words to integer ..."
textcolor
newline

assert [words_to_integer [integer_to_words -0]] {== $_ 0} -v -line #__LINE__
assert [words_to_integer [integer_to_words +0]] {== $_ 0} -v -line #__LINE__
assert [words_to_integer [integer_to_words 0]] {== $_ 0} -v -line #__LINE__
puts "Line: "#__LINE__
assert [words_to_integer [integer_to_words 00000000000000000000]] {== $_ 0} -v -line #__LINE__
assert [words_to_integer [integer_to_words {}]] {== $_ 0} -v -line #__LINE__
assert [words_to_integer [integer_to_words ""]] {== $_ 0} -v -line #__LINE__
puts [integer_to_words 10]
puts [words_to_integer ten]
assert [words_to_integer [integer_to_words 10]] {== $_ 10} -v -line #__LINE__
assert [words_to_integer [integer_to_words 100]] {== $_ 100} -v -line #__LINE__
assert [words_to_integer [integer_to_words 123]] {== $_ 123} -v -line #__LINE__
assert [words_to_integer [integer_to_words 750]] {== $_ 750} -v -line #__LINE__
assert [words_to_integer [integer_to_words 1000]] {== $_ 1000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 10000]] {== $_ 10000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 100000]] {== $_ 100000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 1000000]] {== $_ 1000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 10000000]] {== $_ 10000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 100000000]] {== $_ 100000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 1000000000]] {== $_ 1000000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 10000000000]] {== $_ 10000000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 100000000000]] {== $_ 100000000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 1000000000000]] {== $_ 1000000000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 10000000000000]] {== $_ 10000000000000} -v -line #__LINE__
assert [words_to_integer [integer_to_words 505050]] {== $_ 505050} -v -line #__LINE__
assert [words_to_integer [integer_to_words 1001001001]] {== $_ 1001001001} -v -line #__LINE__
# 999,999,999,999,999,999
puts "[integer_to_words 9999999999999]"
assert [words_to_integer [integer_to_words 9999999999999]] {== $_ 9999999999999} -v -line #__LINE__
assert [words_to_integer [integer_to_words 999999999999999999]] {== $_ 999999999999999999} -v -line #__LINE__
assert [words_to_integer [integer_to_words 123456789]] {== $_ 123456789} -v -line #__LINE__
assert [words_to_integer [integer_to_words 987654321]] {== $_ 987654321} -v -line #__LINE__
assert [words_to_integer [integer_to_words $INT_MAX]] {== $_ $INT_MAX} -v -line #__LINE__
# const INT_MIN   -9223372036854775808
assert [words_to_integer [integer_to_words [+ $INT_MIN 1]]] {== $_ [+ $INT_MIN 1]} -v -line #__LINE__

# Check out of range INT_MIN
set i 0
try {
	assert [words_to_integer [integer_to_words $INT_MIN]] {== $_ $INT_MIN} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


######################################################################################
# Float addition
# FIX check trailing zeroes
set i 123456.7890
assert [integer_to_words $i -hyphen] {eq $_ {one hundred and twenty-three thousand four hundred and fifty-six point seven eight nine}} -v -line #__LINE__



######################################################################################
# Mixed digits and words
set value "one million 769 thousand four hundred and 12"
puts "$value is: [words_to_integer $value]"
assert [words_to_integer $value] {eq $_ 1769412} -v -line #__LINE__

set value "23 million seven hundred 69 thousand 4 hundred and twelve"
puts "$value is: [words_to_integer $value]"
assert [words_to_integer $value] {eq $_ 23769412} -v -line #__LINE__

#assert [words_to_integer "1 twenty three"] {eq $_ 23} -v -line #__LINE__				# Missing hundred = 24
assert [words_to_integer "1 hundred twenty three"] {eq $_ 123} -v -line #__LINE__
assert [words_to_integer "1 hundred and twenty three"] {eq $_ 123} -v -line #__LINE__

assert [words_to_integer "two thousand 8 hundred 2"] {eq $_ 2802} -v -line #__LINE__
assert [words_to_integer "two thousand 8 hundred and 2"] {eq $_ 2802} -v -line #__LINE__

assert [words_to_integer 1 million 200 20 two thousand 8 hundred and 2] {eq $_ 1222802} -v -line #__LINE__	 	# Unquoted
assert [words_to_integer "1 million 200 20 two thousand 8 hundred and 2"] {eq $_ 1222802} -v -line #__LINE__	# Quoted

# puts "INT_MIN is [words_to_integer -9223372036854775808]"		# FAILS

# Check mis-smellings
set i 0
try {
	words_to_integer foo
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

######################################################################################
# Words to double (real)
# ADD teenties
# ADD "." for "point"
# ADD Mixed words and digits
puts "Line: "#__LINE__
puts [words_to_integer one hundred and twenty three point four five six]	# FIX Allow teenties
assert [words_to_integer one hundred and twenty three point four five six] {== $_ 123.456} -v -line #__LINE__
assert [words_to_integer one hundred and twenty three point] {== $_ 123.0} -v -line #__LINE__
assert [words_to_integer one hundred and twenty three point zero] {== $_ 123.0} -v -line #__LINE__
# FAIl 123.1
assert [words_to_integer one hundred and twenty three point zero one] {== $_ 123.01} -v -line #__LINE__
assert [words_to_integer nine quintillion one hundred and twenty three point zero four five six seven] {== $_ 9000000000000000123.04567} -v -line #__LINE__

assert [words_to_integer two thousand one hundred and twenty three point nine eight seven six] {== $_ 2123.9876} -v -line #__LINE__
assert [words_to_integer ninety two thousand one hundred and twenty three point zero zero zero nine] {== $_ 92123.0009} -v -line #__LINE__
assert [words_to_integer one hundred and ninety two thousand one hundred and twenty three . six five four] {== $_ 192123.654} -v -line #__LINE__
assert [words_to_integer two million one hundred and ninety two thousand one hundred and twenty three point zero zero] {== $_ 2192123.00} -v -line #__LINE__

puts "Line: "#__LINE__

######################################################################################
# Done - Unload
lib ticol_numword -unload

unset INT_MIN INT_MAX UNSIGNED_MIN UNSIGNED_MAX -const -nocomplain
unset i value -nocomplain

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #