######################################################################################
# File:			test_longevity.tcl
# Author:		M Shaw
# Purpose:		Test the stability of the core interpreter and MPP
#				calling a minimum number of separate commands
# Language:		Ticol Tcl
#
# Versions:
# 1.00 04/Aug/2019	First version
######################################################################################
# Notes:
# Will test very basic calls through the interpreter core and MPP load using 
# a simple and small subset of commands
#
# Rationale:
# Generates no visible output so it can run as fast as possible with high loop counts
# If the interpreter core can be demonstrated to be leak-free over million(s) of 
# iteratins then the remaining faults to be fixed will lie within service commands 
# and plugin DLLs
#
# This is not intended as a command/function test or plugin test
#
# Date			Loop count		Start mem 	End mem		Result/comment
# -------------------------------------------------------------
# 2019/Aug/04 	1,000,000		1320		1376		Stable 
#	Do			1,000,000		1296		1256		After adding more commands
# 2019/Aug/06	1,000,000		?			2608		After more fixes (used 128kb constant)
# 2019/Aug/07	1,000,000		2076		1500		
######################################################################################
# Paths traced:
# 	Interpreter init
# 	Macro Preprocess
# 	AOPM (Expression Reorganiser)
# 	Decompose expression
# 	Parser/lexer
# 	picolEval engine
# 	Command register and unregister
#	Proc register, call and unregister
#	[upvar] from proc
#	Set global directly from proc
# 	Basic command calls e.g. [set], [unset], math ops
# 	Var set, reference and unset
# 	Basic string parse with dquotes and braces
# 	At least one unary math op
#	At least one of each: [if], [for], [foreach], [loop]
#	[assert]
#	[lib] (load/unload only)
######################################################################################
option expression off
option echo off

# Note to self: Ensure names are mangled so test scripts don't accidentally delete
set _script_name [unescape [info script]]

set loop::limit 1000000
set loop::i 1									# Inherited by tests.tcl

textcolor yellow
puts "PID [pid]: Testing Interpreter and MPP core stability over [comma $loop::limit] loops ..."
textcolor
newline

puts "Line: "#__LINE__
puts "Start time [time]@[date]"
halt

set start_cmd_count [cmdcount]
set start_mem [mem_used -ws]
puts [time {
		console set_title "$_script_name - (Test loop [comma $loop::i] of [comma $loop::limit]) - Used: [comma [- [mem_used -ws] $start_mem]] Kb"
		###########################################
		# [set] nested command expression
		set s [* [+ 3 [/ 22 7.0]] 2.5]			# Any simple command expression
#		puts $s
		unset s
		###########################################
		# [expr]
		set s [expr 1+2*3-4/5.0]			# 6.2 (Wincalc && Ticol)
		assert $s {== $_ 6.2}	-line #__LINE__
		unset s
		###########################################
		# [if]
		set s 2
		if {> $s 1} {set t 1} else {set t 0}
		assert $t {== $_ 1}	-line #__LINE__
		unset t
		unset s
		###########################################
		# [for]
		for {set s 0} {< $s 3} {++ s} {
			;
		}
		assert $s {== $_ 3}	-line #__LINE__
		###########################################
		# [foreach]
		unset x -nocomplain
		set count 0
		foreach x $qbf {
			++ count
		}
		#puts $count
		assert $count {== $_ 9}	-line #__LINE__
		unset x count
		###########################################
		# [while]
		set s 10
		while {> $s 0} {
			-- s
		}
		assert $s {== $_ 0}	-line #__LINE__
		unset s
		###########################################
		# [loop]
		set count 0
		loop x 0 3 {		# 0..2 inclusive
			++ count
		}
		###########################################
		# Exit +1 higher than increment
		assert $count {== $_ 3}	-line #__LINE__
		assert $x {== $_ 3}	-line #__LINE__
		unset x count
		###########################################
		# [time]
		set count 0
		time {++ count} 10 -quiet
		assert $count {== $_ 10}	-line #__LINE__
		unset count
		###########################################
		# Dquotes and braces
		unset s -nocomplain
		set s "Hello world"
		set s $qbf
		unset s
		set s {hi there}
		unset s -nocomplain
		###########################################
		# Weird vars 1 #
		set { } 123
		assert ${ } {eq $_ 123}	-line #__LINE__
		unset { }
		###########################################
		# Weird vars 2 #
		set {My Var} 456
		assert ${My Var} {eq $_ 456}	-line #__LINE__
		unset {My Var}
		###########################################
		# Proc #
		set f "x"
		set q "y"
		proc foo {} {
			upvar q
			set q hi
			set ::f Hello						# Set global directly
			assert $::f {eq $_ Hello}	-line #__LINE__
#puts "Line: "#__LINE__
			#vars -level 0
		}
		foo
		#vars
		assert $f {eq $_ Hello}	-line #__LINE__
		assert $q {eq $_ hi}	-line #__LINE__
		undef foo -nocomplain
		###########################################
		# [lib] load and unload
		# Consumes memory but doesn't seem to leak
		###########################################
		lib ticol_cpu
		lib ticol_cpu -unload
		###########################################

/*
		# source test_reg.tcl
		if {is_mod $loop::i 1000} {
			textcolor darkyellow
			puts $loop::i
			textcolor
		} # */
		++ loop::i 
		sleep 5									# Don't flatline the CPU in a short loop
	} $loop::limit -info
]

unset s -nocomplain

puts "Finish time [time]@[date]"
puts "It looks like we used (or lost) [- [mem_used -ws] $start_mem] Kb during the run"
# Assert that we used (lost) no more than 2048Kb (Units in Kb)
assert [- [mem_used -ws] $start_mem] {< $_ 2048} -v -line #__LINE__

textcolor green
puts "* Done *"
puts "Command count is '[cmdcount]'"
puts "Total command run: [- [cmdcount] $start_cmd_count]"
textcolor

beep 1000 700; beep 800 1000; beep 1000 500
pause 

vars

# EOF #