#############################################################################################
# File:				test_loop.tcl
# Purpose:			Test [loop] iterator
# Author:			M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 17/Jan/2017	Test cases for experimental rapid loop
# 1.01 22/Jun/2017	Replace [checkex] with [assert]
# 1.02 06/Jun/2022	Changed [loop] bounds to make more sense
# 1.03 27/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
#############################################################################################
# Notes:
# DONE FIX loop i 0 10 {} will include 10 Should this not be 0..9?
# We should probably be consistent with TclX->[loop]
# Currently behaves like BASIC as 
# for j= 0 to 10: ? j :next ' displays 0..10
# TclX behaves like 
# % loop i 0 10 {puts $i} # displays 0..9
# % loop i 0 -9 -1 {puts $i}		# displays 0.. -8
# A simpler Tcl For Loop. At least twice as fast as [for]
# https://wuhrr.wordpress.com/2011/01/31/a-simpler-tcl-for-loop/ 
#############################################################################################
/*
/G shows timings as 7us / call
Timing [loop] ...
3 ms/iter
Timing [for] ...
8 ms/iter

Timing [loop] ...
4 ms/iter
Timing [for] ...
8 ms/iter
*/
#############################################################################################

textcolor yellow
puts "Testing \[loop\] iterator ..."
textcolor
newline

newline
textcolor yellow
puts "Incrementing by 1 with default increment"
textcolor
loop i 0 10 {
	puts "i is $i"
}
# We now exit at guard+1 like other languages (06/2022)
assert $i {== $_ 10} -v -line #__LINE__

newline
textcolor yellow
puts "Incrementing by 1 with explicit increment"
textcolor
loop i 0 10 1 {
	puts "loop 0..10 => i is $i"
}
# We now exit at guard+1 like other languages (06/2022)
assert $i {== $_ 10} -v -line #__LINE__

newline
textcolor yellow
puts "\[loop\] 0..10 with \[continue\] at 5"
textcolor
set sum 0
loop i 0 10 1 {			# 10 increments 0..9
	if {== $i 5} {
		puts "Skip 5 and increment past 6"
		++ i			# Check/test extra increment
		continue
	}
	+= sum $i
	puts "i is $i, sum is $sum"
}
# We now exit at guard+1 like other languages (06/2022)
assert $i {== $_ 10} -v -line #__LINE__
assert $sum {== $_ 34} -v -line #__LINE__
unset sum

newline
textcolor yellow
puts "\[loop\] 0..10 with \[break\] at 5"
textcolor
loop i 0 10 1 {			# 10 increments 0..9
	if {== $i 5} {
		puts "Break at 5"
		break
	}
	puts "i is $i"
}
assert $i {== $_ 5} -v -line #__LINE__

newline
textcolor yellow
puts "Incrementing by 1 with return from proc"
textcolor
set i 0
undef foo -nocomplain
proc foo {} {
	upvar i
	loop i 0 10 1 {
		puts "i is $i"
		if {== $i 5} {return $i}
	}
}
foo 
assert $i {== $_ 5} -v -line #__LINE__

newline
textcolor yellow
puts "Incrementing by 2"
textcolor
loop i 0 20 2 {				# 20 iters - 0..19, with step of 2. Termination == 18. Postcondition = +2 == 20
	puts "i is $i"
}
# We now exit at guard+1 like other languages (06/2022)
assert $i {== $_ 20} -v -line #__LINE__

/*
# FIX this requires a big loop
set big_increment 898723948729374
set big_limit [* 898723948729374 5]
newline
textcolor yellow
puts "Incrementing by 5 * $big_increment to $big_limit '[string length $big_limit]'"
textcolor
loop i 0 $big_limit $big_increment {
	puts "i is $i '[string length $i]'"
}
puts "i is $i"
assert $i {== $_ [* 898723948729374 5]} -v -line #__LINE__
unset big_increment
unset big_limit
# */

newline
textcolor yellow
puts "Decrementing by -1 with auto-detect of increment"
textcolor
set i 0
loop i 10 0 {						# 10 iters 10..1 inclusive. Termination case == 0
   puts "i is $i at line "#__LINE__
}
# Exit case will be 1 lower than the bottom guard
assert $i {== $_ 0} -v -line #__LINE__

newline
textcolor yellow
puts "Decrementing by -1 with explicit decrement at "#__LINE__
textcolor
loop i 10 0 -1 {				# 10 iters 10..1 inclusive. Termination case == 0
   puts "i is $i"
}
assert $i {== $_ 0} -v -line #__LINE__

newline
textcolor yellow
puts "+ve to -ve range at "#__LINE__
textcolor
set i 0
set k 0
loop i 5 -5 {						# 10 loops. Termination at -4, termination case is -1 so -5
	# loop i 5 -5 -1 {
	puts "i is $i"
	++ k
}
assert $i {== $_ -5} -v -line #__LINE__
assert $k {== $_ 10} -v -line #__LINE__		# 11 loops

newline
textcolor yellow
puts "-ve to +ve range"
textcolor
set i 0
set k 0
loop i -5 5 {			# 10 loops, terminate at 4, termination case is 5
   puts "i is $i"
	++ k   
}
assert $i {== $_ 5} -v -line #__LINE__
assert $k {== $_ 10} -v -line #__LINE__		# 11 loops

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor yellow
puts "Check invalid range 10->10 (code should not be called)"
textcolor
set k 0
set i 0
loop i 10 10 {						# FIX Hmmmm... do we post increment
   puts "i is $i"					# DONE FIX inits and loops ONCE. Should never run {code}
   set k 1
}
assert $i {== $_ 10} -v -line #__LINE__			# x1 loop with i == 10. No loop iterations so no post increment
assert $k {== $_ 0} -v -line #__LINE__			# Never incremented	
unset k

puts "Line "#__LINE__
newline
textcolor yellow
puts "Check catch invalid increment (0)"
textcolor
catch {
	puts "Incrementing by 0 so no iterations"
	loop i 0 10 0 {					# 10 loops 0.. 9
		puts "i is $i (You ain't seen me, right?)"
	}
}
newline

puts "Line "#__LINE__
set k 0
# Increment to INT64_MAX in 10 steps
loop i 0 9223372036854775800 922337203685477580 {
	++ k
	puts "${k}:\t$i"
}
assert $k {== $_ 10} -v -line #__LINE__

set k 0
# Increment to INT64_MAX in 9 steps
loop i 0 [- 9223372036854775800 922337203685477580] 922337203685477580 {
	++ k
	puts "${k}:\t$i"
}
assert $k {== $_ 9} -v -line #__LINE__

set k 0
loop i 5 -5 -1 {
	puts $i
	+= k $i
}
puts $k
assert $k {== $_ 5} -v -line #__LINE__


#############################################################################################
# Timings
#############################################################################################

# 3ms /iter
newline
textcolor yellow
puts "Timing \[loop\] ..."
textcolor
puts "[time {
 loop i 0 1 {}
} 10000] ms/iter"

# 8ms / iter
newline
textcolor yellow
puts "Timing \[for\] ..."
textcolor
option expression off
puts "[time {
 for {set i 0} {< $i 1} {++ i} {}
} 10000] ms/iter"

unset i k -nocomplain

vars

#############################################################################################

newline
textcolor green
puts "* Done *"
textcolor

# EOF #