#######################################################################################
# File: 			test_lpushl_pop.tcl
# Purpose:			Tests for [lpush], [lpop] and [ltail]
# Language:			Ticol Tcl v1.27
#
# Versions:
# 1.00 07/Jan/2023	First version, [lpush], [lpop], [ltail]
# 1.01 13/Jan/2023	[lpeek]
#######################################################################################
# Notes:
# [lpush]
# [lpop]
# [ltail]
# [lpeek]
#######################################################################################
textcolor yellow
puts "Testing \[lpush\] \[lpop\] in file [file name [info script]]"
textcolor
newline

textcolor yellow
puts "* \[lpush\] ..."
textcolor

puts "Line "#__LINE__
# Pushing an empty list onto an empty list results in an empty list
set s [lpush [lpush "" ""] ""]
assert $s {eq $_ ""} -v -line #__LINE__
newline

# Pushing an item onto an empty list results in a list with one item
set s [lpush [lpush "" ""] Hello]
assert $s {eq $_ "Hello"} -v -line #__LINE__
newline

# Assembling nested calls results in a list
set s [lpush [lpush "" Cruel] world]
puts "s is '$s'"
assert $s {eq $_ "Cruel world"} -v -line #__LINE__
newline

# Multi word sub-lists
set s [lpush [lpush "" "Hello darkness"] "my old friend"]
puts "s is '$s'"
assert $s {eq $_ "{Hello darkness} {my old friend}"} -v -line #__LINE__
newline

# Multi word sub-lists using a variable inbound
set s [lpush [lpush "" "Here is"] $qbf]
puts "s is '$s'"
assert $s {eq $_ "{Here is} {$::qbf}"} -v -line #__LINE__
newline

# push to a variable
set s ""
lpush s Hello -var
lpush s world -var
puts "s is '$s'"
assert $s {eq $_ "Hello world"} -v -line #__LINE__
newline

# Pushing an empty list onto an empty list results in an empty list
# We can't chain nested calls when using variables
set s ""
lpush s "" -var
lpush s "" -var
assert $s {eq $_ ""} -v -line #__LINE__
newline

# Pushing an item onto an empty list results in a list with one item
# We can't chain nested calls when using variables
set s ""
lpush s "" -var
lpush s "Hello" -var
assert $s {eq $_ "Hello"} -v -line #__LINE__
newline

# Assembling nested calls results in a list
# We can't chain nested calls when using variables
set s ""
lpush s "" -var
lpush s "Cruel" -var
lpush s "world" -var
puts "s is '$s'"
assert $s {eq $_ "Cruel world"} -v -line #__LINE__
newline

# Multi word sub-lists
# We can't chain nested calls when using variables
set s ""
lpush s "" -var
lpush s "Hello darkness" -var
lpush s "my old friend" -var
puts "s is '$s'"
assert $s {eq $_ "{Hello darkness} {my old friend}"} -v -line #__LINE__
newline

# Multi word sub-lists using a variable inbound
# We can't chain nested calls when using variables
set s ""
lpush s "" -var
lpush s "Here is" -var
lpush s $qbf -var
puts "s is '$s'"
assert $s {eq $_ "{Here is} {$::qbf}"} -v -line #__LINE__
newline

#######################################################################################
# lpop
#######################################################################################
puts "Line "#__LINE__
textcolor yellow
puts "* \[pop\] ..."
textcolor
# We can't do much in non-var mode other than return the head with the tail left intact
set s $qbf
assert [lpop $s] {eq $_ "The"} -v -line #__LINE__
newline

# Test safe operation on NULL lists
set s ""
assert [lpop $s] {eq $_ ""} -v -line #__LINE__
assert [lpop ""] {eq $_ ""} -v -line #__LINE__
assert [lpop {}] {eq $_ ""} -v -line #__LINE__

# Check safe operation on null -var lists
set s ""
set head [lpop s -var]
assert $head {eq $_ ""} -v -line #__LINE__
assert $s {eq $_ ""} -v -line #__LINE__

# lpop in -var mode can split the head and tail
set s $qbf
set head [lpop s -var]
puts "Head is '$head'"
assert $head {eq $_ "The"} -v -line #__LINE__
puts "Tail is '$s'"
assert $s {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__
newline

# Repeat loop and call beyond list exhaustion
set s $qbf
loop i 0 12 {
	set head [lpop s -var]
	puts "Loop $i: Head is '$head' tail is '$s'"
	if {== $i 7} {
		assert $head {eq $_ "lazy"} -v -line #__LINE__
		assert $s {eq $_ "dog"} -v -line #__LINE__
	}
}
assert $head {eq $_ ""} -v -line #__LINE__
assert $s {eq $_ ""} -v -line #__LINE__
newline

#######################################################################################
# ltail
#######################################################################################
puts "Line "#__LINE__
textcolor yellow
puts "* \[ltail\] ..."
textcolor

assert [ltail {}] {eq $_ ""} -v -line #__LINE__
assert [ltail ""] {eq $_ ""} -v -line #__LINE__
assert [ltail $qbf] {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__
newline

#######################################################################################
# Man file example
#######################################################################################
textcolor yellow
puts "* man file example..."
textcolor
puts [lpush [lpush "" Hello] world]
set s ""
lpush s Hello -var
lpush s world -var
puts $s
set head [lpop s -var]
puts $head
puts $s
set s ""
lpush s {Now is the winter} -var
lpush s {of our discontent} -var
puts $s

#######################################################################################
# [lpop v -var -n X]
#######################################################################################
newline
textcolor yellow
puts "* \[lpop\] multi-element -n mode..."
textcolor
set s $qbf					# The quick brown fox jumps over the lazy dog (can't modify a const)
set q [lpop s -var -n 1]
assert $q {eq $_ The} -v -line #__LINE__
assert $s {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__

set q [lpop s -var -n 3]
assert $q {eq $_ "quick brown fox"} -v -line #__LINE__
assert $s {eq $_ "jumps over the lazy dog"} -v -line #__LINE__

set q [lpop s -var -n 1]
assert $q {eq $_ "jumps"} -v -line #__LINE__
assert $s {eq $_ "over the lazy dog"} -v -line #__LINE__
# Exhaust the source list var...
set q [lpop s -var -n 200]
assert $q {eq $_ "over the lazy dog"} -v -line #__LINE__
assert $s {eq $_ ""} -v -line #__LINE__

#######################################################################################
# Trap attempt to set a const
#######################################################################################
newline
textcolor yellow
puts "* \[lpop\] attempt to set const qbf..."
textcolor
set i 0
try {
	set q [lpop qbf -var]
} catch {
	puts "Successfully caught attempt to set const qbf"
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

#######################################################################################
# [lpeek]
#######################################################################################
newline
textcolor yellow
puts "* \[lpeek\] ..."
textcolor
assert [lpeek "x"] {eq $_ "x"} -v -line #__LINE__
assert [lpeek $qbf] {eq $_ "The"} -v -line #__LINE__
assert [lpeek qbf -var] {eq $_ "The"} -v -line #__LINE__
assert [lpeek ""] {eq $_ ""} -v -line #__LINE__

#######################################################################################
# Clean up
#######################################################################################
unset head i q s tail -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #


