#####################################################################################
# File:				test_proc_return_types.tcl
# Author:			M Shaw
# Purpose:			Unit test returning various types from a proc
# Language:			Ticol Tcl v1.20+, v1.26+
#
# Versions:
# 1.00 08/Nov/2016	First version
# 1.01 13/Nov/2017	Revised
# 1.02 05/Feb/2022	Updated. Made into a unit test
#####################################################################################
# Notes
# Ticol structs are mainly designed to interact with lib calldll_* but can also be
# used as a structured type within scripts
#
# For struct, we populate the struct members with various types of data and
# call to populate the struct via [proc ret_val] then test the individual results
#####################################################################################
option echo on

undef ret_val -nocomplain

proc ret_val {value} {
	upvar $value												# Some kind of ADT is passed to us
	textcolor green

	if {[eq [type $value] stack]} {								# Push something onto a referred stack object
		puts "ret_val: Stack: '[stack item $value 0]'"
		stack push $value "s1 Stack item OUT"   				# Return stack value
		
	} elseif {[eq [type $value] array]} {						# Assign an array element
		textcolor red
		puts "ret_val: Array: [array item $value 1]"
		array set $value 1 "Ret-array"                			# Return array item
		
	} elseif {[eq [type $value] struct]} {						# Populate a struct
		textcolor cyan
		puts "ret_val: Struct $value item(s):"	
#puts "check1: '[-> ${value}.i]'" # Dereferences to var content
#puts "check2: '[set ::${value}.i]'"
#puts "struct i IN:\t'[set ::[-> ${value}.i]]'"  # Int
		#array set ::${value}.a(1) 1 "a1"
#puts "check set array as: '[-> ${value}.a]'"
		array set ::[-> ${value}.a] 1 "a1"
		struct set ::${value}.f 	"f2"
		struct set ::${value}.i 	"i3"						# PASS
#puts "struct k: '[stack item ::[-> ${value}.k] 0]'" # Stack
		stack push ::[-> ${value}.k] "k4"						# ADTs require a field dereference using [->]
		#struct set ::${value}.k "k4"							# PASS (sets as a simple var)
		struct set ::${value}.sr 	"sr5"						# Simple variables do not require field dereference
		struct set ::${value}.sl 	"sl6"
	} else {													# General type, e.g. string, number integer etc. (sn4)
		# Set literal var
		puts "ret_val: String or number $value is '[set $value]'"
		set $value "sn4"
	}
	textcolor
}
#####################################################################################

struct ts {		# Test struct: holding various data types
	a 20		# Array
	f 40		# Double/Float Literal
	i 20		# Integer Literal
	k 20		# Stack Reference
	sr 20		# String Reference
	sl 256		# String Literal
}

# // PRELIMINARY ASSIGN OF ADTs to STRUCT FIELDS //

set i 123456789
struct set ts.i i			# Reference
struct set ts.i 123456789	# Value

set f $pi
struct set ts.f f

set s "Hello I am s"
# FIX sl

set r "Hello string"
struct set ts.sl r
struct set ts.sl "Hello literal string"

set r "Hello string"
struct set ts.sr r							# Reference (name)
struct set ts.sl "Hello literal string"		# Literal


set a(1) "This is array item 1"
struct set ts.a a

stack create stk 200
stack push stk "Hello stack IN"
struct set ts.k stk

#####################################################################################
# // CALL //
#####################################################################################

ret_val s    # Pass a variable by name
textcolor yellow
puts "\tProc returned string literal as:\t'$s'"
assert $s {eq $_ "sn4"} -v -line #__LINE__

ret_val r    # Pass a variable by reference
textcolor yellow
puts "\tProc returned string reference as:\t'$r'"
assert $r {eq $_ "sn4"} -v -line #__LINE__

ret_val i    # Pass a variable by name
textcolor yellow
puts "\tProc returned integer literal as:\t'$i'"
assert $i {eq $_ "sn4"} -v -line #__LINE__

ret_val f    # Pass a variable by name
textcolor yellow
puts "\tProc returned float literal as:\t'$i'"
assert $f {eq $_ "sn4"} -v -line #__LINE__

ret_val a    # Pass an array by name
textcolor yellow
puts "\tProc returned array a(1) as:\t\t'$a(1)'"
assert $a(1) {eq $_ "Ret-array"} -v -line #__LINE__

ret_val stk  # Pass a stack by name
textcolor yellow
set q [stack pop stk]
puts "\tProc returned stack as:\t\t'$q'"
assert $q {eq $_ "s1 Stack item OUT"} -v -line #__LINE__

puts "Line :"#__LINE__

#####################################################################################
# // STRUCT Translation and Returns //
#####################################################################################

newline
ret_val ts   # Pass a pre-assigned struct by name

textcolor magenta
puts "Proc returned stack as:\t\t'[ofaddress ts.i]'"
puts "\ts.i is '$ts.i'"									# PASS Standard reference method
assert $ts.i {eq $_ "i3"} -v -line #__LINE__

puts "Line :"#__LINE__

#puts "\ta(1) is '[set a(1)]'"								# PASS Alternate reference method
#assert $a(1) {eq $_ "s1 Stack item OUT"} -v -line #__LINE__

puts "\tts.a '[array item [ofaddress ts.a] 1]' ($ts.a)"		# PASS Field reference method
assert $ts.a {eq $_ "a"} -v -line #__LINE__

puts "\tts.f '[ofaddress ts.f]' ($ts.f)"					# PASS "
assert $ts.f {eq $_ "f2"} -v -line #__LINE__

puts "\tts.i '[ofaddress ts.i]' ($ts.i)"					# PASS "
assert $ts.i {eq $_ "i3"} -v -line #__LINE__

puts "\tts.k '[stack pop [ofaddress ts.k]]'"				# PASS "
assert $ts.k {eq $_ "stk"} -v -line #__LINE__

#puts "\tts.k '[ofaddress ts.k]' ($ts.k)"					# PASS (If re-set as a simple variable)
assert $ts.k {eq $_ "stk"} -v -line #__LINE__

puts "\tts.sr '[ofaddress ts.sr]' ($ts.sr)"					# PASS Field reference method
assert $ts.sr {eq $_ "sr5"} -v -line #__LINE__

puts "\tts.sl '[ofaddress ts.sl]' ($ts.sl)"					# PASS "
assert $ts.sl {eq $_ "sl6"} -v -line #__LINE__


undef ret_val -nocomplain
struct unset ts -nocomplain
unset f i r s -nocomplain
stack unset stk -nocomplain
unset q -nocomplain
array unset a -nocomplain

#####################################################################################
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #