###################################################################################
# File: 			test_proc_static.tcl
# Purpose:			Test [proc static]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 05/Nov/2017	First version
# 1.01 05/Nov/2017	Implemented built-in (non-proc) command [static]
# 1.02 07/Nov/2017	Rewrote naive proc version to handle being run in root scope
# 1.03 17/Jun/2018	Slight mods to static proc
# 1.04 12/Jul/2018	Reverted changes
# 1.05 01/Aug/2019	Renamed to avoid clash with inbuilt [static] in loop tests
###################################################################################
# Notes:
# Note that [uplevel 1] will run the command in the parent callframe. If the command
# being called is [upvar] then this will jump up +1 more level
###################################################################################
# TODO
# Test local alias default value scope cascades immediately up to global copy
###################################################################################

option expression off
#option breakpoint on
puts "Line: "#__LINE__
# undef static -nocomplain								# [undef] the built-in to use the proc version
#halt

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# IMPORTANT: Our design for proc variable inheritence cascades values back up the call-chain
# to the root namespace. So [set] at proc level cacades the values upwards back to root
# and not down from root to the current proc scope
# Thus to set an upvar'ed value's root value you set the local [upvar]'ed variable and NOT
# the root variable directly
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


puts "Line: "#__LINE__
# /* # Debug version
if {! [defined test_static]} {
	proc test_static {name {value {}}} {
#puts "Enter static"
		if {> [info level] 1} {							# If called from root lvel this will be level 1
#puts "Case proc: Level is [info level]"
			set scope [uplevel 1 {info procname}]		# Save procname as new scope name (simpler to parse)
#puts "Setting varname as ${scope}::$name"			
			set varname ${scope}::$name					# Create mangled root scope var name
			#uplevel 1 "upvar @0 $varname $name"     	# Run upvar in the root callframe
			if {! [is_set ::$varname]} {                # If root var is empty and we have an arg
#puts "Case: var ::$varname not set"
				uplevel 1 "upvar @0 $varname $name"     # Run upvar in the root callframe
				uplevel 1 "set $name $value"            # Set local alias. See inheritance notes
			} else {
#puts "Case: var ::$varname is set so upvar it"
				uplevel 1 "upvar @0 $varname $name"     # Run upvar in the root callframe to set parent's local var
			}
		} elseif {! [is_set ::$name]} {                 # We are calling [static] in the root namespace scope
#puts "Case root: Level is [info level]"
			set ::$name $value							# Use set but only on initialisation
		}
	}
} # */


undef foo -nocomplain
unset foo::q -nocomplain

set init $false									# Used to control "one-shot" assert

# level 1
proc foo {{x {}}} {
	test_static q 23							# Skidoo
	if {! $::init} {
		assert $::foo::q {== $_ 23} -v -line #__LINE__
	}
	set ::init $::true							# Trigger only once
#puts "local q is: '$q' root q is: '$::foo::q'"	# Verify both root and local have been initialised
#vars * -level [- [info level] 1]				# Debug: Show vars at parent level
#vars * 										# Debug: Show vars at current level
# puts "\r\nEnter foo: q is '$q'"
	if {&& [!= $x 0] [ne $x {}]} {
		set q $x
#		puts "foo->q was set to '$q'"
	}
#	puts "Exit foo->q::$q"
	return $q
}

undef garp -nocomplain

proc garp  {{x {}}} {
	return [foo $x]
}

puts "Line: "#__LINE__
vars 
# level 0
assert [foo 1] {== $_ 1} -v -line #__LINE__		# Initial static set
assert [foo] {== $_ 1} -v -line #__LINE__		# Pass no arg, test for value persistence in local var
assert [foo 0] {== $_ 1} -v -line #__LINE__		# Should still be static value
assert [foo 2] {== $_ 2} -v -line #__LINE__		# > 0 will [set] directly
assert [foo 0] {== $_ 2} -v -line #__LINE__		# Else, static value preserved

assert [foo fred] {eq $_ fred} -v -line #__LINE__ 	# > 0, so "fred"
assert [foo] {== $_ fred} -v -line #__LINE__		# Static preserved
assert [foo 0] {eq $_ fred} -v -line #__LINE__		# Static preserved
vars 

puts "Line: "#__LINE__

# 2-level call
assert [garp 1] {== $_ 1} -v -line #__LINE__		# > 0 so will set
assert [garp 0] {== $_ 1} -v -line #__LINE__		# Preserved
assert [garp] {== $_ 1} -v -line #__LINE__			# Preserved

vars foo*

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Test set statics at root level #
# Current design: static merely declares a var and optionally intialises it
# Once created and initialised any arg value will be ignored and [set]
# should be used to set the var
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "\r\nTesting use of \[static\] proc at root scope level ..."
textcolor
newline
test_static moo
#vars
assert $moo {== $_ {}} -v -line #__LINE__			# static with no args should be {}
test_static moo 23									# arg is ignored once static exists
assert $moo {== $_ {}} -v -line #__LINE__			# Should still be ""

unset moo											# Reset
test_static moo 23									# Set and init
assert $moo {== $_ 23} -v -line #__LINE__			# Check static has held value

test_static moo										# Call with no args this time
assert $moo {== $_ 23} -v -line #__LINE__			# Should still be 23
test_static moo 11									# Attempt to misuse and change
assert $moo {== $_ 23} -v -line #__LINE__			# Should still be 23
set moo 11											# Correctly use [set] now
assert $moo {== $_ 11} -v -line #__LINE__			# Should have changed to 11

vars m*

puts "Line: "#__LINE__
unset moo* -glob -nocomplain
unset ???::q -glob -nocomplain

#vars m*
#puts [procs]

undef foo -nocomplain
undef garp -nocomplain
if {is_proc test_static} {								# Avoid undefining the built-in command
	undef test_static -nocomplain
}
#puts [procs]

puts "Line: "#__LINE__
unset init -nocomplain
unset foo::q -nocomplain

newline
vars
###################################################################################
newline

textcolor green
puts "* Done *"
textcolor

# EOF #