############################################################################################
# File:				test_resolve.tcl
# Author:			M Shaw
# Purpose:			Resolve an abiguous path and filespec
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 17/Aug/2019 First version
###########################################################################################
# Notes:
# -addslash
# -noescape
# DONE ADD -trace to [assert] as it is a black box and we need to see what it is comparing
# FIX Shuld we not escape returns by default unless -noescape ?
# DONE FIX Add filespec argument
# ADD Tests
# What this should do:
# Resolve partial paths in either Windows or Unix format
# Append an optional filetype or wildcard spec
###########################################################################################
# Should be shorthand for a common idiom, aka:
/*
	if {eq [file extension $filespec] ""} {						# No filespec - assume path given
		#set filespec [file normalize $filespec]				# Set the filespect to the current, normalised path
		set filespec [escape [file normalize $filespec]]		# Set the filespect to the current, normalised path
		set filespec [addslash $filespec]						# Safely append any required \\ chars
		set path $filespec										# Store our path
		append filespec "*.ans"
	} else {
		set path [file dirname $filespec]
		set path [addslash $path]
	}
# */
#
# Inputs could be: <ambiguious_path> <filespec>|""
# <filespec> could be ""-> path only, "*" -> path+*.*, "*.*" (ditto), *.typ -> path+*.typ, filename.typ -> path+filename.typ
# Example:
# [file resolve "" "*"] 					# Current directory assumed
# [file resolve "." "*.tcl"]				# Current directory
# [file resolve "." "*"]					# Current directory
# [file resolve "\" "*"]					# Root directory
# [file resolve ".\" "*.*"]					# Current directory
# [file resolve "/foo" "*.*"]		# Detect if foo is a dir or filename
#	If filename-> "<FQPATH>/foo"
#	If dirname -> "<FQPATH>/foo/*.*"
# All of the above accept "/"
# [file resolve "/" "*"]					# Root directory
# Options:
# -noescape
#
# We use [echo] as we want to examine the raw string without backslash translation
# Tests:
###########################################################################################
option echo on
option expression off

puts "Line: @"#__LINE__

set curpath [getcwd]		# Returns escaped path
set unescaped_curpath [unescape $curpath]

echo "curpath '$curpath'"
echo "unescaped_curpath '$unescaped_curpath'"
newline

# Backslash
# EMPTY CASES
puts "Line: @"#__LINE__
echo "\[file resolve \"\"\] -> '[file resolve ""]'"
echo [file resolve ""] -hex
# Raw trace is processing with printf escapes despite using echo
assert [file resolve ""] {eq $_ $unescaped_curpath} -v -noescape -trace -line  #__LINE__		# FAILS why?

echo "\[file resolve ""\] -> '[file resolve "" ""]'"
echo [file resolve ""] -hex

textcolor yellow
puts "Line: @"#__LINE__
echo "\[file resolve "" ""\] -> '[file resolve {} {}]'"
textcolor
echo '[file resolve "" ""]'
# echo [file resolve "" ""] -hex
echo '[file resolve "" "*"]'
echo '[file resolve "" "*.*"]'
echo '[file resolve "" "*.typ"]'
echo '[file resolve "" "filename"]'
echo '[file resolve "" "filename.typ"]'
newline

textcolor yellow
puts "Line: @"#__LINE__
echo "\[file resolve "." ""\] -> '[file resolve {} {}]'"
textcolor
echo '[file resolve "." ""]'
# echo [file resolve "." ""] -hex
echo '[file resolve "." "*"]'
echo '[file resolve "." "*.*"]'
echo '[file resolve "." "*.typ"]'
echo '[file resolve "." "filename"]'
echo '[file resolve "." "filename.typ"]'
echo '[file resolve "." "subdir"]'		# Should check if what is being appended is a folder and if so add a slash ?
newline

textcolor yellow
puts "Line: @"#__LINE__
echo "\[file resolve ".\\" ""\] -> '[file resolve {} {}]'"
textcolor
echo '[file resolve ".\\" ""]'
# echo [file resolve ".\\" ""] -hex
echo '[file resolve ".\\" "*"]'
echo '[file resolve ".\\" "*.*"]'
echo '[file resolve ".\\" "*.typ"]'
echo '[file resolve ".\\" "filename"]'
echo '[file resolve ".\\" "filename.typ"]'
echo '[file resolve ".\\" "subdir"]'		# Should check if what is being appended is a folder and if so add a slash ?
newline


textcolor yellow
puts "Line: @"#__LINE__
echo "\[file resolve ".." ""\] -> '[file resolve {} {}]'"
textcolor
echo '[file resolve ".." ""]'
# echo [file resolve ".." ""] -hex
echo '[file resolve ".." "*"]'
echo '[file resolve ".." "*.*"]'
echo '[file resolve ".." "*.typ"]'
echo '[file resolve ".." "filename"]'
echo '[file resolve ".." "filename.typ"]'
echo '[file resolve ".." "subdir"]'		# Should check if what is being appended is a folder and if so add a slash ?
newline


# NON-EMPTY CASES
assert [file resolve "."] {eq $_ $unescaped_curpath} -v -noescape -line #__LINE__
puts "Line: @"#__LINE__

# DONE FIX need to remove a segment from curpath (added [file pathstrip])
echo [file resolve ".."]
assert [escape [file resolve ".."]] {eq $_ [file pathstrip $unescaped_curpath 1]} -v -line #__LINE__
puts "Line: @"#__LINE__
puts "'[file resolve "\\"]' @"#__LINE__
assert [file resolve "\\"] {eq $_ "\\"} -v -line #__LINE__
puts "Line: @"#__LINE__
echo [file resolve ".\\"]
echo $unescaped_curpath
assert [file resolve ".\\"] {eq $_ $unescaped_curpath} -v -line #__LINE__
newline

# Forward slash

unset unescaped_curpath curpath -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #