###############################################################
# File: 			test_return_exit_proc.tcl
# Purpose:			Testing that [return can exit a proc]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 25/Jun/2018	First version
# 1.01 18/Nov/2022	Updated
###############################################################
# Notes: 
# FIX while a proc can return a value it should probably never
# return code return (2) ?
# FIX show that [return -code ok] does not exit current CX
#	Also check with ActiveState for correct operation
# FIX Check return from recursion
# Ensure that [return] can exit a proc
# v1.26 all tests pass 25/Jun/2018
###############################################################
option expression off

######################################################################
## Validate basic proc functioning
set q 0
proc foo {} {				# Check the test method
	#upvar q
	#set q 1
	set ::q 1
}
foo
assert $q {== $_ 1} -v -line #__LINE__
undef foo

######################################################################
## Validate that level 1 proc returns from [return] statement
set q 0
proc foo {} {
	return
	set ::q 1
}
foo
assert $q {== $_ 0} -v -line #__LINE__
undef foo

######################################################################
## Validate that level 1 proc returns from [return] statement which 
# is nested in an [if] block
set q 0
proc foo {} {
	if {$::true} {
		return
	}
	set ::q 1
}
foo
assert $q {== $_ 0} -v -line #__LINE__

######################################################################
## Validate that level 1 proc returns from [return] statement which 
# is nested in an [if] block when using -code return
set q 0
proc foo2 {} {
	if {$::true} {
		return -code return
	}
	set ::q 1
}
foo2
assert $q {== $_ 0} -v -line #__LINE__


######################################################################
## Check that child proc exits without continuing to execute
## is nested in an [if] block and that parent does not exit
## Any scriptlet call will be evaluated by its calling wrapper and the last 
## code which was returned will be checked
## If code 2 then this may be passed back and therefore cause exit
set q 0
proc foo3 {} {
	set a 1
	if {$a} {
		if {$::true} {
			return			# Where would we halt this chain of passing return (2) back ?
		}
	}
	set ::q 1
}
foo3
assert $q {== $_ 0} -v -line #__LINE__

proc bar1 {} {
	set ::q 0
/*
	if {$::true} {		# This should isolate the code 2 return
		foo3			# THUS this will cause an immediate exit!!!!
		# puts "Hello"	# Uncommenting this will result in a PASS
	} # */
	loop 1 {
		foo3
		puts Hi			# FIX ME (remove line)
	}
	set ::q 23
}
bar1
assert $q {== $_ 23} -v -line #__LINE__

######################################################################
## Check child proc returns when asked
proc bar2 {} {
	set ::q 0
	if {$::true} {
		foo3
	}
}
bar2
assert $q {== $_ 0} -v -line #__LINE__

undef bar1 bar2 
######################################################################
## Check that [return] from a child proc does not cause the 
## parent proc to exit prematurely
proc bar2 {} {
	if {$::true} {return}
}

set q 0
proc bar1 {} {
	bar2
	puts "In [info procname] at line "#__LINE__
	set ::q 1			# this line must be executed
	puts "In [info procname] at line "#__LINE__
}
bar1
assert $q {== $_ 1} -v -line #__LINE__

############################################################################
## Check that recursive calls are terminated by [return]
set q 0
set r 23
proc recurse {} {
	++ ::q
	puts "Enter \[recurse\] q is $::q"
	if {>= $::q 10} {
		# break						# Will exit script
		# return -code return
		# return -code ok			# This does not 'return' (exit) the block and will stack overflow
		puts "Exit recursion at 10"
		return
	}
	recurse
	puts "Decrementing ::r in recurse during tail recursion exit"
	-- ::r
}
recurse
assert $q {== $_ 10} -v -line #__LINE__
assert $r {== $_ 14} -v -line #__LINE__

##############################################################
undef -glob bar* -nocomplain
undef -glob foo* -nocomplain
undef -glob recurse -nocomplain
unset q -nocomplain
vars 
puts [procs]

newline
textcolor green
puts "* Done *"
textcolor

# EOF #