###################################################################################################
# File:				test_rnd64.tcl
# Author:			M Shaw
# Purpose:			Unit test for [rnd64] 64 bit random numbers using [rnd] x2
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 06/Jun/2022	First version
###################################################################################################
# Notes:
# Munges high and low 32 bit fields to generate numbers in the correct range
# We have to demonstrate via bucketing that the distribution spread is even
# DONE FIX Improve test by sampling decile buckets
# DONE FIX IF (2..10) then mid-range is 10-2= 8 /2 = 4, 2+4=6 ? (lo + hi-lo/2 
# DONE FIX Distribution skews are wrong 
# Testing high range... (2147483647 .. 9223372036854775807)
# 2147483647+ ((9223372036854775807 - 2147483647) /2) => 4611686019501129727
# 	Half range count was 49917 (49.917 %)
# Testing full range... (0 .. 9223372036854775807)
#	Half range count was 24994 (24.994 %)
# DONE FIX Find some way to check the distribution within ranges
# DONE ADD Check range bounds (low and high)
###################################################################################################
# const INT_MAX 9223372036854775807
# const LONG_MAX 2147483647
###################################################################################################

source "defs.tcl"			# Read-in the constants

set loop_max 10000			# Needs a minimum of 100,000 loops for statistics to be in bounds

textcolor yellow
puts "Testing \[rnd64\] for $loop_max loops ..."
textcolor
newline

puts "LONG_MAX $LONG_MAX"
puts "INT_MAX  $INT_MAX"
newline
# const INT_HALF_RANGE      4611686018427387903		# $INT_MAX / 2
#							9223372036854775807		# 19 digits
# /10  =                     922337203685477580
# range =					4611686019501129727
# /2 = 						4611686018427387903

# Create bucket boundary definitions for 64-bit int range...
const INT_HALF_RANGE		4611686019501129727	# LONG_MAX + ((INT_MAX - LONG_MAX) /2)
const INT_HALF_FULL_RANGE	4611686018427387903	#
const INT_QUARTILE_1 [/ $INT_MAX 4]
const INT_QUARTILE_2 [* $INT_QUARTILE_1 2]
const INT_QUARTILE_3 [* $INT_QUARTILE_1 3]
const INT_DECILE_1 [/ $INT_MAX 10]					# => 922337203685477580.7 (18 digits)
const INT_DECILE_2 [* $INT_DECILE_1 2]
const INT_DECILE_3 [* $INT_DECILE_1 3]
const INT_DECILE_4 [* $INT_DECILE_1 4]
const INT_DECILE_5 [* $INT_DECILE_1 5]
const INT_DECILE_6 [* $INT_DECILE_1 6]
const INT_DECILE_7 [* $INT_DECILE_1 7]
const INT_DECILE_8 [* $INT_DECILE_1 8]
const INT_DECILE_9 [* $INT_DECILE_1 9]
const INT_DECILE_10 [* $INT_DECILE_1 10]

###################################################################################################

textcolor yellow
puts "Testing low range... (0 .. $LONG_MAX)"
textcolor

# This test should be fine since we isolate the LONG range and call the standard routine
loop i 0 $loop_max {
	set r [rnd64 1 $LONG_MAX]
	assert $r {!= $_ 0} -line #__LINE__
	assert $r {&& [>= $_ 1] [<= $_ $::LONG_MAX]} -line #__LINE__
}
newline
textcolor green
puts "* LOW RANGE PASS *"
newline
###################################################################################################

textcolor yellow
puts "Testing high range... ($LONG_MAX .. $INT_MAX)"
textcolor
set half_range_count 0

loop i 0 $loop_max {
	set r [rnd64 $LONG_MAX $INT_MAX]
	assert $r {!= $_ 0} -line #__LINE__
	assert $r {>= $_ $::LONG_MAX} -line #__LINE__
	if {> $r $INT_HALF_RANGE} {
		++ half_range_count
	}
}
newline
printf "Half range count was $half_range_count (%.3f %%)\r\n" [* [/ $half_range_count [float $loop_max]] 100]
newline
textcolor green
puts "* HIGH RANGE PASS *"
newline

###################################################################################################
set loop_max 10000
textcolor yellow
puts "Testing full range... (0 .. $INT_MAX)"
textcolor

# Init half range count
set half_range_count 0

# Init quartile buckets
set q1 0
set q2 0
set q3 0
set q4 0

# Init decile buckets
set d1 0
set d2 0
set d3 0
set d4 0
set d5 0
set d6 0
set d7 0
set d8 0
set d9 0
set d10 0

set u5_digits 0
set u10_digits 0
set u15_digits 0
set u20_digits 0

loop i 0 $loop_max {
	set r [rnd64 0 $INT_MAX]
	if {< [string length $r] 5} {
		++ u5_digits
	}
	if {< [string length $r] 10} {
		++ u10_digits
	}
	if {< [string length $r] 15} {
		++ u15_digits
	}
	if {< [string length $r] 20} {
		++ u20_digits
	}
	assert $r {!= $_ 0} -line #__LINE__
	assert $r {<= $_ $::INT_MAX} -line #__LINE__
	if {> $r $INT_HALF_FULL_RANGE} {
		++ half_range_count
	}
	if {< $r $INT_QUARTILE_1} {
		++ q1
	} elseif {< $r $INT_QUARTILE_2} {
		++ q2
	} elseif {< $r $INT_QUARTILE_3} {
		++ q3
	} else {
		++ q4
	}
	if {< $r $INT_DECILE_1} {
		++ d1
	} elseif {< $r $INT_DECILE_2} {
		++ d2
	} elseif {< $r $INT_DECILE_3} {
		++ d3
	} elseif {< $r $INT_DECILE_4} {
		++ d4
	} elseif {< $r $INT_DECILE_5} {
		++ d5
	} elseif {< $r $INT_DECILE_6} {
		++ d6
	} elseif {< $r $INT_DECILE_7} {
		++ d7
	} elseif {< $r $INT_DECILE_8} {
		++ d8
	} elseif {< $r $INT_DECILE_9} {
		++ d9	
	} else {
		++ d10	
	}
}
newline

printf "Half range count was $half_range_count (%.3f %%)\r\n" [* [/ $half_range_count [float $loop_max]] 100]
printf "Quartiles were $q1, $q2, $q3, $q4\r\n"
printf "q1 %.3f %%\r\n" [* [/ $q1 [float $loop_max]] 100.0]
printf "q2 %.3f %%\r\n" [* [/ $q2 [float $loop_max]] 100.0]
printf "q3 %.3f %%\r\n" [* [/ $q3 [float $loop_max]] 100.0]
printf "q4 %.3f %%\r\n" [* [/ $q4 [float $loop_max]] 100.0]
newline

printf "Deciles are: 1: $d1, 2: $d2, 3: $d3, 4: $d4, 5: $d5, 6: $d6, 7: $d7, 8: $d8, 9: $d9, 10: $d10\r\n"
newline

printf "d1 %.3f %%\r\n" [* [/ $d1 [float $loop_max]] 100.0]
printf "d2 %.3f %%\r\n" [* [/ $d2 [float $loop_max]] 100.0]
printf "d3 %.3f %%\r\n" [* [/ $d3 [float $loop_max]] 100.0]
printf "d4 %.3f %%\r\n" [* [/ $d4 [float $loop_max]] 100.0]
printf "d5 %.3f %%\r\n" [* [/ $d5 [float $loop_max]] 100.0]
printf "d6 %.3f %%\r\n" [* [/ $d6 [float $loop_max]] 100.0]
printf "d7 %.3f %%\r\n" [* [/ $d7 [float $loop_max]] 100.0]
printf "d8 %.3f %%\r\n" [* [/ $d8 [float $loop_max]] 100.0]
printf "d9 %.3f %%\r\n" [* [/ $d9 [float $loop_max]] 100.0]
printf "d10 %.3f %%\r\n" [* [/ $d10 [float $loop_max]] 100.0]
newline

set pc 6 # 6% in order to guarantee that tests won't randomly fail
puts "Testing for quartile (25%) distribution in range +/- $pc%..."
assert [inrange $q1 [/ $loop_max 4] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $q2 [/ $loop_max 4] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $q3 [/ $loop_max 4] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $q4 [/ $loop_max 4] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
newline

set pc 10
# We can lower to 6% with 100,000 loops
# The more loops the more tight the test can be
puts "Testing for decile (10%) distribution in range +/- $pc%..."
assert [inrange $d1 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d2 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d3 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d4 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d5 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d6 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d7 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d8 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d9 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
assert [inrange $d10 [/ $loop_max 10] $pc] {== $_ 1} -v -line #__LINE__		# +/-1%
newline

textcolor
puts "Under 20 digits = $u20_digits"
puts "Under 15 digits = $u15_digits"
puts "Under 10 digits = $u10_digits"
puts "Under 5 digits = $u5_digits"
newline

textcolor white
puts "Decile Graph"
puts "------------"
textcolor blue black
# puts "[/ $d1 [float $loop_max]]"
puts "d1:  [string repeat [chr 254] [* 600 [/ $d1 [float $loop_max]]]]"
puts "d2:  [string repeat [chr 254] [* 600 [/ $d2 [float $loop_max]]]]"
puts "d3:  [string repeat [chr 254] [* 600 [/ $d3 [float $loop_max]]]]"
puts "d4:  [string repeat [chr 254] [* 600 [/ $d4 [float $loop_max]]]]"
puts "d5:  [string repeat [chr 254] [* 600 [/ $d5 [float $loop_max]]]]"
puts "d6:  [string repeat [chr 254] [* 600 [/ $d6 [float $loop_max]]]]"
puts "d7:  [string repeat [chr 254] [* 600 [/ $d7 [float $loop_max]]]]"
puts "d8:  [string repeat [chr 254] [* 600 [/ $d8 [float $loop_max]]]]"
puts "d9:  [string repeat [chr 254] [* 600 [/ $d9 [float $loop_max]]]]"
puts "d10: [string repeat [chr 254] [* 600 [/ $d10 [float $loop_max]]]]"
newline

textcolor green
puts "* FULL RANGE PASS *"
textcolor
newline

loop i 0 2001 {
	set r [rnd64 0 $INT_MAX]
}

###################################################################################################
# This will also 'unread' defs.tcl
unset d* q* u* -glob -nocomplain
unset u* -glob -nocomplain
unset INT* -glob -nocomplain -const
unset *_BITS -glob -nocomplain -const
unset UINT_* -glob -nocomplain -const
unset *_MAX -glob -nocomplain -const
unset *_MIN -glob -nocomplain -const
unset half_range_count i loop_max pc r  -nocomplain

vars -const
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #