##############################################################################################
# File:				test_safe_recursion.tcl
# Author:			M Shaw
# Purpose:			Test file for save recursion limits
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 04/Jul/2017	First version
# 1.01 29/Jun/2018	Updated for silent exit on breakpoint on recurse limit
# 1.02 08/Jan/2023	Updated for commonly configured local recursion max (1000)
##############################################################################################
# Notes:
# Windows Task Manager will show a max memory amount allocated of about 5Mb at stack overflow
# The memory is restored and the stack does unwind and there are no cumulative leaks
# AFAIK Windows registers this max allocation and retains it as a high water mark
# Tested memory-stable at 100,000 recursive [proc] stack overflows
# Apparent leaks show at around 5600Kb @ 100000
##############################################################################################
# INI [config]->MaxRecursion now optionally sets max recursion 
# A const is set in the code which guards max recursion and traps with $errorMsg
# DONE FIX recursion quits at 98 with no exception raised with option breakpoint on
# 	Was a stupid quiet die() due to stack overflow in class Stack::
##############################################################################################
# Compile with linker 
#    /stack:0x300000 
# or /stack:0x200000 
# v1.24b - recursion guard broken
# v1.24a
# Testing guard on safe recursion limits
# IGNORE $::errorMsg was: 'Stack overflow in proc 'foo' at level 10000' at 0
# assert: Line 49: Assertion failed: '0' != '== $_ 10000'
# First-chance exception in ticol.exe: 0xC00000FD: Stack Overflow.
# 	DONE FIX stack overflow should not generate a fatal exception
#		Return PICOL_ERR and deny + set return val to "stack overflow"
# 7353, 568 bytes per call
# Reduced to 572
##############################################################################################
option expression on
option echo on
#option breakpoint on

#assert 0 {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
# We usually have this set at 1000 via INI
set tcl_recursion_max 1000			# As compiled in or set by ini config (internal default is 5000)

textcolor yellow
puts "Testing guard on safe recursion limits..."
textcolor
newline
halt

puts "Line: "#__LINE__
unset foo -nocomplain
undef foo -nocomplain 

set highest_level 0									# Instability may report that this var does not exist in catch

puts "Line: "#__LINE__
proc foo {} {
	#option breakpoint push off
	#puts [option breakpoint]
	#puts [info level]
	#try {
		#set ${::highest_level}q Hello
set ::highest_level [max $::highest_level [info level]]
	#} catch {
	#	puts "Fail to assign in foo"
	#}
	#puts "$::highest_level"
	#puts "$::highest_level available mem:[info memavail]"
	#walk
	#vars -const -level 0	
	# pause
	foo							# Recursive proc
	#option breakpoint pop		# Will reset as recursion unwinds
}

textcolor yellow
puts "Calling local proc \[foo\] recursively"
textcolor
newline

# Call foo recursively n times
puts "Line: "#__LINE__
# FIX proc overflow does not set errorMsg
option echo off
halt
time {
	set q 0
	try {
		foo						# Recurse to overflow error trap
	} catch {					# Should catch safely and not GPF
		/*
		textcolor darkyellow
		puts "TRAP: \$::errorMsg: '$::errorMsg' at level:$highest_level"
		textcolor
		newline	# */
		set ${highest_level}::q 1
	}
} 100000

# If we got here we didn't crash. errorMsg will trap and catch set if working correctly
puts "highest_level was $highest_level and tcl_recursion_max is $tcl_recursion_max"
assert $highest_level {== $_ $tcl_recursion_max} -v -line #__LINE__
#assert $q {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
vars

# Retry ...
set q 0
try {
	foo						# Recurse to overflow error trap
} catch {					# Should catch safely and not GPF
	textcolor darkyellow
	puts "TRAP: \$::errorMsg: '$::errorMsg' at level:$highest_level"
	textcolor
	newline
	set ${highest_level}::q 1
}

undef foo -nocomplain 
unset q highest_level tcl_recursion_max -nocomplain

option breakpoint off


###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #