#####################################################################################
# File: 			test_static.tcl
# Purpose:			Tests for [static] including unlinkage, relinkage and protection
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Aug/2019	First version
# 1.01 26/Sep/2019	Updated with asserts
# 1.02 10/Feb/2022	Updated for protected statics and deletion methods
#####################################################################################
# Notes:
# ADD recursive fixups to level N=10
#
# Static variables are 'protected' against deletion using [unset]. They may be 
# assigned to as normal but have the scope lifespan of the running script
# and they cannot be [unset]
#
# Static variables may be defined either in [proc] scope or root scope
# Static variables may or may not have an initiator value
#
# Static variables are stored in the root namespace and have the following name
# format: 
# 		<procname>::<varname> when defined in a proc;	e.g. foo::x
# 		<varname> when defined in root scope			e.g. x
#
# Within a proc the static is addressed via any local alias but may also be
# addressed directly via the global scope alias using the "::" prefix
# This should be avoided since any local changes will overwrite the global state
#
# When defined within a [proc] static variables remain for the life of a script
# even if the proc is destroyed using [undef]
# If a new [proc] is subsequently defined after destruction of one which 
# previously defined a static, the new [proc] does not inherit the old variable(s)
# 
# A [sub] will behave in precisely the same way as a [proc] WRT [static]
# 
# [static <varnanme> -unset] understands standard vars, arrays, stack, struct, dict
#####################################################################################

#####################################################################################
# Protect/unset stack
stack create st 10
static st
assert [is_static st] {== $_ $true} -v -line #__LINE__
static st -unset
assert [is_set st] {== $_ $false} -v -line #__LINE__

#####################################################################################
# Protect/unset dictionary
set dct [dict create foo bar]
static dct
assert [is_static dct] {== $_ $true} -v -line #__LINE__
static dct -unset
assert [is_set dct] {== $_ $false} -v -line #__LINE__

#####################################################################################
# Protect/unset struct
struct create st {a 10}
static st
assert [is_static st] {== $_ $true} -v -line #__LINE__
static st -unset
assert [is_set st] {== $_ $false} -v -line #__LINE__

vars
newline

#####################################################################################
# Protect/unset standard var
set a 10
static a 23		# Initialisation is skipped where a var already exists and is thus initialised
assert $a {== $_ 10} -v -line #__LINE__
assert [is_static a] {== $_ $true} -v -line #__LINE__
set i 0
try {
	unset a
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [is_set a] {== $_ $true} -v -line #__LINE__
static a -unset
assert [is_set a] {== $_ $false} -v -line #__LINE__

# exit
#####################################################################################

puts "Line: "#__LINE__
puts "procs->[procs]"
undef foo -nocomplain
proc foo {} {
	textcolor yellow
	puts "Enter proc foo @"#__LINE__
	static a 
	textcolor white
	#puts "In proc \[is_static foo::a\] ? [is_static foo::a] @"#__LINE__
	#puts "In proc \[is_static ::foo::a\] ? [is_static ::foo::a] @"#__LINE__
	#puts "In proc \[is_static a\] ? [is_static a] @"#__LINE__
	assert [info exists a] {== $_ $::true} -v -line #__LINE__
	assert [info exists ::foo::a] {== $_ $::true} -v -line #__LINE__
	set a 2
	assert $a {== $_ 2} -v -line #__LINE__
	assert $::foo::a {== $_ 2} -v -line #__LINE__
	textcolor cyan
	puts "In proc \[is_static foo::a\] ? [is_static foo::a] @"#__LINE__
	puts "In proc \[is_static ::foo::a\] ? [is_static ::foo::a] @"#__LINE__
	puts "In proc \[is_static a\] ? [is_static a] @"#__LINE__
	textcolor
}

puts "Line: "#__LINE__
foo
newline
textcolor magenta
puts "In root \[is_static foo::a\] ? [is_static foo::a]"
puts "In root \[is_static ::foo::a\] ? [is_static ::foo::a]"
puts "In root \[is_static a\] ? [is_static a]"
textcolor
newline

puts "Line: "#__LINE__
#set foo::a 2
assert [info exists foo::a] {== $_ $::true} -v -line #__LINE__
assert $foo::a {== $_ 2} -v -line #__LINE__
set foo 69
assert $foo {== $_ 69} -v -line #__LINE__
vars
puts "Unsetting (::)foo::a @"#__LINE__
unset ::foo::a -nocomplain		# Protected
assert [info exists ::foo::a] {== $_ $::true} -v -line #__LINE__
newline

puts "Unsetting foo::a @"#__LINE__
unset foo::a -nocomplain		# Protected
catch {unset f* -glob -const}
#vars
assert [info exists ::foo::a] {== $_ $::true} -v -line #__LINE__
assert [info exists foo::a] {== $_ $::true} -v -line #__LINE__
newline


#####################################################################################
puts "Line: "#__LINE__
unset x -nocomplain
static x -unset -nocomplain
static x 23 23
assert $x {== $_ 23} -v -line #__LINE__
set i 0
try {
	unset x
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
set x 32
assert $x {== $_ 32} -v -line #__LINE__
set i 0
try {
	unset x
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
newline

#####################################################################################
# Define static in proc, then undef proc and redefine to check static relinkage
#####################################################################################
puts "Line: "#__LINE__
textcolor yellow
puts "Validating proc static declaration creation, unlinkage and re-linkage after proc \[undef\] ..."
textcolor

undef bar -nocomplain
proc bar {} {
	puts "Enter proc bar"
	static x 23
	static s
	set s $::qbf
}
bar
assert $bar::x {== $_ 23} -v -line #__LINE__
assert $bar::s {eq $_ $qbf} -v -line #__LINE__
undef bar -nocomplain

proc bar {} {
	puts "Line: "#__LINE__
	newline
	textcolor magenta
	puts "Enter proc [info procname] (2nd incarnation)"
	textcolor	set i 0
	try {
		puts $x			# validate that existing var linkage is now gone
	} catch {
		set i 1
	}
	puts "Line: "#__LINE__
	assert $i {== $_ 1} -v -line #__LINE__
	assert $::bar::x {== $_ 23} -v -line #__LINE__			# Validate old global
	
	static x 												# Re-declare without assignment
	assert $x {== $_ 23} -v -line #__LINE__					# Validate old global
	puts "bar:: local x = $x"
	puts "Line: "#__LINE__
	set x 123
	assert $x {== $_ 123} -v -line #__LINE__				# Validate old global
	assert $::bar::x {== $_ 123} -v -line #__LINE__			# Validate old global
	try {
		puts $s			# validate that existing var linkage is now gone
	} catch {
		set i 1
	}
	puts "Line: "#__LINE__
	assert $i {== $_ 1} -v -line #__LINE__
	static s
	assert $s {eq $_ $::qbf}
	set ::bar::s "This should not survive local s assignment"	# Comment out line below to test!
	set s "Hello world"
	textcolor magenta
	puts "Exit proc [info procname] (2nd incarnation)"
	textcolor
}
puts "Line: "#__LINE__
bar																# Call proc [bar]
newline

puts "Line: "#__LINE__
textcolor yellow
puts "Validating prior static in global scope has been updated by proc local var"
textcolor
assert $bar::x {== $_ 123} -v -line #__LINE__					# Validate change in old global in global scope
assert $bar::s {eq $_ "Hello world"} -v -line #__LINE__			# Validate change in old global in global scope

#####################################################################################
# static -unset
#####################################################################################
puts "Line: "#__LINE__
assert [is_set foo::a] {== $_ 1} -v -line #__LINE__
static foo::a -unset
assert [is_set foo::a] {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
assert [is_set bar::s] {== $_ 1} -v -line #__LINE__
static bar::s -unset
assert [is_set bar::s] {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
assert [is_set bar::x] {== $_ 1} -v -line #__LINE__
static bar::x -unset
assert [is_set bar::x] {== $_ 0} -v -line #__LINE__

vars

# exit
#####################################################################################
# Recursive
# Show that a [proc] at highest nested level will correctly set a static var on
# recursive "unwind"
#####################################################################################
puts "Line: "#__LINE__
# Note that we can't undef foo::a
if {is_set foo::b} {
	die "foo::b exists at line "#__LINE__
}

puts "Line: "#__LINE__
undef foo -nocomplain
static b -unset -nocomplain
proc foo {x} {
	static b 1 	# -nocomplain			# Don't complain about bad assignment if b exists
	textcolor magenta
	puts "ENTER recursive foo: foo->b is $b"
	textcolor
	++ b
	if {> $b 10} {
		return
	}
	foo
}

puts "Line: "#__LINE__
foo
puts "Line: "#__LINE__
assert $foo::b {== $_ 11} -v -line #__LINE__				# Loop variable
assert [is_set foo::b] {== $_ 1} -v -line #__LINE__
puts "Line: "#__LINE__
static foo::b -unset
assert [is_set foo::b] {== $_ 0} -v -line #__LINE__

vars

# exit
#####################################################################################
# Checking var with whitespace
#####################################################################################
#/* # BORKED!
puts "Line: "#__LINE__
undef foo -nocomplain
proc "my proc" {} {
	static a 32
	assert [info exists a] {== $_ $::true} -v -line #__LINE__
	set a 23
}

"my proc"				# Call it
undef "my proc"

assert ${my proc::a} {== $_ 23} -v -line #__LINE__

puts "Line :"#__LINE__
if {[defined "my proc"]} {
	die "Failed at line "#__LINE__
} else {
	puts "Proc \"my proc\" unregistered sucessfully"
}

assert [info exists "my proc::a"] {== $_ $::true} -v -line #__LINE__

vars

# FIX
#assert ${my proc}::a {== $_ 32} -v -line #__LINE__

# */
####################################################################################
# Check [static -unset] on cleanup
####################################################################################
# FIX clean up static array definitions and cleanup
#static a(0) 10
array unset a -nocomplain
array create a
assert [is_static a] {== $_ $false} -v -line #__LINE__
#/*
# Have not yet finalised thus behaviour
set i 0
try {
	static a 234	# FIX [static] Should not allow initialiser for static <arrayvar>
} catch {
	set i 1
}
assert $i {== $_ $true} -v -line #__LINE__
# */

set i 0
try {
	static a 234 -nocomplain	# FIX [static] Should not allow initialiser for static <arrayvar>
} catch {
	set i 1
}
assert $i {== $_ $false} -v -line #__LINE__

set i 0
try {
	static a					# Test with no assignment (should succeed and return i==0)
} catch {
	set i 1
}
assert $i {== $_ $false} -v -line #__LINE__
assert [is_static a] {== $_ $true} -v -line #__LINE__
puts "a is of type $a"
vars
newline

set i 0
try {
	unset a						# Raise static is protected error (can't use [unset] with a static)
} catch {
	set i 1
}
assert $i {== $_ $true} -v -line #__LINE__

# exit
puts "Line: "#__LINE__
textcolor darkyellow
puts "Unsetting {my proc::a} using \[unset\] ... (should fail)"
unset {my proc::*} -glob -nocomplain

assert [is_set "my proc::a"] {== $_ 1} -v -line #__LINE__
static  "my proc::a" -unset
assert [is_set  "my proc::a"] {== $_ 0} -v -line #__LINE__
newline

assert [is_static x] {== $_ 1} -v -line #__LINE__
static x -unset 
assert [is_set  x] {== $_ 0} -v -line #__LINE__

#####################################################################################
# Test static linking and array element assignment + update cascade
#####################################################################################
undef foo -nocomplain
# Declare a proc and link + create + set 2 array elements after it has been created
proc foo {} {
	static a			# Fixup linkage and create foo::a
}
set foo::a(0) 23
set foo::a(1) 24
assert $foo::a(0) {== $_ 23} -v -line #__LINE__
assert $foo::a(1) {== $_ 24} -v -line #__LINE__
newline

# Now re-declare the proc and overwrite the global static array elements using this new proc
undef foo -nocomplain
proc foo {} {
	static a
	set a(0) "Foo_0"
	set a(1) "Foo_1"
}
set foo::a(2) "Foo_2"
foo
vars
array walk foo::a
puts "a(0)=$foo::a(0)"
assert $foo::a(0) {eq $_ "Foo_0"} -v -line #__LINE__
puts "a(1)=$foo::a(1)"
assert $foo::a(1) {eq $_ "Foo_1"} -v -line #__LINE__
puts "a(2)=$foo::a(2)"
assert $foo::a(2) {eq $_ "Foo_2"} -v -line #__LINE__
# exit

#####################################################################################
# Test unset on a promoted 'static' array as we exit (was promoted above)
#####################################################################################
# Check we can still destroy an standard array using unset
array create ar
assert [is_set ar] {== $_ $true} -v -line #__LINE__
array unset ar
assert [is_set ar] {== $_ $false} -v -line #__LINE__

puts "Line: "#__LINE__
set i 0
try {
	array unset a -nocomplain	# Will fail since a is now protected as a static
} catch {
	set i 1
}
assert $i {== $_ $true} -v -line #__LINE__

puts "Line: "#__LINE__
static a -unset					# Will delete an array since internal call to [unset] understands <array> type
puts "Line: "#__LINE__
vars
puts "Line: "#__LINE__
assert [is_static foo::a] {== $_ $true} -v -line #__LINE__
#unset foo::a
static foo::a -unset 			# Unset 

puts "Line: "#__LINE__
textcolor
unset i s x -nocomplain
newline

#####################################################################################
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #