#################################################################################
# File: 			test_store.tcl
# Purpose:			Tests for [store] command
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 ??/Dec/2017	First version
# 1.01 05/Jan/2018	Updated
# 1.02 27/Jan/2018	Added interleaved and -var x tests
#################################################################################
# Notes:
#################################################################################
# FIX a.txt has CRLF, b.txt does not
#################################################################################

# FIX [split] currently removes all empty lines via removing all mask chars

puts "Line :"#__LINE__

store get		# Clear the buffer

# A test string taken from the man pages
set s "\
.architecture
.ticol architecture
 Ticol Tcl architecture

This is a simplified block diagram of the unthreaded Ticol Tcl architecture

..orange
	+--+----------------------------------------------------------+
	| 9| Tcl Command Line Interpreter (CLI) | EXE Command Line    |
	+--+---------------------------+------------------------------+
	| 8| tcl script                | tcx script                   |
	+--+                           +------------------------------+
	| 7|                           | Obfuscation Decoder/Encoder  |
	+--+---------------------------+------------------------------+
	| 6| Macro Preprocessor (MPP)                                 |
	+--+----------------------------------------------------------+
	| 5| Registered commands, \[proc\] registration, \[undef\]        |
	+--+---------------------------+------------------------------+
	| 4| \[proc\]                    | Known Tcl commands           |
	+--+---------------------------+------------------------------+
	| 3| \[eval\]                                                   |
	+--+---------------------------+------------------------------+
	| 2| Tcl Parser                | Registered \[lib\] commands    |
	|  |                           | \[lib calldlll\]               |
	+--+---------------------------+------------------------------+
	| 1| ticol.exe                 | External DLLs                |
	+--+---------------------------+------------------------------+
..end

The CLI or ticol.exe command line load plaintext or obfuscated scripts
These are processed before execution by the Macro Preprocessor (MPP).
The resulting script content may register \[proc\] instances and interact
with any registered command

The Macro PreProcessor (MPP) performs a number of tasks, including
stripping out comment lines, processing the macro language and optimising
\[calc\] expressions as well as performing basic sanity checks on the
script

All commands are evaluated internally by the internal 'eval' function
which may also be called directly using \[eval\]. Eval parses the script
content and breaks it up into commands and arguments. The \[lib\]
command can load exported Tcl commands into the root namespace where
they may be called the same as any other command (See: lib interface)

See also: Ticol, Tcl, lib, Ticol, lib interface
..end
.........1.........2.........3.........4.........5.........6.........7.....|...8
.{*}
.argument expansion
.expand
 {*} - Argument Expansion Operator

	command {*}{word ?word...?}

Translates a list into an argument array for a single command

CAUTION: Use with care!

If a word is prefixed with {*} followed by a non-whitespace character then
the leading {*} is discarded and the rest of the word is parsed and 
substituted as for any normal Tcl word. (Word is analogous to a parameter or
argument). The substitution process is equivalent to calling \[subst\]

After substitution is complete, the word is then parsed as a Tcl list 
without command or variable substitutions. Backslash substitutions are
performed as is normal for a list and individual internal words may be 
surrounded by either braces or double-quote characters. Its words are then
added as individual arguments to the command being substituted

Be extremely cautious not to accidentally use an unquoted \"{*}\" string
within your code where you do not intend it to expand arguments. e.g.
don't use...

..code
	puts \[unescape \"\[string repeat 80 \{*\}]\\\\highlight0\\\\cf0\\\\par \"\]
..end

as this will attempt to expand within the nested call to \[string repeat\]
and will lead to impossible to debug errors

Supplying a whitespace character (tab or space) after {*} is unlawful

For instance:

..code
	cmd a {*}{b \[c\]} d {*}{\$e f {g h}}
..end

is equivalent to supplying:

..code
	cmd a b {\[c\]} d {\$e} f {g h}
..end

And results in the effective arguments:

	a
	b
	{\[c\]}
	d
	{\$e}
	f
	g h

 ...being passed to a command

Example:

..code
	set a \"puts Hello\"     \# Set var to a command with args
	\$a                     \# Attempt to dereference to a command with args
	{*}\$a                  \# Correct method of dereferencing
..end

Results

	puts' is not recognized as an internal or external command
	Hello

Example:

..code
	# Unset all variables 'a' to 'z' (supplying individual arguments)
	unset {*}\[split \[range a-z\]\] -nocomplain
..end

Example:

..code
	set square {lmap {x {expr \$x * \$x}}}   \# Lambda like call
	puts \[{*}\$square 42\]
..end

Result:

	1764

See also: subst
..end
"
# Note that the final CRLF is required for the test to pass */


set s [unescape $s]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line :"#__LINE__

array unset a -nocomplain

# FIX not returning empty lines
split $s "\r\n" -array a -literal
#array walk a

puts "Line :"#__LINE__

##############################################################################
# FIX problem. We assume we always add CRLF when tokenising to array and back,
# but this may not be the case for the last line
# which could end up with no CRLF at the end. We can't tell the difference!
# So:        ..end"  will fail
# but:       ..end\r\n"  will pass
##############################################################################
for {set i 0} {< $i [array size a]} {++ i} {
	# if {eq $a($i) ""} [die "empty line at $i"]
	#puts "test_store:store add->'$a($i)'"
	store add "$a($i)\r\n"							# We have to re-insert the CRLFs stripped by array
}

puts "Results"
puts "------------------------------------------------------"
puts "Array size:         [array size a]"
puts "original length is: [string length $s]"
puts "Store length is:    [store length]"
puts "Difference:         [- [string length $s] [store length]]"
puts "------------------------------------------------------"

puts "Line :"#__LINE__
set stored_length [store length]
set t [store get]			# This will clear [store length]
puts "Store retrieved length of t is [string length $t]"

# FIX has not CRLFCRLF sequences
#puts "$t"

puts "Line :"#__LINE__
writefile "a.txt" $s			# Original string in s as declared here
writefile "b.txt" $t			# Stored version in t

puts "Line :"#__LINE__
set j [string length $s]
set k [string length $t]
puts "String length s $j; string length t $k"
textcolor yellow

puts "Line :"#__LINE__
try {
	for {set i 0} {< $i [min $j $k]} {++ i} {
		#puts '[index $s $i]' -nonewline
		if {ne [index $s $i] [index $t $i]} {
			textcolor red
			puts "\r\n\r\nContent mismatch at char $i @line: "#__LINE__
			puts "char $i '[asc [index $s $i]]' NE '[asc [index $t $i]]'"
			textcolor cyan
			puts "s:'[mids $s $i 70]'..."
			puts "t:'[mids $t $i 70]'..."
			stop
		}
	}
} catch {
	textcolor red
	puts "Error at $i '[mids $s $i]'"
	textcolor
}

puts "j $j k $k"
if {> $k $j} {
	puts "Tail is '[tohex [mids $t [+ $j 1]]]'"
}

puts "Line :"#__LINE__
puts "String length is: [string length $s] Store length is '$stored_length'"
assert [eq [string length $s] $stored_length] {== $_ $true} -v -line #__LINE__
assert [eq $s $t] {== $_ $::true} -v -line #__LINE__

# using global store
set s ""
store get 
store add "Hello"
store add " world"
store get -var s
assert $s {eq $_ "Hello world"} -v -line #__LINE__

set s ""
store get p
store add p "Hello"
store add p " world"
store get p -var s
assert $s {eq $_ "Hello world"} -v -line #__LINE__

# Interleaved store using default and q (return via body)
set p ""
set q ""
set s ""
set t ""
store get 							# Ensure buffer is cleared
store get q							# Ensure buffer is cleared
store add "Hello"
store add q "The quick "
store add " world"
store add q "brown fox "
set s [store get]
store add q "jumps over the lazy dog"
assert $s {eq $_ "Hello world"} -v -line #__LINE__
set t [store get q]
assert $t {eq $_ $::qbf} -v -line #__LINE__

# Interleaved store using default and q (return vars s, t)
set p ""
set q ""
set s ""
set t ""
store get 							# Ensure buffer is cleared
store get q							# Ensure buffer is cleared
store add "Hello"
store add q "The quick "
store add " world"
store add q "brown fox "
store get -var s
store add q "jumps over the lazy dog"
assert $s {eq $_ "Hello world"} -v -line #__LINE__
store get q -var t
assert $t {eq $_ $::qbf} -v -line #__LINE__
store clear


# Interleaved store using vars p and q
set p ""
set q ""
set s ""
set t ""
store get p							# Ensure buffer is cleared
store get q							# Ensure buffer is cleared
store add p "Hello"
store add q "The quick "
store add p " world"
store add q "brown fox "
store get p -var s						# p will cease to exist / be a valid reference
store add q "jumps over the lazy dog"

# FIX [store get nosuchvar] will fail
puts "Line: "#__LINE__
puts [store list]

puts "Line: "#__LINE__
if {store exists p} {
	puts [store length p]
} else {
	puts "Storage item p does not exist"
}
puts "Line: "#__LINE__
puts [store length q]

assert $s {eq $_ "Hello world"} -v -line #__LINE__
store get q -var t
assert $t {eq $_ $::qbf} -v -line #__LINE__
store clear

##################################################################
puts "Line: "#__LINE__

store add $qbf
store add foo bar
store add "foo bar" $qbf
textcolor yellow
puts "'[store list]' @"#__LINE__
textcolor
assert [store list] {eq $_ "{} 43 foo 3 {foo bar} 43"} -v -line #__LINE__
store add "foo bar" $qbf
textcolor yellow
puts "'[store list]'"
textcolor
assert [store list] {eq $_ "{} 43 foo 3 {foo bar} 86"} -v -line #__LINE__
foreach {l r} [store list] {
	puts "{$l} -> $r"
}
store clear
assert [store list] {eq $_ ""} -v -line #__LINE__

##################################################################
newline
puts "Line: "#__LINE__
textcolor yellow
puts "Test \[store clear\] and \[store exists {}\]"
textcolor
store clear
store add $qbf
assert [store exists {}] {== $_ 1} -v -line #__LINE__
assert [store exists] {== $_ 1} -v -line #__LINE__
store clear
assert [store exists {}] {== $_ 0} -v -line #__LINE__

store add foo bar
assert [store exists foo] {== $_ 1} -v -line #__LINE__
store clear
assert [store exists foo] {== $_ 0} -v -line #__LINE__
store clear

# Man example
store add $qbf
store add foo $qbf
store add "foo bar" Hi
puts "'[store list]'"


store clear					# Clear all storage

assert [store list] {eq $_ {}} -v -line #__LINE__

array unset a -nocomplain
unset i j k l p q r s stored_length t -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Pass *"
textcolor

# EOF #