###########################################################################################
# File:				test_subst.tcl
# Author:			M Shaw
# Purpose:			Test [subst]
#
# Versions:
# 1.00 06/Sep/2019 First version
# 1.01 26/Sep/2019 Added more embedded command tests
###########################################################################################
# Notes:
# Leak test (loop_tests.tcl) no leaks
# -nocommands only applies to braced commands which can be interpreted by [subst]
# Unbraced commands are always called
###########################################################################################
/*
subst ?-nobackslashes? ?-nocommands? ?-novariables? string

DESCRIPTION
This command performs variable substitutions, command substitutions, and backslash 
substitutions on its string argument and returns the fully-substituted result. 
The substitutions are performed in exactly the same way as for Tcl commands. 
As a result, the string argument is actually substituted twice, once by the Tcl parser 
in the usual fashion for Tcl commands, and again by the subst command. 

If any of the -nobackslashes, -nocommands, or -novariables are specified, then the 
corresponding substitutions are not performed. For example, if -nocommands is specified, 
command substitution is not performed: open and close brackets are treated as ordinary 
characters with no special interpretation.

Note that the substitution of one kind can include substitution of other kinds. 
For example, even when the -novariables option is specified, command substitution is 
performed without restriction. This means that any variable substitution necessary to 
complete the command substitution will still take place. Likewise, any command 
substitution necessary to complete a variable substitution will take place, even when 
-nocommands is specified. See the EXAMPLES below.

set a 44
subst {xyz {$a}}
returns “xyz {44}”, not “xyz {$a}” and the script

set a "p\} q \{r"
subst {xyz {$a}}
returns “xyz {p} q {r}”, not “xyz {p\} q \{r}”.

*/
###########################################################################################


#/*
set i 0
puts [subst "abc,[puts hello],def"]
textcolor darkyellow
subst "abc,[set i 1],def"
textcolor
assert $i {== $_ 1} -v -line #__LINE__

set a hello
textcolor darkyellow
puts "'[subst "abc $a def"]' @"#__LINE__
textcolor
assert [subst "abc $a def"] {eq $_ "abc hello def"} -v -line #__LINE__

textcolor darkyellow
puts "'[subst "abc {$a} def"]' @"#__LINE__
textcolor
assert [subst "abc {$a} def"] {eq $_ "abc {hello} def"} -v -line #__LINE__

##############################################################################################################
# Outer braces block resolution of vars - [subst] should not even see these ?
# In fact it doesn't see the outer braces but they will have prevented var resolution
# It will see "abc {$a} def" whereas if no outer {} then it sees "abc hello def"
# So, as per ActiveState docs, substitution is performed twice, once by the parser, second by subst
# This means that subst is performing no useful secondary var substitution!!!
# Proper fix means fixing picolSubst to parse and replace vars directly rather than current kludge to call picolEval
##############################################################################################################

textcolor darkyellow
puts "'[subst {abc {$a} def}]' @"#__LINE__
textcolor
assert [subst "abc {$a} def"] {eq $_ "abc {hello} def"} -v -line #__LINE__
# */

#/*
puts "Line: "#__LINE__
set a "p\} q \{r"
puts "'$a'"
echo "-> '[subst {xyz {$a}}]'"
# DONE FIX replaceVarsF must unescape \{ and \}
assert [subst {xyz {$a}}] {eq $_ "xyz {p} q {r}"} -v -line #__LINE__

# subst -> picolEval -> picolList
# returns “xyz {p} q {r}”, not “xyz {p\} q \{r}”.
# FIX We get xyz {{p} q {r}}		# We are gaining extra braces!!
# */

# DONE ADD test varname containing spaces
set "my var" 123
puts ${my var}
puts "-> '[subst {xyz {${my var}}}]'"	# FIX!!
puts "-> '[subst {xyz ${my var}}]'"		# FIX!!
assert [subst {xyz ${my var}}] {eq $_ "xyz 123"} -v -line #__LINE__


###########################################################################################
# Embedded commands
puts [subst "abc,[puts Hello!],def"]		# Correct
assert [subst "abc,[puts Hello!],def"] {eq $_ "abc,,def"} -v -line #__LINE__
# FIX ActiveState calls command even when protected by braces
#% puts [subst {abc,[puts hello],def}]
#hello
#abc,,def
puts "Line: "#__LINE__
puts [subst {abc,[puts hello],def}]	# -> "abc,[puts hello],def"

# Insert via command return
assert [subst {abc [+ 2 2] def}] {eq $_ "abc 4 def"} -v -line #__LINE__
assert [subst {abc [+ 2 2] [* 7 9] def}] {eq $_ "abc 4 63 def"} -v -line #__LINE__
assert [subst {abc [+ 2 2] [* 7 9] [string left $qbf 3] def}] {eq $_ "abc 4 63 The def"} -v -line #__LINE__
assert [subst {abc [+ 2 2] def} -nocommands] {eq $_ {abc [+ 2 2] def}} -v -line #__LINE__

# Insert via proc return
undef foo -nocomplain
proc foo {} {return 23}
assert [subst {abc [foo] def}] {eq $_ "abc 23 def"} -v -line #__LINE__
undef foo


puts "Line: "#__LINE__
puts [subst -nocommands {abc,[puts Hello!],def}]		# -nocommand is ineffective if unbraced

# FIX Compare fails due to backslash issues
# DONE ADD -nobackslashes argument 
echo [subst -nocommands -nobackslashes {abc,[puts Hello!],def}]
puts "Line: "#__LINE__
# assert rvalue *MUST* be braced here rather than quoted to prevent command interpretation
assert [subst -nocommands -nobackslashes {abc,[puts Hello!],def}] {eq $_ {abc,[puts Hello!],def}} -v -line #__LINE__

# Ensure null returns are corrrect
set s "a b {} c d"
assert [lindex $s 2] {eq $_ ""} -v -line #__LINE__
assert [subst [lindex $s 2]] {eq $_ ""} -v -line #__LINE__

###########################################################################################
# FIX? braces have no effect. (ActiveState is same)
puts [subst -nocommands "{abc,[puts Hello!],def}"]		# -nocommand is ineffective if braces are quoted
assert [subst "{abc,[puts Hello!],def}"] {eq $_ "{abc,,def}"} -v -line #__LINE__
# % puts [subst -nocommands "{abc,[puts Hello!],def}"]	# Activestate (quoted)
# Hello!
# {abc,,def}
# % puts [subst -nocommands {abc,[puts Hello!],def}]	# ActiveState (unquoted)
# abc,[puts Hello!],def
assert [subst {abc,[puts Hello!],def}] {eq $_ "abc,[puts Hello!],def"} -v -line #__LINE__

unset a i "my var" -nocomplain

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #