###################################################################################
# File: 			test_unset.tcl
# Purpose:			Test file for [unset]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 21/Aug/2017	First test version. Mods added for -glob deletion
# 1.01 26/Jun/2018	Added tests for addition of -nocase to -glob
# 1.01 22/Jan/2022	Updated for [struct unset] handling multiple structs
###################################################################################
# Notes: 
# Syntax: 	unset var ?var ... ? ?-nocomplain?
#			unset mask ?mask ...? -glob ?-nocomplain?
# Unset can only delete individual array elements
# Note that enum values must be unset using the -const argument
###################################################################################
/*

TODO
-------------------------------------------------------
ADD Test for memory leaks
DONE FIX unset is corrupting the vartable (poss 'prev')
ADD test scope resolution for std and array vars
DONE Test standard unset
DONE Test array element unset
DONE Test struct unset
DONE Test unset -glob for std vars
ADD struct glob unset
ADD and Test unset -glob for array vars
DONE Test enum unset
*/

puts "Line: "#__LINE__
set a 1
set b 2
set c 3
assert [is_set a] {== $_ 1} -v -line #__LINE__
assert [is_set b] {== $_ 1} -v -line #__LINE__
assert [is_set c] {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
unset a 
unset b
unset c
assert [is_set a] {== $_ 0} -v -line #__LINE__
assert [is_set b] {== $_ 0} -v -line #__LINE__
assert [is_set c] {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
set a 1
set b 2
set c 3
puts "Line: "#__LINE__
unset a b c
assert [is_set a] {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
set a 1
set b 2
set c 3
unset * -nocomplain
assert [is_set a] {== $_ 1} -v -line #__LINE__
assert [is_set b] {== $_ 1} -v -line #__LINE__
assert [is_set c] {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
unset ? -glob
assert [is_set a] {== $_ 0} -v -line #__LINE__
assert [is_set b] {== $_ 0} -v -line #__LINE__
assert [is_set c] {== $_ 0} -v -line #__LINE__
newline

puts "Line: "#__LINE__
set apple 1
set avocado 2
set apricot 3
assert [is_set apple] {== $_ 1} -v -line #__LINE__
assert [is_set avocado] {== $_ 1} -v -line #__LINE__
assert [is_set apricot] {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
unset a* -glob -nocomplain
assert [is_set apple] {== $_ 0} -v -line #__LINE__
assert [is_set avocado] {== $_ 0} -v -line #__LINE__
assert [is_set apricot] {== $_ 0} -v -line #__LINE__
newline

puts "Line: "#__LINE__
enum {
	RANDOM_STYLE_MIXED
	RANDOM_STYLE_LOWER
	RANDOM_STYLE_UPPER
	RANDOM_STYLE_NUMBERS
	RANDOM_STYLE_ALL
	RANDOM_STYLE_BINSTR
}

puts "Line: "#__LINE__
unset RANDOM_STYLE_M* -glob -const -nocomplain
assert [is_set RANDOM_STYLE_MIXED] {== $_ 0} -v -line #__LINE__
assert [is_set RANDOM_STYLE_LOWER] {== $_ 1} -v -line #__LINE__
newline

puts "Line: "#__LINE__
unset RANDOM_STYLE_L* RANDOM_STYLE_U* -glob -const
assert [is_set RANDOM_STYLE_LOWER] {== $_ 0} -v -line #__LINE__
assert [is_set RANDOM_STYLE_UPPER] {== $_ 0} -v -line #__LINE__
assert [is_set RANDOM_STYLE_NUMBERS] {== $_ 1} -v -line #__LINE__
assert [is_set RANDOM_STYLE_ALL] {== $_ 1} -v -line #__LINE__
assert [is_set RANDOM_STYLE_BINSTR] {== $_ 1} -v -line #__LINE__
newline

puts "Line: "#__LINE__
set count [unset RANDOM_STYLE_A* RANDOM_STYLE_N* -glob -const]
assert [is_set RANDOM_STYLE_NUMBERS] {== $_ 0} -v -line #__LINE__
assert [is_set RANDOM_STYLE_ALL] {== $_ 0} -v -line #__LINE__
assert [is_set RANDOM_STYLE_BINSTR] {== $_ 1} -v -line #__LINE__
puts "Count was $count"
newline

puts "Line: "#__LINE__
set count [unset RANDOM_STYLE_* -glob -const]
assert [is_set RANDOM_STYLE_BINSTR] {== $_ 0} -v -line #__LINE__
puts "Count was $count"

# No variables are left. This will err if we don't use -nocomplain
unset RANDOM_STYLE_* -glob -nocomplain

unset count -nocomplain

# Array variables ######################################################

set a(0) Hello
set a(1) There

# FIX unset <array> won't accept >1 array arg
array walk a
unset a(0) a(1)
assert [is_set a(0)] {== $_ 0} -v -line #__LINE__
array walk a

#unset a* -glob
catch {
	assert [is_set a(1)] {== $_ 0} -v -line #__LINE__
}
array walk a
unset a(1)
assert [is_set a(1)] {== $_ 0} -v -line #__LINE__

array unset a


# Struct ##############################################################

puts "Line: "#__LINE__
struct s {a 10 b 20 c 30}
assert [is_set s] {== $_ 1} -v -line #__LINE__
vars s*
unset s
assert [is_set s] {== $_ 0} -v -line #__LINE__
newline

# Using [unset]
puts "Line: "#__LINE__
struct s {a 10 b 20 c 30}
struct t {a 10 b 20 c 30}
struct u {a 10 b 20 c 30}
unset s t u
assert [is_set s] {== $_ 0} -v -line #__LINE__
assert [is_set t] {== $_ 0} -v -line #__LINE__
assert [is_set u] {== $_ 0} -v -line #__LINE__
puts "Line: "#__LINE__
vars
newline

# Using [struct unset]
puts "Line: "#__LINE__
struct s {a 10 b 20 c 30}
struct t {a 10 b 20 c 30}
struct u {a 10 b 20 c 30}
struct unset s t u
assert [is_set s] {== $_ 0} -v -line #__LINE__
assert [is_set t] {== $_ 0} -v -line #__LINE__
assert [is_set u] {== $_ 0} -v -line #__LINE__
puts "Line: "#__LINE__
vars
newline


/*
# Not implemented yet
struct s {a 10 b 20 c 30}
struct t {a 10 b 20 c 30}
struct u {a 10 b 20 c 30}
vars
unset * -glob
vars
assert [is_set s] {== $_ 0} -v -line #__LINE__
assert [is_set t] {== $_ 0} -v -line #__LINE__
assert [is_set u] {== $_ 0} -v -line #__LINE__
*/

undef foo -nocomplain
unset foo -nocomplain -const

set foo 1
set Foo 1
set FOO 1

unset f*o -glob -nocase -nocomplain

assert [is_set foo] {== $_ 0} -v -line #__LINE__
assert [is_set Foo] {== $_ 0} -v -line #__LINE__
assert [is_set FOO] {== $_ 0} -v -line #__LINE__

vars
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* \[unset\] Done *"
textcolor

# EOF #